****************************************************************
c     subroutine library for address transformation from
c     1D to nD array (1 <= n <= 6)   (n=idm in the program)
c     hmax(1)-hmax(6) array size for 1 to 6 axes
c     nt=hmax(1)*hmax(2)..hmax(6)  : total 1D size
c     mm : data necessary for address transformation
c     1D array 1-nt
c     6D array 1-hmax(1), 1-hmax(2),.., 1-hmax(6)
c     for hkl indices within the Brillouin zone
c     tounitcell transforms hkl with negative indices to positive
c     6D array defined above
c     its inverse transformation is tobrillzone
c     getitohkl transform 1D index i to hkl in the Brillouin zone
c     getitoxyz transfrom 1D index i to xyz with positive indices
c     their inverse transformations are gethkltoj and getxyztoj

c     in order to use these transformation, call calmm first
c     with giving hmax and idm
c     array size of hmax and mm should be 6 and 7
*****************************************************************

      subroutine calmm(hmax,idm) ! calculate mm
      integer hmax(*)
      common /cmm/ mm(7)
      call setidm(idm)
      call sethmax(hmax)
      mm(1)=1
      nt=1
      do j=1,idm
         mm(j+1)=mm(j)*hmax(j)
         nt=nt*hmax(j)
      end do
      call setmm(mm)
      return
      end

      subroutine setindfindx()
c     set independent structure factors 1D indices into hkl0
c     this is necessary for retoring old density
      include 'abcdri.inc'
      integer getnt,gethkltoj
      integer h(6),hr(6)
      call getnsymo(nsymo)
      call geticent(icent)
      nt=getnt()
      ntf0=0
      call zerovl(isold,nt)     ! logical values (usually 1 bite array)
      do i=0,nt-1               ! number of points in the unit cell
         call getitohkl(i,h)    ! h : hkl index in Brillowin zone
         if(.not.isold(i)) then
            do ns=1,nsymo
               do ic=0,icent
                  call gethr(h,hr,ns,ic)
                  j=gethkltoj(hr)
                  if(ns.eq.1.and.ic.eq.0) then
                     if(i.ne.j) write(6,*) 'i,j=',i,j
                     hkl0(ntf0)=j
                     ntf0=ntf0+1
                  else
                     if(j.gt.nt-1) write(6,*) 'j=',j
                     if(j.lt.0) write(6,*) 'j=',j
                     isold(j)=.true.
                  end if
               end do
            end do
         end if
      end do
c      write(6,*) 'ntf0=',ntf0
c      do i=0,ntf0-1
c         j=hkl0(i)
c         call getitohkl(j,h)
c         call wtvctrif('hkl0=',h,6,'(6i4)')
c      end do
c      stop
      return
      end
      
      subroutine setindxindx()
c     set independent grid points 1D indices into ix0
c     this is necessary for retoring old density
      include 'abcdri.inc'
      integer getnt,getxyztoj
      integer x(6),xr(6)        ! grid points
      call getnsymo(nsymo)
      call geticent(icent)
      nt=getnt()
      ntx0=0
      call zerovl(isold,nt)     ! logical values (usually 1 bite array)
      do i=0,nt-1               ! number of points in the unit cell
         call getitoxyz(i,x)    ! x : grid points in unit cell
         if(.not.isold(i)) then
            do ns=1,nsymo
               do ic=0,icent
                  call getrxi(x,xr,ns,ic)
                  j=getxyztoj(xr)
                  if(ns.eq.1.and.ic.eq.0) then
c                     if(i.ne.j) write(6,*) 'i,j=',i,j
                     ix0(ntx0)=j ! from 0 to ntx0-1
                     ntx0=ntx0+1
                  else
c                     if(j.gt.nt-1) write(6,*) 'j=',j
c                     if(j.lt.0) write(6,*) 'j=',j
                     isold(j)=.true.
                  end if
               end do
            end do
         end if
      end do
c     write(6,*) 'ntx0=',ntx0
      return
      end

      integer function getntx0()
c     set independent grid points 1D indices into ix0
c     this is necessary for retoring old density
      include 'abcdri.inc'
      getntx0=ntx0
      return
      end

      integer function getntf0()
c     set independent grid points 1D indices into ix0
c     this is necessary for retoring old density
      include 'abcdri.inc'
      getntf0=ntf0
      return
      end

c      subroutine savef0()       ! save independent part of br,bi in br0 and bi0
cc     this is necessary for restoring old density
cc     call it before calculating new br,bi
c      include 'abcdri.inc'
c      integer h(6),hr(6)
c      do i=1,ntf0               ! number of independent points
c         j=hkl0(i)              ! index for independent points
c         br0(i)=br(j)
c         bi0(i)=br(j)
c      end do
c      return
c      end

c      subroutine savexr0()      ! set independent part of br in xr0
cc     this is necessary for restoring old density
c      include 'abcdri.inc'
c      integer h(6),hr(6)
c      do i=1,ntx0                ! number of independent points
c         j=ix0(i)              ! index for independent points
c         xr0(i)=br(j)
c      end do
c      return
c      end

c     in order to restore old electron density
c
      subroutine setffromf0(f000)
c     set equivalent structure factor (br,bi) from independent ones
c     this is necessary for restoring old density
      real f000                 ! structur factor at the origin
      include 'abcdri.inc'
      integer getnt,getnind,gethkltoj
      integer h(6),hr(6)
      complex b                 ! calculated structure factor for
                                ! equivalent reflection
      pi2=2*getpi()
c     write(6,*) 'pi2 in setffromf0',pi2

c     initialize br,bi
      nt=getnt()
      call zerovr(br,nt)        ! zero vector
      call zerovr(bi,nt)        ! zero vector
      call getnsymo(nsymo)
      call geticent(icent)
c      write(6,*) 'nsymo,icent:',nsymo,icent
c      stop

      br(0)=f000
      nind=getnind()
      do i=1,nind               ! number of observed reflections
         j=hklo1d(i)            ! index for observed reflections in primitive cell
c        write(6,*) 'j=',j
         call getitohkl(j,h)    ! h : observed independent reflection indices
c        call wtvctrif('h=',h,6,'(6i4)')
         do ns=1,nsymo
            do ic=0,icent
               call gethr(h,hr,ns,ic) ! symmetry operation for primitive cell
               k=gethkltoj(hr)  ! 1D index for hr
               yt=getyt(h,ns,ic)
               b=cmplx(ar0(i),ai0(i))*cexp(pi2*yt*(0.,-1.))
               br(k)=real(b)
               bi(k)=aimag(b)
c              call wthklbrbi(hr,br(k),bi(k))  ! for test
            end do
         end do
      end do
      return
      end

      subroutine wthklbrbi(hr,br,bi)
      integer hr(6)
      real br,bi
      write(6,'(a,6i4,a,2g10.2)') 'hr=',hr,' br,bi=',br,bi
      return
      end

      subroutine setbrfromxr0()
c     set equivalent grid points (br) from independent ones
c     this is necessary for getting new density
      include 'abcdri.inc'
      integer getnt,getind,getxyztoj
      integer x(6),xr(6)
      complex fctmp,b
      pi2=2*getpi()
      call getnsymo(nsymo)
      call geticent(icent)
c     initialize br
      nt=getnt()
      call zerovr(br,nt)        ! zero vector
      call zerovr(bi,nt)        ! zero vector
      do i=1,ntx0               ! number of independent points
         j=ix0(i)               ! index for independent points
         call getitoxyz(j,x)    ! x : grid points 
         do ns=1,nsymo
            do ic=0,icent
               call getrxi(x,xr,ns,ic) ! xr : equivalent grid points
               k=getxyztoj(xr)
               br(k)=real(xr0(i))
            end do
         end do
      end do
      return
      end

      subroutine sethmax(hm)
      integer hm(*)
      integer hmax(6)
      common /chmax/ hmax
      call vcpyi(hm,hmax,6)
      return
      end

      subroutine gethmax(hm)
      integer hm(*)
      integer hmax(6)
      common /chmax/ hmax
      call vcpyi(hmax,hm,6)
      return
      end

      subroutine getmm(mmd)
      integer mmd(*)
      common /cmm/ mm(7)
      call vcpyi(mm,mmd,7)
      return
      end

      subroutine setmm(mmd)
      integer mmd(*)
      common /cmm/ mm(7)
      call vcpyi(mmd,mm,7)
c     write(6,*) 'set mm=',(mm(j),j=1,7)
      return
      end

      integer function getnt()
c      common /cidm/ idm
      common /cdim/ idm,nde,ied
      common /cmm/ mm(7)
      getnt=mm(idm+1)
      return
      end

      subroutine getitoxyz(i,xyz) ! get xyz in unitcell
      integer xyz(*)
      integer hmax(6)
      common /chmax/ hmax
      common /cmm/ mm(7)
c      common /cidm/ idm
      common /cdim/ idm,nde,ied
c     i1=i-1
      i1=i
      do k=1,idm
         xyz(k)=mod(i1,mm(k+1))/mm(k)
      end do
      do k=idm+1,6
         xyz(k)=0
      end do
c     do j=1,idm
c     xyz(j)=xyz(j)+1
c     end do
      return
      end

      subroutine getitohkl(i,hkl) ! get hkl in Brillouin zone
c     i runs from 0 to nt-1
      integer hkl(*)
      integer hmax(6)
      common /chmax/ hmax
      common /cmm/ mm(7)
c      common /cidm/ idm
      common /cdim/ idm,nde,ied
      call getitoxyz(i,hkl)     ! hkl in unitcell
      call tobrillzone(hkl)     ! hkl in Brilloin Zone
c     if(i.eq.1) then
c     write(6,*) 'i,hkl',i,(hkl(j),j=1,idm)
c     end if
      return
      end

      subroutine tounitcell(hkl) ! offset=0
      integer hkl(*)
      integer hmax(6)
      common /chmax/ hmax
      common /cmm/ mm(7)
      common /cdim/ idm,nde,ied
c     write(6,*) 'hmax=',hmax,' idm=',idm
      do i=1,idm
         hkl(i)=mod(hkl(i),hmax(i)) ! -hmax < hkl < hmax
         if(hkl(i).lt.0) then
            hkl(i)=hkl(i)+hmax(i) ! 0 <= hkl < hmax
         end if
      end do
      return
      end

      subroutine tobrillzone(hkl) ! offset=0
      integer hkl(*)
      integer hmax(6)
      common /chmax/ hmax
      common /cmm/ mm(7)
c     common /cidm/ idm
      common /cdim/ idm,nde,ied
      do i=1,idm
         if(hkl(i).gt.hmax(i)/2) then
            hkl(i)=hkl(i)-hmax(i)
         end if
      end do
      return
      end

      subroutine getinvhkl(hkl) ! get inverse hkl (hkl in Brillouin zone)
      integer hkl(*)
      integer hmax(6)
      common /chmax/ hmax
      common /cmm/ mm(7)
c      common /cidm/ idm
      common /cdim/ idm,nde,ied
      call takeinv(hkl)
      return
      end
      
      subroutine takeinv(hkl)
      integer hkl(*)
      integer hmax(6)
      common /chmax/ hmax
      common /cmm/ mm(7)
c      common /cidm/ idm
      common /cdim/ idm,nde,ied
      do i=1,idm
         hkl(i)=-hkl(i)
      end do
      do i=idm+1,6
         hkl(i)=0
      end do
      return
      end

      integer function getinvhkltoj(hkl) ! hkl in Brillouin zone
      integer hkl(*)
      integer hmax(6)
      common /chmax/ hmax
      common /cmm/ mm(7)
      common /cdim/ idm,nde,ied
      integer hkld(6)
      integer gethkltoj
c     write(6,*) 'hkl=',(hkl(j),j=1,6)
      call vcpyi(hkl,hkld,6)
      call getinvhkl(hkld)
      getinvhkltoj=gethkltoj(hkld)
      return
      end

      integer function gethkltoj(hkl)
c     hkl may be negative
      integer hkl(*)
      integer hmax(6)
      common /chmax/ hmax
      common /cmm/ mm(7)
      common /cdim/ idm,nde,ied
      integer getxyztoj
      gethkltoj=getxyztoj(hkl)
      return
      end

      integer function getxyztoj(hkl)
      integer hkl(*)
      integer hmax(6)
      common /chmax/ hmax
      common /cmm/ mm(7)
      common /cdim/ idm,nde,ied
      integer h(6)
      call vcpyi(hkl,h,6)
      call tounitcell(h)
c     h : hkl in unitcell offset=0
      getxyztoj=h(1)
      do j=2,idm
         getxyztoj=getxyztoj+h(j)*mm(j) ! zero offset
      end do
      return
      end

      subroutine setng(ng)
      integer ng(*)
      call sethmax(ng)
      return
      end

      subroutine getng(ng)  ! for lodem
      integer ng(*)
      call gethmax(ng)
      return
      end

      function indx(h)          ! offset = 0
      integer h(*)
      integer getxyztoj
      indx=getxyztoj(h)         ! hkl in unitcell offset=0
      return
      end

      subroutine getindx(i,hkl)   ! offset = 0
      integer hkl(*)
      integer mm(7)
      call getidm(idm)
      call getmm(mm)
      i1=i
      do k=1,idm
         hkl(k)=mod(i1,mm(k+1))/mm(k)
      end do
      return
      end

      function iskip(hkln)
      integer hkln(*)
      integer hmax(6)
      integer hprj(6,6)
      common /chprj/ iprj, hprj
      integer vinpi
      call gethmax(hmax)          ! for lodem
      call getidm(idm)
      do i=1,idm
         if(abs(hkln(i)).gt.(hmax(i)-1)/2) then
            iskip=1
c           call wtvctrif('hkln=',hkln,6,'(6i3)')
            return
         end if
      end do
      iskip=0
      return
      end
