c     
c     fourier/patterson map of quasi-crystals
c     
c     five/six-dimensional description
c     input data
c     (1) file1 
c         file1 : file name for hklm_fo_fc file
c     (2) file2   ! dummy for compativility with qcmem
c         file2 : filename of a dummy file (.dxyz file) 
c     (3) title
c     (4) isys,a,c
c         isys: 1:icosahedral 2:icosahedral (cubic)
c                        3:decagonal 4:octagonal
c         a: lattice parameter (in angstrom)  (0.8944 a icosa)
c         c: lattice parameter (in angstrom)  (6/sqrt(5) a icosa)
c     (5) nsymo,icent,brv
c         nsymo: number of generators in symmetry operations
c         icent: give 1 for centrosymmetryc
c         give 0 otherwise
c         brv  : 'p','i' or 'f'
c     (6) symmetry operator in the format of I.T.
c         for example x,y,z,t,u,v; x+1/2,y,z+1/2,t,u,v etc.
c                    ******** repeat (6) nsymo times
c     (7) iplot
c         iplot=1:  patterson map
c              =-1:  Fc patterson map
c              =2:  fourier map
c              =-2: Fc fourier map
c              =3:  difference fourier map
c         icont : 0 calculate xy map, 1 read xy map from file2
c     (8)  f000,sg00,sg0,sg1      ( dummy : for compatibility with qcmem )
c            f000: fc at the origin
c            sg00: sigma for f000  (sg0 : sigma=sqrt(f000)*sg00 is used)
c            sg0: sigma=sg0     (sg0 : sigma=sqrt(fo)*sg0 is used)
c            sg1: sigma=sg1   for strongest (Fcal) reflections
c     (9) qimax, frmt
c         qimax: maximum q vector to be used to draw map
c         frmt: format for hkl, fo, fc data file for icent=1
c               format for hkl, fo, fcr, fci file for icent=0

c     (10) ldev
c         ldev : 1: black and white 2: color

c     (11) ng,icont
c         ng   : number of graphs in a sheet

c     (12) icase, iproj, file3
c          icase=0: plot q1-q2 plane  (use only h1h2h3h40 reflections)
c                1: plot q1-q2 plane  (external space)
c                2: plot q1-q3 plane  (external space)
c                3: plot q2-q3 plane  (external space)
c                4: plot q1-q4 plane
c                5: plot q2-q5 plane
c                6: plot q3-q4 plane (dihedral) or q3-q6 (icosahedral)
c                7: plot q4-q5 plane  (internal space)
c                8: plot q5-q6 plane  (internal space)
c                9: plot ellipsoid in the internal space (icasehedral only)
c               10: plot electron density
c                  along specified direction in q4-q5 space
c               11: plot electron density
c                  along specified direction in q5-q6 space
c               12: plot electron density
c                  along specified direction in q4-q6 space
c                  if icase=>10 then direction cosine (cos, sin)
c           iproj  : 0 for default direction 1: specified direction 

c            for a 3-fold axis cut set isys=2 icase=-4 and
c                 s1=s2 , s3=s4=0
c                 s1=1/sqrt(3),1/sqrt(3),1/sqrt(3),1/sqrt(2),-1/sqrt(2),0,
c                    ,1/sqrt(6),1/sqrt(6),-2/sqrt(6)
c            for a 2-fold axis cut set isys=2 icase=4, 5, or 6
c          file3 : file name of a .xy file
c     ************  if icase < 0 ***********
c     (13)  s1, s2, s3, s4 : orthogonal transformation matrix for q1-q6
c         ((s1(i,j),j=1,3),i=1,3) etc.
c         s1, s2 etc. should be in different lines
c         q(i)'  = sum j s1(i,j) q(j) + sum j s3(i,j) q(j+3) (0 < i, j <=3)
c         q(i+3)'= sum j s4(i,j) q(j) + sum j s2(i,j) q(j+3) (0 < i, j <=3)
c         the map is drawn based on the new axis qi'
c         the plane is selected by abs(icase). for example, 
c         icase= -7 draw the map in the q4'-q5' plane
c     *************************************

c     (14) x       :coordinates of origin
c     (15) xe, u   :coordinates of origin and displacement
c     (16) xi, v   :coordinates of origin and displacement
c     (17) q       :origin shift in external and internal space
c     (18) amagx,amagy,shftxb,shftyb,shftxa,shftya
c         amagx:  magnification for x
c         amagy:  magnification for y
c         shftxb: shift of origin along x before plot in cm
c         shftyb: shift of origin along y before plot in cm
c         shftxa: shift of origin along x after each plot in cm
c         shftya: shift of origin along y after each plot in cm
c     (19) xmin,xmax,ymin,ymax,hscl,xdiv,ydiv  or
c         ri1,ri2, hscl, dtheta, dphi (icase=9 only)
c     
c         xmin:  minimum coordinate along x-axis
c         xmax:  maximum coordinate along x-axis
c         ymin:  minimum coordinate along y-axis
c         ymax:  maximum coordinate along y-axis
c         hscl: hight scale
c         xdiv: length of a mesh edge ( in angstrome)
c         ydiv: length of a mesh edge ( in angstrome)
c         ri1,ri2: radii for the first (x) and second (y) axes
c         hscl: hight scale
c         dtheta: interval of theta in polar coordinate system
c         dphi  : interval of phi in polar coordinate system
c     (20) hmin,div,lf
c             ****if for icase=7,8*******
c     (21) px,py,pz  : direction of the projection
c             **** repeat (12)-(21) ng times
      program qcfour
      integer hmax
      parameter (hmax=10000)
      parameter (ixmax=502)
      parameter (ixmax1=ixmax+1)
      real b(6),det,pvt(6)
      integer ipvt(6),indx(6,2)
      character*80 file1,file2
      character*40 frmt
      include 'nhmax.inc'
      include 'hklf.inc'
      character brv
      real xmin,xmax,ymin,ymax
      common /cabtxy/ xmin,xmax,ymin,ymax,ri1,ri2
      integer hk(6,hmax),fmpl(hmax)
      real hmin(1),div(1),x0(6),q(6)
      real xy(ixmax1*ixmax1),xz(ixmax1*ixmax1)
      real x(6),qd(6),xe(6),xi(6),xd(6),xf(6),qe(6),qi(6),x1(6)
      integer lf(1)
      character*80 title
      common /ctitle/ title
      integer pcol,sctmdl
      common /cpcol/ pcol
      real shftxb,shftyb,shftxa,shftya
      common /cplt/ icase,iplot,shftxb,shftyb,shftxa,shftya,cosq,sinq
      common /cabxy/ ax,bx,coscx,sincx,irl,amagx,amagy
c      integer r
c      common /sym/ r(6,6,60),t(6,60),nsymo,icent
      common /bgn1/ ilm(3),icorc(3),pxj(100,3),hj(100,3)
      common /cproj/ iproj, px, py, pz, shftz
      real dr,sds,dq
c     common /cgetq/ dr(6,6),sds(6,6),dq(6,6),al,cl,er(6,6)
      common /cgetq/ dr(6,6),dq(6,6),de(6,6),di(6,6),
     *     al,cl,eps1,eps2,eps3,eps4
      common /otrfm/ s1(3,3), s2(3,3), s3(3,3), s4(3,3)
      real er(6,6)
c     
      common /ccol/ icol
      common /cwflag/ iwflag(7)
      integer q10,qm,qc
      common /cdrwed/ q10(6),qm(6),qc(6)

      real am(0:5000),bm(0:5000)
      data ic1/1/,ic2/1/,ic3/0/
      data pi2/6.283184/

      data q10/0,0,0,0,1,0/
      data qm/1,0,0,-1,0,0/
      data qc/1,1,1,1,0,0/
      data pi/3.141592/

      ismth=0
      call setsmt(ismth)
      ismtr=0
      call setsmtr(ismtr)

      icorc(1)=ic1
      icorc(2)=ic2
      icorc(3)=ic3
      pcol=1
      icol=10
c     
      call setinp(5)
      call setout(6)
      sctmdl=1
      call freadc('file1=',1,file1,1)
      write(6,'(1h ,a)') file1
      call freadc('file2=',1,file2,1)
      call freadc('title=',1,title,1)
      write(6,'(1h ,a)') title

      call freadi('isys=',1,isys,1)
      call freadr('a=',1,al,0)
      if(isys.ge.3) then
         call freadr('c=',1,cl,0)
      end if
      if(isys.le.2) then
         write(6,*) 'isys=',isys,' a=',al
      else 
         write(6,*) 'isys=',isys,' a=',al,' c=',cl
      end if

      call setsys(isys)
      call setsmh(isys)
      call setsmx(isys)

      if(isys.le.2) then
         idm=6
         nde=3
      else
         idm=5
         nde=2
      end if
      call setidm(idm)
      eps1=0
      eps2=0
      eps3=0
      eps4=0
      call seteps(eps1,eps2,eps3,eps4)
      call setd()
      call setdei()
      if(isys.le.2) cl=al
      call mtrtns()
      call getdet(det)
      if(isys.le.2) then
         vol=al**3/abs(det)
      else
         vol=al**2*cl/abs(det)
      end if
      call setvol(vol)
c     call mcpy(dr,er)
c     call matinv(er,6,6,b,0,det,ipvt,indx,pvt)
c     sds: deformation matrix for unit vectors in 6d space
c     d,sds sent to getq through common area
c***  
      call rdsymo(na,brv)
      if(icent.eq.0) call quit('only centrosymmetric case supported')
      call setbrv(brv)

c      read(5,104) f000,sg00,sg0,sg1
      call freadr('f000=',1,f000,1)
      call freadr('sg00=',1,sg00,0)
      call freadr('sg0=',1,sg0,0)
      call freadr('sg1=',1,sg1,0)
      write(6,'(a,f8.2,3x,a,f8.4,3x,a,f8.4,3x,a,f8.4)')
     *     'f000=',f000,'sg00',sg00,'sg0',sg0,'sg1',sg1

      call freadi('iplot=',1,iplot,1)

      call freadr('qimax=',1,qimax,1)
      call freadc('frmt=',1,frmt,0)
      write(6,*) 'qimax=',qimax, ' frmt=',frmt

      call freadi('ldev=',1,ldev,1)
      write(6,*) 'ldev=',ldev
c     read reflection indices
      call rdhklq(qimax,frmt,iaster,file1) ! read hkl fo fc phase
c     call settitle(title)  ! set title for writetitle
      call drwmap(ldev)
      call quit(' ')
      stop
      end

      subroutine drwmap(ldev)  ! draw electron density map
      character*80 file3
      integer hmax
      parameter (hmax=10000)
      parameter (ixmax=502)
      parameter (ixmax1=ixmax+1)
      include 'nhmax.inc'
      include 'hklf.inc'
      real xmin,xmax,ymin,ymax
      common /cabtxy/ xmin,xmax,ymin,ymax,ri1,ri2
      integer hk(6,hmax),fmpl(hmax)
      real hmin(1),div(1),x0(6),q(6)
      real xy(ixmax1*ixmax1),xz(ixmax1*ixmax1)
      real x(6),qd(6),xe(6),xi(6),xd(6),xf(6),qe(6),qi(6),x1(6)
      integer lf(1)
      character*80 title
      common /ctitle/ title
      integer pcol,sctmdl
      common /cpcol/ pcol
      real shftxb,shftyb,shftxa,shftya
      common /cplt/ icase,iplot,shftxb,shftyb,shftxa,shftya,cosq,sinq
      common /cabxy/ ax,bx,coscx,sincx,irl,amagx,amagy
c      integer r
c      common /sym/ r(6,6,60),t(6,60),nsymo,icent
      common /bgn1/ ilm(3),icorc(3),pxj(100,3),hj(100,3)
      common /cproj/ iproj, px, py, pz, shftz
      real dr,sds,dq
c     common /cgetq/ dr(6,6),sds(6,6),dq(6,6),al,cl,er(6,6)
      common /cgetq/ dr(6,6),dq(6,6),de(6,6),di(6,6),
     *     al,cl,eps1,eps2,eps3,eps4
      common /otrfm/ s1(3,3), s2(3,3), s3(3,3), s4(3,3)
      common /cdivxy/ xdiv,ydiv
      real er(6,6)
c     
      common /ccol/ icol
      common /cwflag/ iwflag(7)
      integer q10,qm,qc
      common /cdrwed/ q10(6),qm(6),qc(6)

      real am(0:5000),bm(0:5000)
      data ic1/1/,ic2/1/,ic3/0/
      data pi2/6.283184/
      data pi/3.141592/
      call getsys(isys)
      call freadi('ng=',1,ng,1)
      call freadi('icont=',1,icont,0)   ! for qcfrt only
      write(6,*) 'ng,icont=',ng,icont
      if(ng.le.0) call quit(' ')

      call plots(0.0,0.0,ldev)
      do n=1,ng
         call freadi('icase=',1,icase,1)
         call freadi('iproj=',1,iproj,0)
         call freadc('file3=',1,file3,0)
         if(icase.lt.0) then
c     orthogonal transformation matrix s1, s2
            call freadrv('s1=',9,s1,1)
            call freadrv('s2=',9,s2,1)
            call freadrv('s3=',9,s3,1)
            call freadrv('s4=',9,s4,1)
            call trnsp3(s1)
            call trnsp3(s2)
            call trnsp3(s3)
            call trnsp3(s4)
         end if
         if(icase.ge.10) then
            call freadr('cos=',1,cosq,1)
            call freadr('sin=',1,sinq,0)
         end if
c     read(5,*) x1,x2,x3,x4,x5,x6
         call freadrv('x=',6,x0,1)
         call freadrv('xe=',6,xe,1)
         call freadr('u=',1,u0,0)
         call freadrv('xi=',6,xi,1)
         call freadr('v=',1,v0,0)
         call freadrv('q=',6,q,1)
c     read(5,'(5f8.4)') amagx,amagy,,shftxb,shftyb,shftxa,shftya
         call freadr('amagx=',1,amagx,1)
         call freadr('amagy=',1,amagy,0)
         call freadr('shftxb=',1,shftxb,0)
         call freadr('shftyb=',1,shftyb,0)
         call freadr('shftxa=',1,shftxa,0)
         call freadr('shftya=',1,shftya,0)
         if(amagx.eq.0) amagx=1
         if(amagy.eq.0) amagy=1
c     read(5,'(9f8.4)') xmax,ymax,hscl,xdiv,ydiv
         if(abs(icase).le.8) then
            call freadr('xmin=',1,xmin,1)
            call freadr('xmax=',1,xmax,0)
            call freadr('ymin=',1,ymin,0)
            call freadr('ymax=',1,ymax,0)
            call freadr('hscl=',1,hscl,0)
            call freadr('xdiv=',1,xdiv,0)
            call freadr('ydiv=',1,ydiv,0)
         else if (abs(icase).eq.9) then
            call freadr('ri1=',1,ri1,1)
            call freadr('ri2=',1,ri2,0)
            call freadr('hscl=',1,hscl,0)
            call freadr('dtheta=',1,dtheta,0)
            call freadr('dphi=',1,dphi,0)
         else
            call freadr('xmin=',1,xmin,1)
            call freadr('xmax=',1,xmax,0)
            call freadr('ymin=',1,ymin,0)
            call freadr('ymax=',1,ymax,0)
            call freadr('hscl=',1,hscl,0)
            call freadr('xdiv=',1,xdiv,0)
         end if
         if(abs(icase).eq.10) then
            qd(1)=0
            qd(2)=0
            qd(3)=0
            qd(4)=xmin*cosq
            qd(5)=xmin*sinq
            qd(6)=0
         else if(abs(icase).eq.11) then
            qd(1)=0
            qd(2)=0
            qd(3)=0
            qd(5)=xmin*cosq
            qd(6)=xmin*sinq
            qd(4)=0
         else if(abs(icase).eq.12) then
            qd(1)=0
            qd(2)=0
            qd(3)=0
            qd(4)=xmin*cosq
            qd(6)=xmin*sinq
            qd(5)=0
         else
            qd(1)=0
            qd(2)=0
            qd(3)=0
            qd(4)=0
            qd(5)=0
            qd(6)=0
         end if
         call getxe(qd,xf)
         call getxe(q,xd)
c     default value
c     
         write(6,'(1h ,5(a,i3))') 'ldev=',ldev
         write(6,'(1h ,a,2i3)') 'iplot,icase=',iplot,icase
         if(icase.lt.0) then
            write(6,*) 's1='
            do i=1,3
               write(6,'(3f10.5)') (s1(i,j),j=1,3)
            end do
            write(6,*) 's2='
            do i=1,3
               write(6,'(3f10.5)') (s2(i,j),j=1,3)
            end do
            write(6,*) 's3='
            do i=1,3
               write(6,'(3f10.5)') (s3(i,j),j=1,3)
            end do
            write(6,*) 's4='
            do i=1,3
               write(6,'(3f10.5)') (s4(i,j),j=1,3)
            end do
         end if
         if(abs(icase).ge.10) then
            write(6,'(1h ,a,2f10.5)') 'cos,sin=',cosq,sinq
         end if
         write(6,'(1h ,a,6f10.5)') 'x=',x0
         write(6,303) 'xe=', (xe(j),j=1,6),'u=',u0
         write(6,303) 'xi=', (xi(j),j=1,6),'v=',v0
 303     format(1h ,a,6f8.5,2x,a,f8.5)
         write(6,'(1h ,a,6f10.5)') 'q=',q
         if(isys.le.2) then
            call getqe0(xe,qe)
         else if(isys.eq.3.or.isys.eq.4) then
            call tr6to5e(xe,x1)
            call getqe0(x1,qe)
         else if(isys.eq.5) then
            call getqe0(xe,qe)
         end if
c     call getqe0(xe,qe)
         qe(4)=0
         qe(5)=0
         qe(6)=0
         call getxe(qe,xe)
         
         if(isys.le.2) then
            call getqe0(xi,qi)
         else if(isys.eq.3.or.isys.eq.4) then
            call tr6to5e(xi,x1)
            call getqe0(x1,qi)
         else if(isys.eq.5) then
            call getqe0(xi,qi)
         end if
c     call getqe0(xi,qi)
         qi(1)=0
         qi(2)=0
         qi(3)=0
         call getxe(qi,xi)
         do i=1,6
c            x(i)=x0(i)+xe(i)*u0+xi(i)*v0-xd(i)+xf(i)
            x(i)=x0(i)+xe(i)*u0+xi(i)*v0
         end do
         write(6,'(1h ,a,6f10.5)')
     *        'amagx,amagy,shftxb,shftyb,shftxa,shftya=',
     *        amagx,amagy,shftxb,shftyb,shftxa,shftya
         if(abs(icase).le.8) then
            write(6,'(1h ,a,7f10.4)')
     *           'xmin,xmax,ymin,ymax,hscl,xdiv,ydiv',
     *           xmin,xmax,ymin,ymax,hscl,xdiv,ydiv
         else if(abs(icase).eq.9) then
            write(6,'(1h ,a,5f10.4)') 'ri1,ri2,hscl,dtheta,dphi=',
     *           ri1,ri2,hscl,dtheta,dphi
         else
            write(6,'(1h ,a,6f10.4)')
     *           'xmin,xmax,ymin,ymax,hscl,xdiv',
     *           xmin,xmax,ymin,ymax,hscl,xdiv
         end if
c     read(5,113) (hmin(i),div(i),lf(i),i=1,ndiv)
         ndiv=1
         do i=1,ndiv
            call freadr('hmin=',1,hmin(i),1)
            call freadr('div=',1,div(i),0)
            call freadi('lf=',1,lf(i),0)
            write(6,225) hmin(i),div(i),lf(i)
         end do
 225     format(1h ,'hmin=',f12.6,2x,'div=',f12.6,2x,'lf=',i3)
         call setxyt(1.,1.,0.,0)  ! use rectangular coordinate system
c     read(5,*) px,py,pz
         if(iproj.eq.1) then
            call freadr('px=',1,px,1)
            call freadr('py=',1,py,0)
            call freadr('pz=',1,pz,0)
            call freadr('shftz=',1,shftz,0)
            write(6,*) 'direction of projection',px,py,pz
         else
            px=0.1
            py=-0.1
            pz=0.2
            shftz=-100
         end if
c     draw fourier or patterson map
c     
c     
c     hmin(i): minimum hight in i-th range
c     div(i) : increase of hight in i-th range
c     lf(i)  : number of contour lines in i-th range
c     ndiv: number of ranges
c     ixm  : number of lattice points along x-axis
c     iym  : number of lattice points along y-axis
c     am,bm : work array to store the x,y coordinates on a contour line
c     (size=5000)
c     xy: hight of the function at the lattice point (i,j)
c     ixm,iym : size of to dimensional array xy
         ndiv=1
         iw=0
         call setqxqy(icase,iqx,iqy)
         if(abs(icase).le.8) then
            ixm=(xmax-xmin)/xdiv+0.001
            iym=(ymax-ymin)/ydiv+0.001
         else if(abs(icase).eq.9) then
            ixm=180/dtheta+0.001
            iym=180/dphi+0.001
         else
            ixm=(xmax-xmin)/xdiv+0.001
            iym=0
         end if
         write(6,*) 'icase=',icase
         write(6,*) 'xmin,xmax,xdiv',xmin,xmax,xdiv
         write(6,*) 'ymin,ymax,ydiv',ymin,ymax,ydiv
         if(ixm.ge.ixmax) ixm=ixmax-1
         if(iym.ge.ixmax) iym=ixmax-1
         write(6,'(a,2i4)') 'ixm,iym=',ixm,iym
         call setxyt(1.,1.,0.,0)
c         ax=al/ixm
c         ax=al/iym
c        call setxyt(ax,bx,0.,0)
         if(icont.eq.1) then
            call readxy(xy,ixm,iym,file3)
         else
c     calculate xy map
            call mapxy(x,xy,ixm,iym,hscl,dtheta,dphi,iqx,iqy)
            call writexy(xy,ixm,iym,file3)
         end if
         call map1(hmin,div,lf,ndiv,am,bm,5000,iw,xy,ixm,iym,xz)
      end do
c      call plot(0.0,2.0,-3)
c      call writetitle(0.3,1.0)
      call plote('end of plot')
      return
      end

      subroutine setqxqy(icase,iqx,iqy)
      common /cqxqy/ iqxd(0:12),iqyd(0:12)
      common /cabtxy/ xmin,xmax,ymin,ymax,ri1,ri2
      common /cabxy/ ax,bx,coscx,sincx,irl,amagx,amagy
      common /cdivxy/ xdiv,ydiv
      parameter (tau=1.618034)
      data (iqxd(i),i=0,12) /1,1,1,2,1,2,3,4,5,0,4,5,4/
      data (iqyd(i),i=0,12) /2,2,3,3,4,5,6,5,6,0,5,6,6/
      data pi /3.1415926536/
      call getsys(isys)
      if(abs(icase).le.12) then
         iqx=iqxd(abs(icase))
         iqy=iqyd(abs(icase))
      else if(icase.eq.17) then
         iqx=iqxd(7)
         iqy=iqyd(7)
      else if(icase.eq.18) then
         iqx=iqxd(8)
         iqy=iqyd(8)
      end if
      if(icase.eq.9) then
         xmax=ri1*pi
         ymax=ri2*pi
         return
      end if

      if(abs(icase).eq.6) then
         if(isys.le.2) then
            iqx=3
            iqy=6
         else
            iqx=3
            iqy=4
         end if
      end if
      if(isys.le.2) then
         fact=1
      else if(isys.eq.3) then
         fact=2/sqrt(5.)
      else if(isys.eq.4) then
         fact=1
      else if(isys.eq.5) then
         fact=2/sqrt(6.)
      else
         call quit('invalid isys')
      end if
      write(6,*) 'icase,iqx,iqy=',icase,iqx,iqy
      call getac(al,cl)
      if(iqx.le.3) then
         call xscale(xmin,xmax,xdiv,al*fact)
         amagx=amagx
         if(isys.ge.3.and.iqx.eq.3) then
            call xscale(xmin,xmax,xdiv,cl)
            amagx=amagx
         end if
      else
         call xscale(xmin,xmax,xdiv,al*fact)
         amagx=amagx
      end if
      if(iqy.le.3) then
         call yscale(ymin,ymax,ydiv,al*fact)
         amagy=amagy
         if(isys.ge.3.and.iqy.eq.3) then
            call yscale(ymin,ymax,ydiv,cl)
            amagy=amagy
         end if
      else
         call yscale(ymin,ymax,ydiv,al*fact)
         amagy=amagy
      end if
      return
      end

      subroutine xscale(xmin,xmax,xdiv,scl)
      xmin=xmin*scl
      xmax=xmax*scl
      xdiv=xdiv*scl
      return
      end

      subroutine yscale(ymin,ymax,ydiv,scl)
      ymin=ymin*scl
      ymax=ymax*scl
      ydiv=ydiv*scl
      return
      end

      subroutine readxy(xy,ixm,iym,file3)
      real xy(*)
      character*80 file3
      call openfileu(3,file3,'OLD') ! unformated input
      read(3) (xy(i),i=1,(ixm+1)*(iym*1))
      close(3)
      return
      end

      subroutine writexy(xy,ixm,iym,file3)
      real xy(*)
      character*80 file3
      call openfileu(3,file3,'UNKNOWN') ! unformated output
      write(3) (xy(i),i=1,(ixm+1)*(iym*1))
      close(3)
      return
      end

      subroutine rdhklq(qimax,frmt,iaster,flnm1)  ! read hkl fo fc phase
      character(*) frmt, flnm1
      character aster
      integer hmax(6)
      integer h(6),hd(6),h1(6)
      include 'nhmax.inc'
      include 'hklf.inc'
      common /fphs/ phsr(nhmax), phsi(nhmax)
      real q(6)
      data pi /3.141592/

      call  getiplt(iplot)
      call gethmax(hmax)
      call geticent(icent)
      call getsmtr(ismtr)
      call openfile(2,flnm1,'OLD') 
      call getidm(idm)
c     write(6,*) 'idm in rdhklq=',idm
                                !      write(6,*) 'icent=',icent
                                !      write(6,*) 'flnm1=',flnm1
      call getsys(isys)
      call zerovi(h1,6)
      i=0
      nskip=0
 100  continue

      read(2,frmt,end=300) (h1(j),j=1,idm),fo,fc,phs
c      write(6,'(a,6i4,4f10.5)') 'h1,fo,fc,phs,sg=',h1,fo,fc,phs,sg
      if(h1(1).eq.999) go to 300
      call smtrh(h1,h,ismtr)    ! simirarity transformation
                                ! write(6,'(a,6i4)') 'h=',h
c      write(6,'(a,6i4,4f10.5)') 'h1,fo,fc,phs,sg=',h1,fo,fc,phs,sg
      call getqi0(h1,q)
      if(isys.le.2) then
         qi=vabs(q(4),3)
      else
         qi=vabs(q(4),2)
      end if
      call vcpyi(h,hd,6)
      call ctoph(hd)            ! indices in primitive cell
      if(qi.le.qimax) then
         i=i+1
         fobs(i)=fo
         fcal(i)=fc
         th=phs*pi/180          ! phase in radian
         phsr(i)=cos(th)
         phsi(i)=sin(th)
         call vcpyi(h1,hkln(1,i),6)
         sig(i)=sg
      else
         nskip=nskip+1
      end if
      go to 100
 300  continue
      is=i
      write(6,'(a,i5)') 'number of reflections used  =',is
      write(6,'(a,i5)') 'number of skiped reflections=',nskip
      close(2)
      return
      end

      subroutine simtr(hd)
      integer hd(*)
      integer h(6),mtrx(4,4)
c     similarity transformation with the ratio of tau
      data mtrx / 0, 0,-1,-1,
     *     1, 1, 1, 0,
     *     0, 1, 1, 1,
     *     -1,-1, 0, 0/
      call vcpyi(hd,h,6)
      do i=1,4
         hd(i)=0
         do j=1,4
            hd(i)=hd(i)+mtrx(i,j)*h(j)
         end do
      end do
      return
      end

      subroutine read2
      character*80 str
c     skip up to the header Nrefl  h k l etc.
      rewind 2
 100  continue
      read(2,'(a)',end=200) str
      if(str(1:6).eq.' Nrefl') then
         go to 76
      else
         go to 100
      end if
 76   continue
c     skip one line
      read(2,*)
      return
 200  call quit(' Nrefl not found')
      return
      end

      integer function check(ih,ik,h,k)
      integer ih,ik
      real h,k
      integer jh,jk
      jh=h*10
      jk=k*10
      if(h*10-jh.gt.0.5) then
         jh=jh+1
      else if(h*10-jh.lt.-.5) then
         jh=jh-1
      end if
      if(k*10-jk.gt.0.5) then
         jk=jk+1
      else if(k*10-jk.lt.-.5) then
         jk=jk-1
      end if
      if(ih.eq.jh.and.ik.eq.jk) then
         check=1
      else
         check=0
      end if
      return
      end

      subroutine mapxy(x0,dnsty,ixm,iym,hscl,dtheta,dphi,iqx,iqy)
c     calculate 2D electron density map
      include 'nhmax.inc'
      include 'hklf.inc'
      common /fphs/ phsr(nhmax), phsi(nhmax)
      real q(6),dnsty(0:ixm,0:iym)  ! 2D density 
      real x0(*)
      common /cabtxy/ xmin,xmax,ymin,ymax,ri1,ri2
      common /cplt/ icase,iplot,shftxb,shftyb,shftxa,shftya,cosq,sinq
      integer hr(6),h0(6)
      real tr(6)
      character brv
      data pi2/6.28318/
      call getsys(isys)
      call getnsymo(nsymo)
      call geticent(icent)
      write(6,*) 'iqx,iqy=',iqx,iqy

      do ix=0,ixm
         do iy=0,iym
            dnsty(ix,iy)=0
         end do
      end do

      call getac(al,cl)
      call getbrv(brv)
      dx=(xmax-xmin)/ixm
      if(abs(icase).le.9) then
         dy=(ymax-ymin)/iym
      end if
      fact0=hscl/getvol()
      write(6,*) 'hscl,vol=',hscl,getvol()
      if(brv.eq.'f') then
         fact0=fact0*32
      else if(brv.eq.'i') then
         fact0=fact0*2
      end if
      fact4=pi2*dtheta/360
      fact5=pi2*dphi/360
c     write(6,*) 'is=',is
c     write(6,*) 'fact0,nsymo',fact0,nsymo
      do i=1,is
c     multiplicity of the reflection
         fact3=fact0*multpl(hkln(1,i))/(nsymo*2)
c        write(6,*) 'fact3,multpl=',fact3,multpl(hkln(1,i))
         if(iplot.eq.1) then    
c     patterson map
            for=(fobs(i)/10)**2
            foi=0
         else if(iplot.eq.2) then
c     fourier map
            for=fobs(i)*phsr(i)
            foi=fobs(i)*phsi(i)
         else if(iplot.eq.-2) then
c     Fc fourier map
            for=fcal(i)*phsr(i)
            foi=fcal(i)*phsi(i)            
         else if(iplot.eq.3) then
c     difference fourier map
            for=(fobs(i)-fcal(i))*phsr(i)
            foi=(fobs(i)-fcal(i))*phsi(i)
         end if
         if(abs(icase).eq.0.and.hkln(5,i).ne.0) go to 150

         fohr=for*fact3
         fohi=foi*fact3
         call vcpyi(hkln(1,i),h0,6)
         do nsy=1,nsymo
            do ic=0,1
               call gettr(tr,nsy,0)
               call gethr(h0,hr,nsy,ic)
               call getqi0(hr,q)
               if(icase.lt.0) then
                  call ottrfm(q)
               end if
               fs0=-(q(iqx)*xmin+q(iqy)*ymin)
               fs1=-vdinp(hr,x0,6)-vdinp(h0,tr,6)
               if(abs(icase).le.8) then
                  do ix=0,ixm
                     do iy=0,iym
                        fs=fs0+fs1-(q(iqx)*ix*dx+q(iqy)*iy*dy)
                        dnsty(ix,iy)=dnsty(ix,iy)+fohr*cos(pi2*fs)
                        if(icent.eq.0) then
                           dnsty(ix,iy)=dnsty(ix,iy)-fohi*sin(pi2*fs)
                        end if
                     end do
                  end do
               else if(abs(icase).eq.9) then
                  do ix=0,ixm
                     do iy=0,iym
                        fix=fact4*ix
                        fiy=fact5*iy
                        x=ri2*sin(fix)*cos(fiy)
                        y=ri2*sin(fix)*sin(fiy)
                        z=ri1*cos(fix)
                        fs=fs1-(q(4)*x+q(5)*y+q(6)*z)
                        dnsty(ix,iy)=dnsty(ix,iy)+fohr*cos(pi2*fs)
                        if(icent.eq.0) then
                           dnsty(ix,iy)=dnsty(ix,iy)-fohi*sin(pi2*fs)
                        end if
                     end do
                  end do
               else if(abs(icase).ge.10) then
                  do ix=0,ixm
                     do iy=0,iym
                        fs=fs1-(q(iqx)*cosq+q(iqy)*sinq)*ix*dx
                        dnsty(ix,iy)=dnsty(ix,iy)+fohr*cos(pi2*fs)
                        if(icent.eq.0) then
                           dnsty(ix,iy)=dnsty(ix,iy)-fohi*sin(pi2*fs)
                        end if
                     end do
                  end do
               end if
            end do
         end do
 150     continue
      end do
c      do iy=0,iym
c         write(6,'(30i4)') (int(dnsty(i,iy)),i=0,ixm)
c      end do
      return
      end

      subroutine ottrfm(q)
c     orthogonal transformation in external and internal spaces
      real q(6),qd(6)
      common /otrfm/ s1(3,3), s2(3,3), s3(3,3), s4(3,3)
      call getac(a,c)
      do i=1,3
         qd(i)=0
         do j=1,3
            qd(i)=qd(i)+s1(i,j)*q(j)+s3(i,j)*q(j+3)/a
         end do
      end do
      do i=1,3
         qd(i+3)=0
         do j=1,3
            qd(i+3)=qd(i+3)+s4(i,j)*q(j)*a +s2(i,j)*q(j+3)
         end do
      end do
      call vcpyr(qd,q,6)
      return
      end

      subroutine savexc(xc,xold,ndiff)
      real xc(*),xold(3,*)
      xold(1,ndiff)=xc(1)
      xold(2,ndiff)=xc(2)
      xold(3,ndiff)=xc(3)
      return
      end

      subroutine savehr(hr,hold,ndiff)
      integer hr(*),hold(6,*)
      do i=1,6
         hold(i,ndiff)=hr(i)
      end do
      return
      end

      integer function icomp(h,hr)
      integer h(*),hr(*)
      icomp=0
      do i=1,6
         if(h(i).ne.hr(i)) return
      end do
      icomp=1
      return
      end

      subroutine savehc(hc,hold,ndiff)
      real hc(*),hold(6,*)
      do i=1,6
         hold(i,ndiff)=hc(i)
      end do
      return
      end

      subroutine fdmin(i1,i2,i,m1,dmin,a,b)
      dimension a(0:*),b(0:*)
      m1=i1
      do j=i1,i2
         d=(a(j)-a(i))**2+(b(j)-b(i))**2
         if(j.eq.i1) then
            dmin=d
            m1=j
         else if(d.lt.dmin) then
            dmin=d
            m1=j
         end if
      end do
      return
      end

      subroutine intch(x1,y1,x2,y2)
      x=x1
      y=y1
      x1=x2
      y1=y2
      x2=x
      y2=y
      return
      end

      subroutine getiplt(iplotd)
      common /cplt/ icase,iplot,shftxb,shftyb,shftxa,shftya,cosq,sinq
      iplotd=iplot
      return
      end

      subroutine getcas(ic)
      real shftxb,shftyb,shftxa,shftya
      common /cplt/ icase,iplot,shftxb,shftyb,shftxa,shftya,cosq,sinq
      ic=icase
      return
      end

      subroutine gauss(p,n)
      parameter (ndata=1500,is=2,ntotal=(ndata-1)*10+1)
      real p(is,ndata)
      common /spl1/ a(ndata,3),b(is,ndata),u(is,ndata),xl(ndata)

      do j=2,n-1
         a(j,1)=xl(j)
         a(j,2)=2.0*(xl(j)+xl(j-1))
         a(j,3)=xl(j-1)
         do k=1,is
            b(k,j)=(3.0*(xl(j-1)*xl(j-1)*(p(k,j+1)-p(k,j)))
     &           +3.0*xl(j)*xl(j)*(p(k,j)-p(k,j-1)))/(xl(j)*xl(j-1))
         end do
      end do

      do i=2,n
         d=a(i-1,2)/a(i,1)
         a(i,1)=0.0
         a(i,2)=a(i,2)*d-a(i-1,3)
         a(i,3)=a(i,3)*d
         do k=1,is
            b(k,i)=b(k,i)*d-b(k,i-1)
         end do
         q=a(i,2)
         a(i,2)=1.0
         a(i,3)=a(i,3)/q
         do k=1,is
            b(k,i)=b(k,i)/q
         end do
      end do
      do k=1,is
         u(k,n)=b(k,n)
         do j=1,n-1
            u(k,n-j)=b(k,n-j)-a(n-j,3)*u(k,n+1-j)
         end do
      end do
      end

      subroutine getwmij(amij,wmij,ctoh,htoc)
      real amij(3,3),wmij(3,3),ctoh(6,6),htoc(6,6),tmp(6,6)
      do i=1,3
         do j=1,3
            tmp(i,j)=0
            do k=1,3
               tmp(i,j)=tmp(i,j)+amij(i,k)*ctoh(k,j)
            end do
         end do
      end do
      do i=1,3
         do j=1,3
            wmij(i,j)=0
            do k=1,3
               wmij(i,j)=wmij(i,j)+htoc(i,k)*tmp(k,j)
            end do
         end do
      end do
      return
      end

      subroutine getsys(isysd)
      common /cisys/ isys
      isysd=isys
      return
      end            

      subroutine setsys(isysd)
      common /cisys/ isys
      isys=isysd
      return
      end            

c*********************************
      subroutine trcs(x,y,n1,n2,n3)
      dimension x(*),y(*),n1(*),n2(*),n3(*)
      real n1,n2,n3
c*****transformation of coordinate systems********************
c     x: coordinates in primary system
c     y: coordinates in plotter system
c     y(1),y(2) are x and y coordinates in plotter system
      y(1)= (n1(1)*x(1)+n1(2)*x(2)+n1(3)*x(3))
      y(2)= (n2(1)*x(1)+n2(2)*x(2)+n2(3)*x(3))
      y(3)= (n3(1)*x(1)+n3(2)*x(2)+n3(3)*x(3))
      return
      end

      subroutine tr5to6(ei,fi)
      real fi(*),ei(*)
      integer sd(6,6)
      data sd/ 1,0,0,0,-1,0,
     *     0,1,0,0,-1,0,
     *     0,0,1,0,-1,0,
     *     0,0,0,1,-1,0,
     *     1,1,1,1, 1,0,
     *     0,0,0,0, 0,1/
      do j=1,6
         fi(j)=0.
         if(j.le.5) then
            do k=1,4
               fi(j)=fi(j)+sd(j,k)*ei(k)
            end do
         else if(j.eq.6) then
            fi(6)=ei(5)
         end if
      end do
      return
      end

      subroutine tr6to5e(ei,fi)
      real fi(*),ei(*)
      integer sd(6,6)
      data sd/ 4,-1,-1,-1, 1, 0,
     *     -1, 4,-1,-1, 1, 0,
     *     -1,-1, 4,-1, 1, 0,
     *     -1,-1,-1, 4, 1, 0,
     *     -1,-1,-1,-1, 1, 0,
     *     0, 0, 0, 0, 0, 1/
      do j=1,6
         fi(j)=0.
         if(j.le.4) then
            do k=1,5
               fi(j)=fi(j)+sd(j,k)*ei(k)/5
            end do
         else if (j.eq.5) then
            fi(5)=ei(6)
         end if
      end do
      return
      end

      subroutine tr6to5i(ei,fi)
      integer fi(*),ei(*)
      integer sd(6,6)
      data sd/ 1,0,0,0,-1,0,
     *     0,1,0,0,-1,0,
     *     0,0,1,0,-1,0,
     *     0,0,0,1,-1,0,
     *     0,0,0,0, 0,1,
     *     0,0,0,0, 0,0/
      do j=1,6
         fi(j)=0.
         do k=1,5
            fi(j)=fi(j)+sd(j,k)*ei(k)
         end do
      end do
      return
      end

      function ifskip(q)
      real q(*)
      common /cgetq/ dr(6,6),dq(6,6),de(6,6),di(6,6),
     *     al,cl,eps1,eps2,eps3,eps4
      data eps/0.001/
      call getsys(isys)
      ifskip=1
      if(q(3).lt.-eps) return
      if(q(2).lt.-eps) return
      if(isys.eq.1) then
         if(-.3090*q(1)+0.9511*q(2).gt.eps) return
      else if(isys.eq.2) then
         if(-.5878*q(1)+0.8090*q(2).gt.eps) return
      else
         call quit('invalid isys')
      end if
      ifskip=0
      return
      end

      subroutine setvol(vold)
      common /cvol/ vol
      vol=vold
      return
      end

      function getvol()
      common /cvol/ vol
      getvol=vol
      return
      end

      subroutine setsmt(ismth)
      common /csmth/ ismths
      ismths=ismth
      return
      end

      subroutine getsmt(ismth)
      common /csmth/ ismths
      ismth=ismths
      return
      end

      subroutine setsmtr(ismtr)
      common /csmtr/ ismtrd
      ismtrd=ismtr
      return
      end

      subroutine getsmtr(ismtr)
      common /csmtr/ ismtrd
      ismtr=ismtrd
      return
      end

      function iskip(hkln,hmax)
      integer hkln(*),hmax(*)
      call getidm(idm)
c     write(6,*) 'hmax in iskip=',(hmax(j),j=1,6)
      do i=1,idm
         if(abs(hkln(i)).gt.hmax(i)/2) then
            iskip=1
            return
         end if
      end do
      iskip=0
      return
      end

      subroutine pahkl(ar,ai,ao,bo,hr,fact)
      integer hr(*)
      real ar(0:*)
      real ai(0:*)
      integer gethkltoj
      j=gethkltoj(hr)
      ar(j)=ao*fact    ! fo
      ai(j)=bo*fact    ! fo
      return
      end

      subroutine reduc(x)
      real x(*)
      data eps /0.00001/
      call getidm(idm)
      do ix=1,idm
         x(ix)=amod(x(ix),1.0)
         if(x(ix).le.-0.5+eps) then
            x(ix)=x(ix)+1.0
         else if(x(ix).gt.0.5)  then
            x(ix)=x(ix)-1.0
         end if
      end do
      return
      end

