****************************************************************
c     subroutine library for address transformation from
c     1D to nD array (1 <= n <= 6)   (n=idm in the program)
c     hmax(1)-hmax(6) array size for 1 to 6 axes
c     nt=hmax(1)*hmax(2)..hmax(6)  : total 1D size
c     mm : data necessary for address transformation
c     1D array 1-nt
c     6D array 1-hmax(1), 1-hmax(2),.., 1-hmax(6)
c     for hkl indices within the Brillouin zone
c     tounitcell transforms hkl with negative indices to positive
c     6D array defined above
c     its inverse transformation is tobrillzone
c     getitohkl transform 1D index i to hkl in the Brillouin zone
c     getitoxyz transfrom 1D index i to hkl with positive indices
c     their inverse transformations are gethkltoj and getxyztoj

c     in order to use these transformation, call calmm first
c     with giving hmax and idm
c     array size of hmax and mm should be 6 and 7
*****************************************************************
      subroutine calmm(hmax,idm) ! calculate mm
      integer hmax(*)
      common /cmm/ mm(7)
      call setidm(idm)
      call sethmax(hmax)
      mm(1)=1
      nt=1
      do j=1,idm
         mm(j+1)=mm(j)*hmax(j)
         nt=nt*hmax(j)
      end do
      call setmm(mm)
      return
      end

      subroutine sethmax(hm)
      integer hm(*)
      integer hmax(6)
      common /chmax/ hmax
      call vcpyi(hm,hmax,6)
      return
      end

      subroutine gethmax(hm)
      integer hm(*)
      integer hmax(6)
      common /chmax/ hmax
      call vcpyi(hmax,hm,6)
      return
      end

      subroutine getmm(mmd)
      integer mmd(*)
      common /cmm/ mm(7)
      call vcpyi(mm,mmd,7)
      return
      end

      subroutine setmm(mmd)
      integer mmd(*)
      common /cmm/ mm(7)
      call vcpyi(mmd,mm,7)
c     write(6,*) 'set mm=',(mm(j),j=1,7)
      return
      end

      integer function getnt()
c     common /cidm/ idm
      common /cdim/ idm,nde,ied
      common /cmm/ mm(7)
      getnt=mm(idm+1)
      return
      end

      subroutine getitoxyz(i,xyz) ! get xyz in unitcell
      integer xyz(*)
      integer hmax(6)
      common /chmax/ hmax
      common /cmm/ mm(7)
c     common /cidm/ idm
      common /cdim/ idm,nde,ied
c     i1=i-1
      i1=i
      do k=1,idm
         xyz(k)=mod(i1,mm(k+1))/mm(k)
      end do
      do k=idm+1,6
         xyz(k)=0
      end do
c     do j=1,idm
c     xyz(j)=xyz(j)+1
c     end do
      return
      end

      subroutine getitohkl(i,hkl) ! get hkl in Brillouin zone
c     i runs from 0 to nt-1
      integer hkl(*)
      integer hmax(6)
      common /chmax/ hmax
      common /cmm/ mm(7)
c      common /cidm/ idm
      common /cdim/ idm,nde,ied
      call getitoxyz(i,hkl)      
      call tobrillzone(hkl)     ! hkl in Brilloin Zone
c     if(i.eq.1) then
c     write(6,*) 'i,hkl',i,(hkl(j),j=1,idm)
c     end if
      return
      end

      subroutine tounitcell(hkl) ! offset=0
      integer hkl(*)
      integer hmax(6)
      common /chmax/ hmax
      common /cmm/ mm(7)
c     common /cidm/ idm
      common /cdim/ idm,nde,ied
c     write(6,*) 'hmax=',hmax,' idm=',idm
      do i=1,idm
         if(hkl(i).lt.0) then
            hkl(i)=hkl(i)+hmax(i) 
         end if
      end do
      return
      end

      subroutine tobrillzone(hkl) ! offset=0
      integer hkl(*)
      integer hmax(6)
      common /chmax/ hmax
      common /cmm/ mm(7)
c     common /cidm/ idm
      common /cdim/ idm,nde,ied
      do i=1,idm
         if(hkl(i).gt.hmax(i)/2) then
            hkl(i)=hkl(i)-hmax(i)
         end if
      end do
      return
      end

      subroutine getinvhkl(hkl) ! get inverse hkl (hkl in Brillouin zone)
      integer hkl(*)
      integer hmax(6)
      common /chmax/ hmax
      common /cmm/ mm(7)
c      common /cidm/ idm
      common /cdim/ idm,nde,ied
      call takeinv(hkl)
      return
      end
      
      subroutine takeinv(hkl)
      integer hkl(*)
      integer hmax(6)
      common /chmax/ hmax
      common /cmm/ mm(7)
c      common /cidm/ idm
      common /cdim/ idm,nde,ied
      do i=1,idm
         hkl(i)=-hkl(i)
      end do
      do i=idm+1,6
         hkl(i)=0
      end do
      return
      end

      integer function getinvhkltoj(hkl) ! hkl in Brillouin zone
      integer hkl(*)
      integer hmax(6)
      common /chmax/ hmax
      common /cmm/ mm(7)
c      common /cidm/ idm
      common /cdim/ idm,nde,ied
      integer hkld(6)
      integer gethkltoj
c     write(6,*) 'hkl=',(hkl(j),j=1,6)
      call vcpyi(hkl,hkld,6)
      call getinvhkl(hkld)
      getinvhkltoj=gethkltoj(hkld)
      return
      end

      integer function gethkltoj(hkl) ! hkl in Brillouin zone
      integer hkl(*)
      integer hmax(6)
      common /chmax/ hmax
      common /cmm/ mm(7)
c      common /cidm/ idm
      common /cdim/ idm,nde,ied
      integer hkld(6)
      integer getxyztoj
      call vcpyi(hkl,hkld,6)
c     write(6,*) 'hkl=',(hkl(j),j=1,6)
      call tounitcell(hkld)     ! hkld in unit cell for using getxyztoj
c     write(6,*) 'hkld=',hkld
      gethkltoj=getxyztoj(hkld)
      return
      end

      integer function getxyztoj(hkl) ! hkl in unitcell offset=0
      integer hkl(*)
      integer hmax(6)
      common /chmax/ hmax
      common /cmm/ mm(7)
c      common /cidm/ idm
      common /cdim/ idm,nde,ied
      getxyztoj=hkl(1)
      do j=2,idm
         getxyztoj=getxyztoj+hkl(j)*mm(j) ! zero offset
      end do
      return
      end

      subroutine setng(ng)
      integer ng(*)
      call sethmax(ng)
      return
      end

      subroutine getng(ng)
      integer ng(*)
      call gethmax(ng)
      return
      end

      function indx(h)          ! offset = 0
      integer h(*)
      integer getxyztoj
      indx=getxyztoj(h)         ! hkl in unitcell offset=0
      return
      end

      subroutine getindx(i,hkl)   ! offset = 0
      integer hkl(*)
      integer mm(7)
      call getidm(idm)
      call getmm(mm)
      i1=i
      do k=1,idm
         hkl(k)=mod(i1,mm(k+1))/mm(k)
      end do
      return
      end

      function iskip(hkln)
      integer hkln(*)
      integer hmax(6)
      call getng(hmax)          ! for lodem
      call getidm(idm)
      do i=1,idm
         if(abs(hkln(i)).gt.(hmax(i)-1)/2) then
            iskip=1
            return
         end if
      end do
      iskip=0
      return
      end
