c
c     
c
c
c     input data file : cdr
c          (0)frc, ip, nset, ncycle, mode
c             frc    : fraction of positive area in map
c                      (0.3 for quasicrystal
c             ip     : random seed number
c             nset   : number of trials   (nset<=1000)
c             ncycle : maximum cycles for each set
c             mode   : 0 fixed frc  1 : dinamical frc
c          (1)  idm, ng, isys, ismtr
c             idm    : dimension (3,4,5 or 6)
c             ng     : number of divisions along each axis (1,2,..idm)
c             isys   : 0 crystal
c                      isys=1,2 icosahedral quasicrystal
c                      isys=3   decagonal quasicrystal
c                      isys=4   octagonal quasicrystal
c                      isys=5   dodecagonal quasicrystal
c             ismtr  : order of similarity transformations
c              qext => qext*tau**ismtr
c              qint => qint*tau**-ismtr
c              xext => xext*tau**-ismtr
c              xint => xint*tau**ismtr
c           
c          (2)  nsymo, icent, ige, brv
c             nsymo : number of generating elements
c             icent : 0 for non-centrosymmetric 1 for centrosymmetric
c
c                     the origine is fixed at a fixed point if any
c                     since we use the symmetry operators for generating
c                     initial phases in the calculation.
c                     if we change them, the origin may flip or float.
c                     in order to fix the origin
c                     we fix 6 (in icosahedral or 5 in dihedral cases)
c                     phases of strong reflections which have
c                     diffraction vectors lineraly independent to each other
c             ige   : 0 give all symmetry operators (4)
c                     1 give generators in (4)
c             brv   : 'p' ('f' is just for face centered icosahedral case)
c                     for crystals, brv should be p
c          (3) symmetry operator  (header)
c          (4) generator in the format of IT
c                x,y,z (in 3D) or x,y,z,t,u,v (in 6D) etc.
c                  ****repeat (6) nsymo times****
c            ****** if icent=0 skip (4')
c         (4') imode 
c             imode : 0 for normal centrosymmetric density modification
c                     1 after non-centrosymmetric density modification
c                       centrosymmetric density modification is used.
c                     in this case, symmetry operators given in (4) should be
c                     those for corresponding non-centrosymmetric space
c                     goup which fulfill the same symmetry conditions
c                 ****** imode=1 does not work yet ******
c
c          (5)  flnm, frmt, prefix
c               flnm   : filename of the hkl-f-sigf  file
c                  data in the fill will be  (hkl(j),j=1,idm), f
c               frmt   : format of the hkl-f-sigf file
c               prefix : prefix of filenames for .out and .phs
c
c
      include 'nrmax.inc'
      include 'nsetmx.inc'
      include 'iref.inc'
      real t(nrmax)             ! for qsort
      integer is(nrmax*2),jp(nrmax) ! for qsort
      include 'origin.inc'

c     common /ref0/ fcal0(nrmax),iphas0(nrmax)

      common /tbl/ stbl(0:451),dtor,rtod,id(6)
      common /examin/ pest(2,nsetmx),rfact(nsetmx)
      common /den/ rmmx(2,nsetmx)

      parameter (nhmax=3000000)
      real cr(0:nhmax-1),ci(0:nhmax-1)
      integer ng(6)     ! number of divisions in the unit cell
      include 'ldmsort.inc'

      integer s(6,6)
      real tr(6)
      character chr,brv
      character*80 prefix,flnm,flnm1,flnm2,frmt

      real x0(6,2)

      real foi(nrmax)           ! fo for independent reflections
      real sgi(nrmax)           ! sigma for independent reflections
      integer hkli(6,nrmax)     ! hkl for independent reflections
      common /cfoi/ hkli,foi,sgi

      integer nfree(0:6)
      character cxyz(6)
      data cxyz /'x','y','z','t','u','v'/

      integer getnt
      logical isFreecmp

      tpi=atan(1.0)*8.0
      dtor=tpi/360.0
      rtod=360.0/tpi

      call setinp(5)
      call setout(6)
      call freadr('frc=',1,frc0,1)
      call freadi('ip=',1,ip,0)
      call freadi('nset=',1,nset,0)
      call freadi('ncycle=',1,ncycle,0)
      call freadi('mode=',1,mode,0)
      call freadi('idm=',1,idm,1)
      call freadiv('ng=',idm,ng,0)
      call freadi('isys=',1,isys,0)
      
      call setsys(isys)
      call setsmh(isys)   ! for similarity transformation
      call setsmx(isys)   ! for similarity transformation
      call setncycle(ncycle)

      ismtr=0
      if(isys.ne.0) then
         call freadi('ismtr=',1,ismtr,0)
      end if

      write(6,'(a,e12.6,12i4)') 'frc,nset,ncycle,mode ',
     *     frc0,nset,ncycle,mode
      write(6,'(a,12i4)') 'ip,idm,ng,isys,ismtr ',
     *     ip,idm,(ng(j),j=1,idm),isys,ismtr
      call setmode(mode)
      call setidm(idm)
      call setsmtr(ismtr)
      do i=idm+1,6
         ng(i)=1
      end do

      na=1
      call rdsymo(na,brv)
      call geticent(icent)

      if(icent.eq.1) then
         call freadi('imode=',1,imode,1)
      else
         imode=0
      end if

      call setimode(imode)
      if(imode.eq.1) then
         call quit('imode=1 does not work yet')
         call seticent(0)       ! reset icent at 0 (noncentrosymmetric)
      end if

      if(isys.eq.0) then
         if(brv.ne.'p') then
            call quit('brv should be p for usual crystals')
         end if
      end if

      if(isys.gt.2) then
         if(brv.ne.'p') then
            call quit('brv should be p for dihedral quasicrystals')
         end if
      end if
      call setbrv(brv)
      if((isys.eq.1.or.isys.eq.2).and.brv.eq.'p') then
         if(mod(ismtr,3).ne.0) then
            call quit('ismtr should 3n')
         end if
      end if

      call freadc('flnm=',1,flnm,1)
      write(6,'(a,a)') ' flnm=',flnm(1:istrln(flnm))
      call freadc('frmt=',1,frmt,0)
      write(6,'(a,a)') ' frmt=',frmt(1:istrln(frmt))
      call freadc('prefix=',1,prefix,0)
      il=istrln(prefix)
      flnm1=prefix(1:il)//'.out'
      flnm2=prefix(1:il)//'.phs'
      il1=istrln(flnm1)
      il2=istrln(flnm2)
      write(6,*) 'flnm1=',flnm1(1:il1)
      write(6,*) 'flnm2=',flnm2(1:il2)

      call openfile(30,flnm1,'UNKNOWN')
      call calmm(ng,idm) ! nt: total number of divisions in a cell
      nt=getnt()
      if(nt.gt.nhmax) then
         write(6,*) 'nt=',nt,'nt > nhmax'
         call exit(1)
      end if
      call setng(ng)  ! for indx, unique, denmod, exam
      do i=0,451
        stbl(i)=sin(dtor*i)
      end do

      write(30,*) 'frc',frc0,' random seed',ip      
      write(6,*) 'frc',frc0,' random seed',ip      

      call rdhklf(foi,sgi,hkli,nind,flnm,frmt)
      call initlodem(hkli,foi,sgi,nind,fmax)

c      write(6,*) '****a fixed point exists****'
c      call fixedpnt(xo)         ! calculate fixed point         
c      call wtvctrrf('fixed point=',xo,idm,'(6f10.5)')

      call fixedpnt(xo)
c     write(6,'(a,3f10.5)') 'fixed point', (xo(i),i=1,3)
      call freeprm(nfree,xo)    ! fixed point for substructure
      write(6,'(a,6a2)') 'origin is not fixed along ',
     *     (cxyz(nfree(j)),j=1,nfree(0))
      call setfrprm(nfree)
      write(6,*) 'ifFreecmp=',(isFreecmp(j),j=1,idm)
c     stop

      nrall=nref
      sc2=1.0
      fsh=getfsh(fmax)
      iset=0
      iwr=1

      call openfile(10,flnm2,'UNKNOWN')
      call chsdenmodf(1)  ! choose density modification function for LDEM
      do iset=1,nset
         call wthead(iset,ip,frc0,mode) ! write header
c        assign phases
         call setrphaseset(hkli,nind,ip)
c     set random initial phase set for all reflections
c     with specified symmetry
c     note that iphas is in common /ref/

c        call fixedpnt(xo)      ! calculate fixed point
c        call wtvctrrf('fixed point ',xo,idm,'(6f10.5)')

c     fix origin for fixed point free space group
c     if(index.eq.0) then
         call getorgn(hkli,iphas)
c        end if
         call initpfrac(frc0)
         call denmodloop(iset,frc0,cr,ci,ic2,sc2,fsh,fcc(iset))
c     calculate center of gravity x0 for positive peaks in cr
c        write(6,*) 'before centgrv'
         if(imode.eq.1) then    ! reset origin at center of gravity
            call getcenter(x0,nind)
            call wtvctrrf('x0=',x0(1,1),idm,'(6f10.5)')
            call wtvctrrf('x0=',x0(1,2),idm,'(6f10.5)')
c           call chkphas(x0,nind)
c           call zerovr(ci,nt)
c     get structre factor from fobs and phase
c     from fobs and iphas
            call cft1(cr,ci,ng,idm,1,icon) ! cr: density  (exp(-kx))
            call seticent(1)
            call denmodloop(iset,frc0,cr,ci,ic2,sc2,fsh,fcc(iset))
         end if
c     call wthklf(hkli,nind,iset)
c        write(6,*) 'before wthklf1'
c        call zerovr(x0,6)    ! for test
         call wthklf1(hkli,nind,iset,x0)
c        call wthklf1_old(hkli,nind,iset,x0)
      end do
      close(10)

      do i=1,nset
         phe(i)=pest(1,i)
         wphe(i)=pest(2,i)
         rmax(i)=rmmx(1,i)
         rmin(i)=rmmx(2,i)
         fmrit(i)=-rmin(i)/rmax(i)
         set(i)=i
      end do
      call ldmsortw(nset)
c     do i=1,nset
c     write(30,200) i,fcc(i),(pest(j,i),j=1,2),rmmx(1,i),rmmx(2,i)
c     end do
c     close(30)
      stop
  100 format(6i4,2f10.4,i5)
  200 format(i10,f10.5,4f10.2)
      end

      subroutine wthead(iset,ip,frc,mode)
      character*50 frmt
      frmt = '(a,i4,a,i8,a,f10.4,a,i2)'
      write(30,*) ' '
      write(30,*) '**************************************************'
      write(30,frmt) 'set number',iset,
     *     ' random seed',ip,' frc ',frc,' mode ',mode
      write(30,*) '**************************************************'
      write(6,*) ' '
      write(6,*) '***************************************************'
      write(6,frmt) 'set number',iset,' random seed',ip,
     *     ' frc ',frc,' mode ',mode
      write(6,*) '***************************************************'
      return
      end

c     random number generator
c      function ranp(iran)
c      iran=12869*iran+6925
c      iran=mod(iran,2**15)
c      ranp=float(iran)/2.0**15   ! 0 < ranp < 1
c      return
c      end

      subroutine setimode(imoded)
      common /cimode/ imode
      imode=imoded
      return
      end

      integer function getimode()
      common /cimode/ imode
      getimode=imode
      return
      end

c     separate complex to real and imaginary parts
      subroutine csep(c,a,b,n)
      complex c(0:*)
      real a(0:*),b(0:*)
      do i=0,n-1
         a(i)=real(c(i))
         b(i)=aimag(c(i))
      enddo
      return
      end

c     marge real and imaginary parts to complex
      subroutine cmar(c,a,b,n)
      complex c(0:*)
      real a(0:*),b(0:*)
      do i=0,n-1
         c(i)=cmplx(a(i),b(i))
      enddo
      return
      end

      subroutine getmaxf(iref,ip,nref,irefm,nmax) ! get maximum idm refl.
                                ! for fixing the origin
      integer iref(6,*),ip(*),irefm(6,*)
      call getidm(idm)
      do i=1,nmax
         j1=ip(i)
         call vcpyi(iref(1,j1),irefm(1,i),6)
      end do
      return
      end

      subroutine getmaxf0(iref,ip,nref,irefm,nmax) ! get maximum idm refl.
                                ! for fixing the origin
      integer iref(6,*),ip(*),irefm(6,*)
      call getidm(idm)
      ir=1
      j1=ip(nref)
      call vcpyi(iref(1,j1),irefm(1,1),6)
      do i=2,nref
         j1=ip(nref-i+1)
         indv=0
         do j=1,ir
            if(indvct(iref(1,j1),irefm(1,j)).eq.0) then
               ! two diffraction vectors are parallel
               go to 10
            end if
         end do
         ir=ir+1
         call vcpyi(iref(1,j1),irefm(1,ir),6)
 10      continue
         if(ir.eq.nmax) go to 20
      end do
 20   continue
      return
      end

      function indvct(iref1,iref2)
      integer iref1(*),iref2(*)
      data eps /0.00001/
      call getidm(idm)
      vinp1=ivinp(iref1,iref2,idm)
      vinp2=ivinp(iref1,iref1,idm)
      vinp3=ivinp(iref2,iref2,idm)
      vratio=vinp1/(sqrt(vinp2)*sqrt(vinp3))
c     write(6,*) 'idm,vratio=',idm,vratio
      if(vratio.lt.-1+eps.or.vratio.gt.1-eps) then
         ! two diffraction vectors are parallel
         indvct=0
      else
         indvct=1
      end if
      return
      end

      subroutine getcentgrv(cr,x0)   ! calculate center of gravity
      real cr(0:*)
      real x0(6)

      real x(6),mx(6)
      integer ix(6)
      real mass
      integer ng(6)
      integer getnt,getnref

      call geticent(icent)
      if(icent.eq.1) then
         call zerovr(x0,6)
         return
      end if

      call getidm(idm)
      call getng(ng)
      nt=getnt()
c     write(6,*) 'idm,nt,ng=',idm,nt,(ng(j),j=1,idm)
c     stop

      mass=0
      call zerovr(x0,6)
      do i=0,nt-1
         if(cr(i).gt.0) then
            call getitoxyz(i,ix) ! get xyz in unitcell
c         if(i.lt.100) then
c            write(6,*) 'ix=',(ix(j),j=1,idm)
c         end if
c         if(cr(i).gt.0) then
            call getcoord(ix,ng,x,idm)
            mass=mass+cr(i)
            call vscaler(x,cr(i),mx,idm)
            call vaddr(mx,x0,x0,idm)
         end if
      end do
      scl=1.0/mass
      call vscaler(x0,scl,x0,idm)
      call wtvctrrf('x0=',x0,idm,'(6f10.5)')
      return
      end

      subroutine getcoord(ix,ng,x,idm)
      integer ix(6)             ! integer coordinates
      integer ng(6)             ! number of divitions
      real x(6)                 ! coordinates
c     call wtvctrif('ix=',ix,idm,'(6i4)')
      call zerovr(x,6)
      do i=1,idm
         x(i)=float(ix(i))/ng(i)
      end do
c     call wtvctrrf('x=',x,idm,'(6f10.5)')
      return
      end

      subroutine setsmtr(ismtr)
      common /csmtr/ ismtrd
      ismtrd=ismtr
      return
      end

      subroutine getsmtr(ismtr)
      common /csmtr/ ismtrd
      ismtr=ismtrd
      return
      end

      subroutine setsmh(isys)
      integer s0(6,6),t0(6,6),s1(6,6),t1(6,6),s2(6,6),t2(6,6)
      integer s3(6,6),t3(6,6)
      integer s,t
      common /csmtrh/ s(6,6),t(6,6)
*************icosahedral f or i*************
      data s0/ 1, 1, 1, 1, 1, 1,
     *         1, 1, 1,-1,-1, 1,
     *         1, 1, 1, 1,-1,-1,
     *         1,-1, 1, 1, 1,-1,
     *         1,-1,-1, 1, 1, 1,
     *         1, 1,-1,-1, 1, 1/
      data t0/-1, 1, 1, 1, 1, 1,
     *         1,-1, 1,-1,-1, 1,
     *         1, 1,-1, 1,-1,-1,
     *         1,-1, 1,-1, 1,-1,
     *         1,-1,-1, 1,-1, 1,
     *         1, 1,-1,-1, 1,-1/
***************decagonal*********************
      data s1/ 0, 0,-1,-1, 0, 0,
     *         1, 1, 1, 0, 0, 0,
     *         0, 1, 1, 1, 0, 0,
     *        -1,-1, 0, 0, 0, 0,
     *         0, 0, 0, 0, 1, 0,
     *         0, 0, 0, 0, 0, 1/
      data t1/-1, 0,-1,-1, 0, 0,
     *         1, 0, 1, 0, 0, 0,
     *         0, 1, 0, 1, 0, 0,
     *        -1,-1, 0,-1, 0, 0,
     *         0, 0, 0, 0, 1, 0,
     *         0, 0, 0, 0, 0, 1/
***************octagonal*********************
      data s2/ 1, 1, 0,-1, 0, 0,
     *         1, 1, 1, 0, 0, 0,
     *         0, 1, 1, 1, 0, 0,
     *        -1, 0, 1, 1, 0, 0,
     *         0, 0, 0, 0, 1, 0,
     *         0, 0, 0, 0, 0, 1/
      data t2/-1, 1, 0,-1, 0, 0,
     *         1,-1, 1, 0, 0, 0,
     *         0, 1,-1, 1, 0, 0,
     *        -1, 0, 1,-1, 0, 0,
     *         0, 0, 0, 0, 1, 0,
     *         0, 0, 0, 0, 0, 1/
***************dodecagonal*********************
      data s3/ 1, 0, 0,-1, 0, 0,
     *         1, 1, 0, 0, 0, 0,
     *         0, 1, 1, 1, 0, 0,
     *         0, 0, 1, 1, 0, 0,
     *         0, 0, 0, 0, 1, 0,
     *         0, 0, 0, 0, 0, 1/
      data t3/ 0, 1, -1, 1, 0, 0,
     *         0, 0, 1, -1, 0, 0,
     *         1,-1, 1, 0, 0, 0,
     *        -1, 1,-1, 1, 0, 0,
     *         0, 0, 0, 0, 1, 0,
     *         0, 0, 0, 0, 0, 1/
c      data s3/ 2, 1, 0,-1, 0, 0,
c     *         2, 2, 1, 0, 0, 0,
c     *         0, 1, 2, 2, 0, 0,
c     *        -1, 0, 1, 2, 0, 0,
c     *         0, 0, 0, 0, 1, 0,
c     *         0, 0, 0, 0, 0, 1/
c      data t3/ 2,-1, 0, 1, 0, 0,
c     *        -2, 2,-1, 0, 0, 0,
c     *         0,-1, 2,-2, 0, 0,
c     *         1, 0,-1, 2, 0, 0,
c     *         0, 0, 0, 0, 1, 0,
c     *         0, 0, 0, 0, 0, 1/
      if(isys.le.2) then
         call mcpyi(s0,s)
         call mcpyi(t0,t)
      else if(isys.eq.3) then
         call mcpyi(s1,s)
         call mcpyi(t1,t)
      else if(isys.eq.4) then
         call mcpyi(s2,s)
         call mcpyi(t2,t)
      else if(isys.eq.5) then
         call mcpyi(s3,s)
         call mcpyi(t3,t)
      end if
      return
      end

      subroutine setsmx(isys)
      real s,t
      common /csmtrx/ s(6,6),t(6,6)
      real s0(6,6),t0(6,6),s1(6,6),t1(6,6),s2(6,6),t2(6,6)
      real s3(6,6),t3(6,6)
*************icosahedral f or i*************
      data s0/ 1, 1, 1, 1, 1, 1,
     *         1, 1, 1,-1,-1, 1,
     *         1, 1, 1, 1,-1,-1,
     *         1,-1, 1, 1, 1,-1,
     *         1,-1,-1, 1, 1, 1,
     *         1, 1,-1,-1, 1, 1/
      data t0/-1, 1, 1, 1, 1, 1,
     *         1,-1, 1,-1,-1, 1,
     *         1, 1,-1, 1,-1,-1,
     *         1,-1, 1,-1, 1,-1,
     *         1,-1,-1, 1,-1, 1,
     *         1, 1,-1,-1, 1,-1/
***************decagonal*********************
      data s1/ 0, 0,-1,-1, 0, 0,
     *         1, 1, 1, 0, 0, 0,
     *         0, 1, 1, 1, 0, 0,
     *        -1,-1, 0, 0, 0, 0,
     *         0, 0, 0, 0, 1, 0,
     *         0, 0, 0, 0, 0, 1/
      data t1/-1, 0,-1,-1, 0, 0,
     *         1, 0, 1, 0, 0, 0,
     *         0, 1, 0, 1, 0, 0,
     *        -1,-1, 0,-1, 0, 0,
     *         0, 0, 0, 0, 1, 0,
     *         0, 0, 0, 0, 0, 1/
***************octagonal*********************
      data s2/ 1, 1, 0,-1, 0, 0,
     *         1, 1, 1, 0, 0, 0,
     *         0, 1, 1, 1, 0, 0,
     *        -1, 0, 1, 1, 0, 0,
     *         0, 0, 0, 0, 1, 0,
     *         0, 0, 0, 0, 0, 1/
      data t2/-1, 1, 0,-1, 0, 0,
     *         1,-1, 1, 0, 0, 0,
     *         0, 1,-1, 1, 0, 0,
     *        -1, 0, 1,-1, 0, 0,
     *         0, 0, 0, 0, 1, 0,
     *         0, 0, 0, 0, 0, 1/
***************dodecagonal*********************
      data s3/ 1, 0, 0,-1, 0, 0,
     *         1, 1, 0, 0, 0, 0,
     *         0, 1, 1, 1, 0, 0,
     *         0, 0, 1, 1, 0, 0,
     *         0, 0, 0, 0, 1, 0,
     *         0, 0, 0, 0, 0, 1/
      data t3/ 0, 1, -1, 1, 0, 0,
     *         0, 0, 1, -1, 0, 0,
     *         1,-1, 1, 0, 0, 0,
     *        -1, 1,-1, 1, 0, 0,
     *         0, 0, 0, 0, 1, 0,
     *         0, 0, 0, 0, 0, 1/
c      data s3/ 2, 1, 0,-1, 0, 0,
c     *         2, 2, 1, 0, 0, 0,
c     *         0, 1, 2, 2, 0, 0,
c     *        -1, 0, 1, 2, 0, 0,
c     *         0, 0, 0, 0, 1, 0,
c     *         0, 0, 0, 0, 0, 1/
c      data t3/ 2,-1, 0, 1, 0, 0,
c     *        -2, 2,-1, 0, 0, 0,
c     *         0,-1, 2,-2, 0, 0,
c     *         1, 0,-1, 2, 0, 0,
c     *         0, 0, 0, 0, 1, 0,
c     *         0, 0, 0, 0, 0, 1/
      if(isys.le.2) then
         call mcpy(s0,s)
         call mcpy(t0,t)
      else if(isys.eq.3) then
         call mcpy(s1,s)
         call mcpy(t1,t)
      else if(isys.eq.4) then
         call mcpy(s2,s)
         call mcpy(t2,t)
      else if(isys.eq.5) then
         call mcpy(s3,s)
         call mcpy(t3,t)
      end if
      return
      end

      subroutine smtrh(h,hd,ind)
      integer h(*),hd(*)
      integer s,t,ht(6)
      common /csmtrh/ s(6,6),t(6,6)
      call getsys(isys)
      if(ind.eq.0) then
         call vcpyi(h,hd,6)
         return
      end if
c        similarity transformation for decagonal and icosahedral
      call vcpyi(h,ht,6)
      if(ind.gt.0) then
c        ext ==> tau  int ==> tau**-1
         do n=1,ind
            do i=1,6
               hd(i)=0
               do j=1,6
                  hd(i)=hd(i)+s(i,j)*ht(j)
               end do
            end do
            if(n.ne.ind) call vcpyi(hd,ht,6)
         end do
         if(isys.le.2) then
            do j=1,6
               hd(j)=hd(j)/2**ind
            end do
         end if
         return
      else if(ind.lt.0) then
c        ext ==> tau**-1  int ==> tau
          do n=1,-ind
             do i=1,6
                hd(i)=0
                do j=1,6
                   hd(i)=hd(i)+t(i,j)*ht(j)
                end do
             end do
             if(n.ne.-ind) call vcpyi(hd,ht,6)
          end do
          if(isys.le.2) then
             do j=1,6
                hd(j)=hd(j)/2**abs(ind)
             end do
          end if
          return
      end if
      end

      subroutine smtrx(h,hd,ind)
      real h(*),hd(*)
      real s,t,ht(6)
      common /csmtrx/ s(6,6),t(6,6)
      call getsys(isys)
      if(ind.eq.0) then
         call vcpyr(h,hd,6)
         return
      end if
c        similarity transformation for decagonal and icosahedral
      call vcpyr(h,ht,6)
      if(ind.gt.0) then
c        ext ==> tau  int ==> tau**-1
         do n=1,ind
            do i=1,6
               hd(i)=0
               do j=1,6
                  hd(i)=hd(i)+ht(j)*s(j,i)
               end do
            end do
            if(n.ne.ind) call vcpyr(hd,ht,6)
         end do
         if(isys.le.2) then
            do j=1,6
               hd(j)=hd(j)/2**ind
            end do
         end if
         return
      else if(ind.lt.0) then
c     ext ==> tau**-1  int ==> tau
         do n=1,-ind
            do i=1,6
               hd(i)=0
               do j=1,6
                  hd(i)=hd(i)+ht(j)*t(j,i)
               end do
            end do
            if(n.ne.-ind) call vcpyr(hd,ht,6)
         end do
         if(isys.le.2) then
            do j=1,6
               hd(j)=hd(j)/2**abs(ind)
            end do
         end if
         return
      end if
      end

      subroutine ldmsortw(n)  ! last output for all sets
      include 'nsetmx.inc'
      include 'ldmsort.inc'
      common /examin/ pest(2,nsetmx),rfact(nsetmx)
      integer ip(nsetmx),st(2*nsetmx)
      character*50 frmt
      call qsort(fmrit,st,ip,n)
      frmt='(2i4,f10.5,4f10.2,2f10.5,f10.2)'

      write(6,'(3a)') 'No. set      fcc      phe',
     *     '     wphe      rmax      rmin      fmrit      rfact',
     *     '     f000'
      write(6,'(a)') '*****************************************'
      do j=1,n
         i=ip(j)
         write(6,frmt)  j,set(i),fcc(i),phe(i),wphe(i),
     *     rmax(i),rmin(i),-fmrit(j),rfact(i),f000(i)
      end do
      write(6,'(a)') '*****************************************'

      write(30,'(3a)') 'No. set      fcc      phe',
     *     '     wphe      rmax      rmin      fmrit      rfact',
     *     '     f000'
      write(30,'(a)') '*****************************************'
      do j=1,n
         i=ip(j)
         write(30,frmt) j,set(i),fcc(i),phe(i),wphe(i),
     *     rmax(i),rmin(i),-fmrit(j),rfact(i),f000(i)
      end do
      write(30,*) '*****************************************'
      return
      end


