c
c               unitary structure factor calculations
c        ****************************************************
c             Willson plot does not work for quasicrystals
c             for MEM  use f000 derived by LDEM 
c             phason temperature factor may be efficient
c        ****************************************************
c
c        for LDEM inverse transformation may be efficient
c        in order to reduce trancation error in Fourier maps
c        in this case use imode=1, then Be is used for
c        the temperature factor
c
c     (0) imode
c         imode=0 : calculate unitary structure factor
c              =1 : multiply additional temperature factor
c     (1) file1 file2
c         file1:name for hkl_fo_sg file   : structure factor
c         file2:name for hkl_eo_sg file   : unitary structure factor
c     (2) frmt1,frmt2
c         frmt1 : format for hkl(1)-hkl(idm), fo and sig in input file
c         frmt2 : format for hkl(1)-hkl(idm), fo and sig in output file
c     (3) title
c     (4) isys, idm, nsub
c         isys: 0 crystal
c         isys: 1:icosahedral 2:icosahedral (cubic)
c               3:decagonal 4:octagonal 5: dodecagonal
c         idm : space dimension
c               3 for usual crystals
c               3+n for nD modulated structures
c               5 for dihedral quasicrystals
c               6 for icosahedral quasicrystals
c         nsub : number of subsystem for isys=0
c          if isys>0 or nsub=1 skip (5)
c     (5) zmtrx
c          z-matrix for second (and third subsystem)
c     (6) a,c  for isys > 0   (for quasicrystals, c is necessary for isys >=3)
c         a,b,c,cosa,cosb,cosc for isys=0  (for crystals)
c     (7) k
c         a*,b*,c* components of wave vector
c         ******repeat (7) idm-3 times *********
c         if imode=1
c     (8) be
c         if imode=0 
c     (8) ne,atp,be
c         ne: number of atomic-scattering-factor tables
c         atp: chemical composition of the sample (eg. 0.7 0.13 0.17 etc.)
c         be : overall isotropic temperature factor
c     **** sometimes, negative be is appropriate for quasicrystals
c     **** in order to reduce strong truncation effects in Fourier maps
c         if imode=1 skip (9)
c     (9) atom (a)
c         atom: atom sympol 'H', 'He', etc. in the periodic table
c         which correspond to the chemical composition specified by atp
c          ****** repeat (9) ne times ******
c
c         
      common /cwflag/ iwflag(6)
      common /cpnf1/ atp(10),atp0(10)
      character*80 flnm1,flnm2,title
      character*50 frmt1,frmt2
      integer zmtrx(6*3)
      common /czmtrx/ zmtrx, nsub
      common /kwave/ y(3,3,3),vc(3)
      data zmtrx /1,2,3,4,5,6,1,2,3,4,5,6,1,2,3,4,5,6/
      iwflag(3)=1
      call setinp(5)
      call setout(6)
      sctmdl=1
      call freadi('imode=',1,imode,1)
      call setimode(imode)
      call freadc('file1=',1,flnm1,1)
      call freadc('file2=',1,flnm2,0)
      call freadc('frmt1=',1,frmt1,1)
      call freadc('frmt2=',1,frmt2,0)
      write(6,'(1h ,2a)') 'hkl-fo-sg file:',flnm1
      write(6,'(1h ,2a)') 'hkl-eo-sg file:',flnm2
      call freadc('title=',1,title,1)
      write(6,'(1h ,a)') title

      call freadi('isys=',1,isys,1)
      if(isys.eq.0) then
         call freadi('idm=',1,idm,0)
         call freadi('nsub=',1,nsub,0)
         write(6,*) 'isys=',isys,' idm=',idm,' nsub=',nsub
         if(nsub.gt.1) then
            call freadiv('zmtrx=',6*(nsub-1),zmtrx(7),1)
         end if
      else if(isys.le.2) then
         idm=6
      else
         idm=5
      end if
      call setsys(isys)
      call rdlc()
      do i=1,idm-3
         call freadr('k=',1,y(1,i,1),1)
      end do
      sctmdl=0
      call rdasf()
      call getne(ne)

      call setsys(isys)

      if(isys.ge.1) then
         call setd()
         call setdei()
         if(isys.le.2) cl=al
         call setmtrtns()
         call getdet(det)
         if(isys.le.2) then
            idm=6
         else
            idm=5
         end if
      else
         call setmtrtns()
      end if
      call setidm(idm)

c      if(isys.le.2) then
c         vol=al**3/abs(det)   ! det is a cell volume of the reciprocal lattice
c      else
c         vol=al**2*cl/abs(det)
c      end if
c      call setvol(vol)
c      write(6,*) 'vol=',vol
      call rdhklq(frmt1,flnm1)
      call getuntrf(ne,pdns,frmt2,flnm2)
      stop
      end

      subroutine setimode(imoded)
      common /cimode/ imode
      imode=imoded
      return
      end

      integer function getimode()
      common /cimode/ imode      
      getimode=imode
      return
      end

      subroutine getuntrf(ne,pdns,frmt,flnm)
      character(*) frmt,flnm
      parameter (nhmax=10000)
      real avrfo(20),avrfc(20)
      integer hkln
      common /fhkl/ hkln(6,nhmax),fobs(nhmax),sig(nhmax),is
      real q(6)
      real log
      integer getimode

      imode=getimode()
      call openfile(3,flnm,'UNKNOWN')  ! for text file
      il=istrln(frmt)
      write(6,*) 'enter getuntrf'
c     call initshell1()
      call getidm(idm)
c      write(6,*) 'idm=',idm
c      write(6,*) 'is=',is
      do i=1,is
         call getqi(hkln(1,i),q)
         absqe=vabs(q,3)
         absqi=vabs(q(4),idm-3)
c        write(6,*) 'absqe=',absqe
         if(imode.eq.0) then
            absf=asf0(absqe/2,absqi/2)
c     write(6,*) (hkln(j,i),j=1,6),absf,absqe/2
            fow=fobs(i)/absf
            sow=sig(i)/absf
         else
            dwf=dwfact(absqe/2,absqi/2)
            fow=fobs(i)*dwf
            sow=sig(i)*dwf
         end if
         write(3,frmt(1:il)) (hkln(j,i),j=1,idm),fow,sow
         write(6,frmt(1:il)) (hkln(j,i),j=1,idm),fow,sow
c        write(6,*) (hkln(j,i),j=1,6),fow,sig(i),asf0(absqe/2),absqe/2
c     call shell1(absqe,fobs(i))
      end do
c     call wtshell1()
c     call getavrfo(avrfo)
c     call getavrfc(avrfc,ne,apc,pdns)
c     do i=1,is
c
c      end do
      close(3)
      return
      end

c      subroutine getavrfc(avrf,ne,apc,pdns)
c      real avrf(*),apc(*)
c      vol=getvol()
c      do ish=1,10
c         avrf(ish)=0
c         do i=1,ne
c            q=0.2*(ish-1)+1.
c            avrf(ish)=avrf(ish)+(asf(q/2,i)*apc(i))**2
c         end do
c         avrf(ish)=sqrt(avrf(ish))/(pdns*vol)
c         write(6,*) 'avrfc=',avrf(ish)
c      end do
c      return
c      end

      subroutine getavrfo(avrf)
      real avrf(*)
      real sfshell(20)
      integer nshell(20)
      common /cshell1/ sfshell,nshell
      do ish=1,10
         if(nshell(ish).ne.0) then
            avrf(ish)=sfshell(ish)/nshell(ish)
         else
            avrf(ish)=0
         end if
         avrf(ish)=sqrt(avrf(ish))
         write(6,*) 'avrfo=',avrf(ish)
      end do
      return
      end

      subroutine initshell1()
      real sfshell(20)
      integer nshell(20)
      common /cshell1/ sfshell,nshell
      do ish=1,10
         sfshell(ish)=0
         nshell(ish)=0
      end do
      return
      end

      subroutine wtshell1()
      real sfshell(20)
      integer nshell(20)
      common /cshell1/ sfshell,nshell
      do ish=1,10
         write(6,*) 'sfshell,nshell=',sfshell(ish),nshell(ish)
      end do
      return
      end

      subroutine shell1(absq,fc)
      real sfshell(20)
      integer nshell(20)
      common /cshell1/ sfshell,nshell
      do ish=1,10
         if(absq.gt.0.2*(ish-1).and.absq.le.0.2*ish) then
            sfshell(ish)=sfshell(ish)+fc**2
            nshell(ish)=nshell(ish)+1
         end if
      end do
      return
      end

      subroutine rdhklq(frmt,flnm)
      character*(*) frmt,flnm
      integer h1(6)
      parameter (nhmax=10000)
      integer hkln
      common /fhkl/ hkln(6,nhmax),fobs(nhmax),sig(nhmax),is
      real q(6)
      call openfile(2,flnm,'OLD')  ! for text file
      call getidm(idm)
c     write(6,*) 'idm in rdhklq=',idm
      i=0
 100  continue
      read(2,frmt,end=300) (h1(j),j=1,idm),fo,sg
      if(idm.eq.5) then
         h1(6)=0
      end if
      write(6,frmt) (h1(j),j=1,idm),fo,sg
      if(h1(1).eq.999) go to 300
      i=i+1
      if(i.gt.nhmax) then
         write(6,*) 'nhmax ',nhmax
         write(6,*) 'number of reflections > nhmax'
         call exit(0)
      end if
      call vcpyi(h1,hkln(1,i),6)
      fobs(i)=fo
      sig(i)=sg
      go to 100
 300  continue
      is=i
      write(6,'(a,i5)') 'number of reflections used=',is
      close(2)
      return
      end

      subroutine setac(a,c)
      common /cgetq/ dr(6,6),dq(6,6),de(6,6),di(6,6),
     *     al,cl,eps1,eps2,eps3,eps4
      al=a
      cl=c
      return
      end

      subroutine setd()
      common /cgethk/ d0(6,6),d1(6,6),d2(6,6),d3(6,6),d4(6,6)
      common /cgetq/ dr(6,6),dq(6,6),de(6,6),di(6,6),
     *     al,cl,eps1,eps2,eps3,eps4
      real piv(6),b(6)
      integer ipv(6),ind(6,2)
      common /cgetd/ det
      save icount
      real*4 sr,ct,c1,c2,c3,c4,s1,s2,s3,s4,su,cu
      parameter (sr=0.894427191,ct=0.447213596)
      parameter (su=-sr,cu=-ct)
      parameter (c1=0.309016994,c2=-0.809016994)
      parameter (c3=-c2,c4=-c1)
      parameter (s1=0.951056516,s2=0.587785252)
      parameter (s3=-s2,s4=-s1)
      parameter (c1sr=c1*sr,c2sr=c2*sr,c3sr=c3*sr,c4sr=c4*sr)
      parameter (s1sr=s1*sr,s2sr=s2*sr,s3sr=s3*sr,s4sr=s4*sr)
      parameter (tau=1.618034,tav=-tau)
      parameter (sqr2=1.41421, sqr3=1.73205, tqr3=-sqr3)
      parameter (t1=0.70710678, t2=-0.70710678)
      parameter (u1=0.8660254, u2=-u1) 
      data  d0/ 0.,   0.,   1., 0.,   0.,   1.,
     *     c2sr, s2sr, ct, c4sr, s1sr, cu,
     *     c2sr, s3sr, ct, c4sr, s4sr, cu,
     *     c1sr, s4sr, ct, c3sr, s2sr, cu,
     *     sr,   0.,   ct, su,   0.,   cu,
     *     c1sr, s1sr, ct, c3sr, s3sr, cu/
c     for cubic and trigonal
      data  d1/ 1., tau,  0.,  tau, -1., 0.,
     *     tau, 0.,  1.,  -1,  0.,  tau,
     *     tau, 0., -1.,  -1., 0.,  tav,
     *     0.,  1., tav,   0., tau, 1.,
     *     -1., tau, 0.,  tav, -1., 0.,
     *     0.,  1., tau,   0., tau, -1./
c     for decagonal  (******reciprocal lattice*******)
      data d2/  c1, s1, 0., c2, s2, 0.,
     *     c2, s2, 0., c1, s4, 0.,
     *     c2, s3, 0., c1, s1, 0.,
     *     c1, s4, 0., c2, s3, 0.,
     *     0., 0., 1., 0., 0., 0.,
     *     0., 0., 0., 0., 0., 1./       
c     for octagonal
      data d3/  1., 0., 0., 1., 0., 0.,
     *     t1, t1, 0., t2, t1, 0.,
     *     0., 1., 0., 0.,-1., 0.,
     *     t2, t1, 0., t1, t1, 0.,
     *     0., 0., 1., 0., 0., 0.,
     *     0., 0., 0., 0., 0., 1./
c     for dodecagonal
      data d4/  1., 0., 0., 1., 0., 0.,
     *     u1, 0.5, 0., u2, 0.5, 0.,
     *     0.5, u1, 0., 0.5, u2, 0.,
     *     0., 1.,  0., 0., 1.,  0.,
     *     0., 0.,  1., 0., 0.,  0.,
     *     0., 0.,  0., 0., 0.,  1./
      data icount /0/
      icount=icount+1
      if(icount.eq.1) then
c     initialize
         do i=1,6
            do j=1,6
               d0(i,j)=d0(i,j)/2
            end do
         end do
         do i=1,6
            do j=1,6
               d1(i,j)=d1(i,j)/sqrt(1+tau**2)/2
            end do
         end do
         do i=1,6
            do j=1,4
               d2(i,j)=d2(i,j)/sqrt(5.)
            end do
         end do
         do i=1,6
            do j=1,4
               d3(i,j)=d3(i,j)/2
            end do
         end do
         do i=1,6
            do j=1,4
               d4(i,j)=d4(i,j)/sqrt(2.)
            end do
         end do
         call trnsp(d0)
         call trnsp(d1)
         call trnsp(d2)
         call trnsp(d3)
         call trnsp(d4)
      end if
      call getsys(isys)
      if(isys.eq.1) then
         call mcpy(d0,dq)
      else if(isys.eq.2) then
         call mcpy(d1,dq)
      else if(isys.eq.3) then
         call mcpy(d2,dq)
      else if(isys.eq.4) then
         call mcpy(d3,dq)
      else if(isys.eq.5) then
         call mcpy(d4,dq)
      else
         call quit('wrong isys')
      end if
      call mcpy(dq,dr)
      call matinv(dr,6,6,b,0,det,ipv,ind,piv)
      return
      end

      subroutine getdet(det0)
      common /cgetd/ det
      det0=det
      return
      end

      subroutine setdei()
      common /cgetq/ dr(6,6),dq(6,6),de(6,6),di(6,6),
     *     al,cl,eps1,eps2,eps3,eps4
      common /cphasn/ amij1(3,3),amij2(3,3),amij3(3,3)
      common /cphonn/ bmij1(3,3)
      real fij1(3,3),fij2(3,3),fij3(3,3),fij4(3,3),fij5(3,3),fij6(3,3)
      integer q50(6,6),q30(6,10),q20(6,15),q10d0(6),q2d0(6),qmd0(6)
      common /cprjax/ q5p(6,6),q3p(6,10),q2p(6,15),q10d(6),q2d(6),qmd(6)
      real ctoh(6,6),htoc(6,6),ded(6,6)
      real piv(6),b(6)
      integer ipv(6),ind(6,2)
      real wmij2(3,3),wmij3(3,3),wmij4(3,3)
      save icount
      data icount /0/
c     for pentagonal D5 and decagonal
      data fij1 /0,0,0,0,0,0,0,0,1/
      data fij2 /1,0,0,0,0,0,0,0,0/
      data fij3 /0,0,0,0,1,0,0,0,0/
c     for cubic T and trigonal 
      data fij4 /1,0,0,0,1,0,0,0,1/
      data fij5 /0,0,0,0,0,0,0,0,1/
      data fij6 /1,0,0,0,1,0,0,0,0/
      data ctoh /0.0,     -0.816496, 0.816496, 0., 0., 0.,
     *     0.816496,-0.816496, 0.0,      0., 0., 0.,
     *     0.577350, 0.577350, 0.577350, 0., 0., 0.,
     *     0., 0., 0., 0.0,     -0.816496, 0.816496,
     *     0., 0., 0., 0.816496,-0.816496, 0.0,
     *     0., 0., 0., 0.577350, 0.577350, 0.577350/
      data q50/1,0,0,0,0,0,     ! 1
     *     0,1,0,0,0,0,         ! 2
     *     0,0,1,0,0,0,         ! 3
     *     0,0,0,1,0,0,         ! 4
     *     0,0,0,0,1,0,         ! 5
     *     0,0,0,0,0,1/         ! 6
      data q30/1,1,1,0,0,0,     ! 1
     *     1,0,1,1,0,0,         ! 2
     *     1,0,0,1,1,0,         ! 3
     *     1,0,0,0,1,1,         ! 4
     *     1,1,0,0,0,1,         ! 5  3-fold axis in cubic approximant
     *     0,1,1,0,-1,0,        ! 6
     *     0,0,1,1,0,-1,        ! 7
     *     0,-1,0,1,1,0,        ! 8
     *     0,0,-1,0,1,1,        ! 9
     *     0,1,0,-1,0,1/        ! 10
      data q20/1,1,0,0,0,0,     ! 1
     *     1,0,1,0,0,0,         ! 2
     *     1,0,0,1,0,0,         ! 3
     *     1,0,0,0,1,0,         ! 4  y-axis in cubic approximant
     *     1,0,0,0,0,1,         ! 5
     *     0,1,1,0,0,0,         ! 6  x-axis in cubic approximant
     *     0,0,1,1,0,0,         ! 7
     *     0,0,0,1,1,0,         ! 8
     *     0,0,0,0,1,1,         ! 9
     *     0,1,0,0,0,1,         ! 10
     *     0,1,0,0,-1,0,        ! 11
     *     0,0,1,0,0,-1,        ! 12
     *     0,-1,0,1,0,0,        ! 13
     *     0,0,-1,0,1,0,        ! 14  normal to 3-fold axis in cubic apprx.
     *     0,0,0,-1,0,1/        ! 15  z-axis in cubic approximant
      data q10d0 /0,0,0,0,1,0/
      data q2d0  /1,1,1,1,0,0/
      data qmd0  /1,0,0,-1,0,0/
      icount=icount+1
      if(icount.eq.1) then
         call trnsp(ctoh)
         call mcpy(ctoh,htoc)
         call matinv(htoc,6,6,b,0,det,ipv,ind,piv)
         call getsys(isys)
         do i=1,6
            call getqi(q50(1,i),q5p(1,i))
            call norml(q5p(1,i),3)
            call norml(q5p(4,i),3)
         end do
         do i=1,10
            call getqi(q30(1,i),q3p(1,i))
            call norml(q3p(1,i),3)
            call norml(q3p(4,i),3)
         end do
         do i=1,15
            call getqi(q20(1,i),q2p(1,i))
            call norml(q2p(1,i),3)
            call norml(q2p(4,i),3)
         end do
         call getqi(q10d0,q10d)
         call norml(q10d(1),3)
         call norml(q10d(4),3)
         call getqi(qmd0,qmd)
         call norml(qmd(1),3)
         call norml(qmd(4),3)
         call getqi(q2d0,q2d)
         call norml(q2d(1),3)
         call norml(q2d(4),3)
         if(isys.eq.1) then
c     Y. Ishii, Phys Rev. B39 (1989) 11862
c     D5 distortion
            call mcpy3(fij1,amij1)
            call mcpy3(fij2,amij2)
            call mcpy3(fij3,amij3)
            call mcpy3(fij1,bmij1)
         else if(isys.eq.2) then
c     T distortion  : eps1
            call mcpy3(fij4,amij1)
c     D3 phason distortion : eps2
            call mcpy3(fij5,amij2)
            call mcpy3(fij6,amij3)
c     D3 phonon distortion : eps4
            call mcpy3(fij5,bmij1)
            call getwmij(amij2,wmij2,ctoh,htoc)
            call getwmij(amij3,wmij3,ctoh,htoc)
            call getwmij(bmij1,wmij4,ctoh,htoc)
c     else if(isys.eq.3) then
         else if(isys.ge.3) then
c     orthorhombic phason strain
            call mcpy3(fij1,amij1)
            call mcpy3(fij2,amij2)
            call mcpy3(fij3,amij3)
         end if
         write(6,*) 'amij1='
         write(6,'(3f9.5)') ((amij1(i,j),j=1,3),i=1,3)
         if(isys.eq.2) then
            write(6,*) 'amij2='
            write(6,'(3f9.5)') ((wmij2(i,j),j=1,3),i=1,3)
            write(6,*) 'amij3='
            write(6,'(3f9.5)') ((wmij3(i,j),j=1,3),i=1,3)
            write(6,*) 'bmij1='
            write(6,'(3f9.5)') ((wmij4(i,j),j=1,3),i=1,3)
         else
            write(6,*) 'amij2='
            write(6,'(3f9.5)') ((amij2(i,j),j=1,3),i=1,3)
            write(6,*) 'amij3='
            write(6,'(3f9.5)') ((amij3(i,j),j=1,3),i=1,3)
            write(6,*) 'bmij1='
            write(6,'(3f9.5)') ((bmij1(i,j),j=1,3),i=1,3)
         end if
      end if
      if(isys.ne.2) then
         do i=1,6
            de(i,i)=1
            di(i,i)=1
         end do
         do i=4,6
            do j=1,3
               de(i,j)=eps1*amij1(i-3,j)+eps2*amij2(i-3,j)+
     *              eps3*amij3(i-3,j)
            end do
         end do
         do i=1,3
            do j=1,3
               de(i,j)=de(i,j)+eps4*bmij1(i,j)
            end do
         end do
      else if(isys.eq.2) then
         do i=1,6
            de(i,i)=1
         end do
         do i=4,6
            do j=1,3
               de(i,j)=eps2*amij2(i-3,j)+eps3*amij3(i-3,j)
            end do
         end do
         do i=1,3
            do j=1,3
               de(i,j)=de(i,j)+eps4*bmij1(i,j)
            end do
         end do
         call mtmpl(de,ctoh,ded)
         call mtmpl(htoc,ded,de)
         do i=4,6
            do j=1,3
               de(i,j)=de(i,j)+eps1*amij1(i-3,j)
            end do
         end do         
      end if
      call mcpy(de,di)
      call matinv(di,6,6,b,0,det,ipv,ind,piv)
      write(6,*) 'deformation matrix de'

      do i=1,6
         write(6,'(6f8.4)') (de(i,j),j=1,6)
      end do
      write(6,*) 'deformation matrix di'
      do i=1,6
         write(6,'(6f8.4)') (di(i,j),j=1,6)
      end do
      return
      end

      subroutine setmtrtns()
      call getsys(isys)
      if(isys.eq.0) then
         call setmtrtns0()
      else
         call setmtrtns1()
      end if
      return
      end

      subroutine setmtrtns0()
      common /cmtrt/ gab(6,6)
      common /mtrtns/ gabx(3,3),gaby(3,3)
      write(6,*) 'enter setmtrtns0'
      do i=1,6
         do j=1,6
            if(i.le.3.and.j.le.3) then
               gab(i,j)=gabx(i,j)
            else
               gab(i,j)=0
            end if      
         end do
      end do
c     call wtmtrxr('gab',gab,6)
c     stop
      return
      end

      subroutine setmtrtns1()
      common /cgetq/ dr(6,6),dq(6,6),de(6,6),di(6,6),
     *          al,cl,eps1,eps2,eps3,eps4
      common /cmtrt/ gab(6,6)
      real dt(6,6)
      call getsys(isys)
      call mcpy(dr,dt)
      call trnsp(dt)
      do i=1,6
         do j=1,6
            gab(i,j)=vinp(dt(1,i),dt(1,j),6)*al**2
         end do
      end do
      if(isys.ge.3) then
         gab(5,5)=gab(5,5)*cl**2/al**2
      end if
      return
      end

      subroutine setidm(idmd)
      common /cdim/ idm,nde
      idm=idmd
      nde=idm-3
      return
      end

      subroutine getidm(idmd)
      common /cdim/ idm,nde
      idmd=idm
      return
      end

      subroutine openfile(iun,flnm,st) ! for text file
      integer iun
      character*(*) flnm,st
      lflnm=istrln(flnm)
      lst=istrln(st)
      open (unit=iun,file=flnm(1:lflnm),status=st(1:lst),err=910)
      go to 810
 910  write(6,*) 'can''t open ',flnm(1:lflnm)
      call exit(1)
 810  continue
      return
      end

      subroutine getqi(h,q)
      integer h(*)
      real q(*)
      call getsys(isys)
      if(isys.eq.0) then
         call getqi0(h,q) ! for crystals
      else
         call getqi1(h,q)
      end if
      return
      end

      subroutine gethr(h,hr)
      integer h(*)
      real hr(*)
      common /kwave/ y(3,3,3),vc(3)
      call getidm(idm)
      do i=1,3
         hr(i)=h(i)
      end do         
      do i=1,3
         do j=1,idm-3
            hr(i)=hr(i)+y(i,j,1)*h(j)
         end do
      end do
c      call wtvctrif('h',h,3,'(3i4)')
c      call wtvctrrf('hr',hr,3,'(3f10.4)')
      return
      end

      subroutine getqi0(h,q)
      integer h(*)
      real q(*)
      common /mtrtns/ gabx(3,3),gaby(3,3),gab(3,3,3),gabn(3,3,3)
      real hr(3)
      call getidm(idm)
      call gethr(h,hr)
      call zerovr(q,6)
c     call wtmtrxrf('gaby',gaby,3,'(3f10.4)')
c     stop
      do i=1,3
         do j=1,3
            q(i)=q(i)+hr(j)*gaby(j,i)
         end do
      end do
c     call wtvctrr('q=',q,3)
      return
      end

      subroutine getqi1(h,q)
      integer h(*)
      real q(*)
      common /cgetq/ dr(6,6),dq(6,6),de(6,6),di(6,6),
     *     al,cl,eps1,eps2,eps3,eps4
      call getsys(isys)
      do i=1,6
         q(i)=0.
         do j=1,6
            q(i)=q(i)+h(j)*dq(j,i)
         end do
         if(isys.le.2) then
            if(i.le.3) then
               q(i)=q(i)/al
            else
               q(i)=q(i)/al
            end if
         else
            if(i.le.2) then
               q(i)=q(i)/al
            else if(i.eq.3) then
               q(i)=q(i)/cl
            else if(i.le.5) then
               q(i)=q(i)/al
            end if
         end if          
      end do
      return
      end

      subroutine getsys(isysd)
c     common /cgethk/ d0(6,6),d1(6,6),d2(6,6),d3(6,6),isys
      common /cisys/ isys
      isysd=isys
      return
      end            

      subroutine setsys(isysd)
c     common /cgethk/ d0(6,6),d1(6,6),d2(6,6),d3(6,6),isys
      common /cisys/ isys
      isys=isysd
      return
      end            

      subroutine getwmij(amij,wmij,ctoh,htoc)
      real amij(3,3),wmij(3,3),ctoh(6,6),htoc(6,6),tmp(6,6)
      do i=1,3
         do j=1,3
            tmp(i,j)=0
            do k=1,3
               tmp(i,j)=tmp(i,j)+amij(i,k)*ctoh(k,j)
            end do
         end do
      end do
      do i=1,3
         do j=1,3
            wmij(i,j)=0
            do k=1,3
               wmij(i,j)=wmij(i,j)+htoc(i,k)*tmp(k,j)
            end do
         end do
      end do
      return
      end

c      subroutine quit(str)
c      character*(*) str
c      write(6,*) str
cc      time=etime(tarray)
cc      write(6,*) '******elapsed time=',time,'sec******'
c      close(unit=2)
c      close(unit=3)
c      call exit()
c      end

      subroutine getmm(mm,hmax,idm)
      integer mm(*)
      integer hmax(*)
      mm(1)=1
      nt=1
      do j=1,idm
         mm(j+1)=mm(j)*hmax(j)
         nt=nt*hmax(j)
      end do
      return
      end

      integer function getnt(hmax,idm)
      integer hmax(*)
      nt=hmax(1)
      do j=2,idm
         nt=nt*hmax(j)
      end do
      getnt=nt
      return
      end

      subroutine setvol(vold)
      common /cvol/ vol
      vol=vold
      return
      end

      function getvol()
      common /cvol/ vol
      getvol=vol
      return
      end

      subroutine settbl(at,a1,b1,a2,b2,a3,b3,a4,b4,c,sl)
      character*(*) at
      parameter(natm=400)
      character*5 atm
      common /cgenasf/ atm(natm),aa(4,natm),bb(4,natm),cc(natm),
     *     sctlng(natm)
      common /cic/ ic
      ic=ic+1
      istr=istrln(at)   ! number of non-blank non-! caracters      
      atm(ic)(1:5)='     '
      atm(ic)(1:istr)=at(1:istr)
      aa(1,ic)=a1
      aa(2,ic)=a2
      aa(3,ic)=a3
      aa(4,ic)=a4
      bb(1,ic)=b1
      bb(2,ic)=b2
      bb(3,ic)=b3
      bb(4,ic)=b4
      cc(ic)=c
      sctlng(ic)=sl
c     write(6,*) ic,(aa(j,ic),bb(j,ic),j=1,4),cc(ic),sctlng(ic)
      return
      end

      subroutine gettbl(at,ai,bi,c)
      character*(*) at
      real ai(4),bi(4),c
      parameter(natm=400)
      character*5 atm
      common /cgenasf/ atm(natm),aa(4,natm),bb(4,natm),cc(natm),
     *     sctlng(natm)
      common /cic/ ic
      istr=istrln(at)
      do i=1,natm
         if(at(1:5).eq.atm(i)(1:5)) then
            ai(1)=aa(1,i)
            ai(2)=aa(2,i)
            ai(3)=aa(3,i)
            ai(4)=aa(4,i)
            bi(1)=bb(1,i)
            bi(2)=bb(2,i)
            bi(3)=bb(3,i)
            bi(4)=bb(4,i)
            c=cc(i)
c           write(6,*) i,(ai(j),bi(j),j=1,4),c,sc
            return
         end if
      end do
      call quit('scattering factor table not found')
      end

      subroutine genasf()
      parameter(natm=400)
      character*5 atm
      common /cgenasf/ atm(natm),aa(4,natm),bb(4,natm),cc(natm),
     *     sctlng(natm)
      common /cic/ ic
      ic=0
      call settbl('Vc',0.,0.,0.,0.,0.,0.,0.,0.,0.,0.)
      call settbl('H',0.48992,20.65930,0.26200,7.74039,0.19677,
     *     49.55190,0.04988,2.20159,0.00131,-3.7409)
      call settbl('H-',0.89766,53.13680,0.56562,15.18700,0.41581,
     *     186.57600,0.11697,3.56709,0.00239,-3.7409)
      call settbl('D',0.48992,20.65930,0.26200,7.74039,0.19677,
     *     49.55190,0.04988,2.20159,0.00131,6.674)
      call settbl('He',0.87340,9.10370,0.63090,3.35680,0.31120,
     *     22.92760,0.17800,0.98210,0.00640,3.26)
      call settbl('Li',1.12820,3.95460,0.75080,1.05240,0.61750,
     *     85.39050,0.46530,168.26100,0.03770,-1.9)
      call settbl('Li+',0.69680,4.62370,0.78880,1.95570,0.34140,
     *     0.63160,0.15630,10.09530,0.01670,-1.9)
      call settbl('Be',1.59190,43.64270,1.12780,1.86230,0.53910,
     *     103.48300,0.70290,0.54200,0.03850,7.79)
      call settbl('Be2+',6.26030,0.00270,0.88490,0.83130,0.79930,
     *     2.27580,0.16470,5.11460,-6.10920,7.79)
      call settbl('B',2.05450,23.21850,1.33260,1.02100,1.09790,
     *     60.34980,0.70680,0.14030,-0.19320,5.30)
      call settbl('C',2.31000,20.84390,1.02000,10.20750,1.58860,
     *     0.56870,0.86500,51.65120,0.21560,6.6484)
      call settbl('N',12.21260,0.00570,3.13220,9.89330,2.01250,
     *     28.99750,1.16630,0.58260,-11.52900,9.36)
      call settbl('O',3.04850,13.27710,2.28680,5.70110,1.54630,
     *     0.32390,0.86700,32.90890,0.25080,5.805)
      call settbl('O-',4.19160,12.85730,1.63969,4.17236,1.52673,
     *     47.01790,-20.30700,-0.01404,21.94120,5.805)
      call settbl('F',3.53920,10.28250,2.64120,4.29440,1.51700,
     *     0.26150,1.02430,26.14760,0.27760,5.654)
      call settbl('F-',3.63220,5.27756,3.51057,14.73530,1.26064,
     *     0.44226,0.94071,47.34370,0.65340,5.654)
      call settbl('Ne',3.95530,8.40420,3.11250,3.42620,1.45460,
     *     0.23060,1.12510,21.71840,0.35150,4.547)
      call settbl('Na',4.76260,3.28500,3.17360,8.84220,1.26740,
     *     0.31360,1.11280,129.42400,0.67600,3.63)
      call settbl('Na+',3.25650,2.66710,3.93620,6.11530,1.39980,
     *     0.20010,1.00320,14.03900,0.40400,3.63)
      call settbl('Mg',5.42040,2.82750,2.17350,79.26110,1.22690,
     *     0.38080,2.30730,7.19370,0.85840,5.375)
      call settbl('Mg2+',3.49880,2.16760,3.83780,4.75420,1.32840,
     *     0.18500,0.84970,10.14110,0.48530,5.375)
      call settbl('Al',6.42020,3.03870,1.90020,0.74260,1.59360,
     *     31.54720,1.96460,85.08860,1.11510,3.449)
      call settbl('Al3+',4.17448,1.93816,3.38760,4.14553,1.20296,
     *     0.22875,0.52814,8.28524,0.70679,3.449)
      call settbl('Si',6.29150,2.43860,3.03530,32.33370,1.98910,
     *     0.67850,1.54100,81.69370,1.14070,4.149)
      call settbl('Si.v',5.66269,2.66520,3.07164,38.66340,2.62446,
     *     0.91695,1.39320,93.54580,1.24707,4.149)
      call settbl('Si4+',4.43918,1.64167,3.20345,3.43757,1.19453,
     *     0.21490,0.41653,6.65365,0.74630,4.149)
      call settbl('P',6.43450,1.90670,4.17910,27.15700,1.78000,
     *     0.52600,1.49080,68.16450,1.11490,5.13)
      call settbl('S',6.90530,1.46790,5.20340,22.21510,1.43790,
     *     0.25360,1.58630,56.17200,0.86690,2.847)
      call settbl('Cl',11.46040,0.01040,7.19640,1.16620,6.25560,
     *     18.51940,1.64550,47.77840,-9.55740,9.5792)
      call settbl('Cl-',18.29150,0.00660,7.20840,1.17170,6.53370,
     *     19.54240,2.33860,60.44860,-16.37800,9.5792)
      call settbl('Ar',7.48450,0.90720,6.77230,14.84070,0.65390,
     *     43.89830,1.64420,33.39290,1.44450,1.909)
      call settbl('K',8.21860,12.79490,7.43980,0.77480,1.05190,
     *     213.18700,0.86590,41.68410,1.42280,3.71)
      call settbl('K+',7.95780,12.63310,7.49170,0.76740,6.35900,
     *     -0.00200,1.19150,31.91280,-4.99780,3.71)
      call settbl('Ca',8.62660,10.44210,7.38730,0.65990,1.58990,
     *     85.74840,1.02110,178.43700,1.37510,4.90)
      call settbl('Ca2+',15.63480,-0.00740,7.95180,0.60890,8.43720,
     *     10.31160,0.85370,25.99050,-14.87500,4.90)
      call settbl('Sc',9.18900,9.02130,7.36790,0.57290,1.64090,
     *     136.10800,1.46800,51.35310,1.33290,12.29)
      call settbl('Sc3+',13.40080,0.29854,8.02730,7.96290,1.65943,
     *     -0.28604,1.57936,16.06620,-6.66670,12.29)
      call settbl('Ti',9.75950,7.85080,7.35580,0.50000,1.69910,
     *     35.63380,1.90210,116.10500,1.28070,-3.30)
      call settbl('Ti2+',9.11423,7.52430,7.62174,0.45759,2.27930,
     *     19.53610,0.08790,61.65580,0.89715,-3.30)
      call settbl('Ti3+',17.73440,0.22061,8.73816,7.04716,5.25691,
     *     -0.15762,1.92134,15.97680,-14.65200,-3.30)
      call settbl('Ti4+',19.51140,0.17885,8.23473,6.67018,2.01341,
     *     -0.29263,1.52080,12.94640,-13.28000,-3.30)
      call settbl('V',10.29710,6.86570,7.35110,0.43850,2.07030,
     *     26.89380,2.05710,102.47800,1.21990,-0.3824)
      call settbl('V2+',10.10600,6.88180,7.35410,0.44090,2.28840,
     *     20.30040,0.02230,115.12200,1.22980,-0.3824)
      call settbl('V3+',9.43141,6.39535,7.74190,0.38335,2.15343,
     *     15.19080,0.01687,63.96900,0.65657,-0.3824)
      call settbl('V5+',15.68870,0.67900,8.14208,5.40135,2.03081,
     *     9.97278,-9.57600,0.94046,1.71430,-0.3824)
      call settbl('Cr',10.64060,6.10380,7.35370,0.39200,3.32400,
     *     20.26260,1.49220,98.73990,1.18320,3.635)
      call settbl('Cr2+',9.54034,5.66078,7.75090,0.34426,3.58274,
     *     13.30750,0.50911,32.42240,0.61690,3.635)
      call settbl('Cr3+',9.68090,5.59463,7.81136,0.33439,2.87603,
     *     12.82880,0.11357,32.87610,0.51828,3.635)
      call settbl('Mn',11.28190,5.34090,7.35730,0.34320,3.01930,
     *     17.86740,2.24410,83.75430,1.08960,-3.73)
      call settbl('Mn2+',10.80610,5.27960,7.36200,0.34350,3.52680,
     *     14.34300,0.21840,41.32350,1.08740,-3.73)
      call settbl('Mn3+',9.84521,4.91797,7.87194,0.29439,3.56531,
     *     10.81710,0.32361,24.12810,0.39397,-3.73)
      call settbl('Mn4+',9.96253,4.84850,7.97057,0.28330,2.76067,
     *     10.48520,0.05445,27.57300,0.25188,-3.73)
      call settbl('Fe',11.76950,4.76110,7.35730,0.30720,3.52220,
     *     15.35350,2.30450,76.88050,1.03690,9.54)
      call settbl('Fe2+',11.04240,4.65380,7.37400,0.30530,4.13460,
     *     12.05460,0.43990,31.28090,1.00970,9.54)
      call settbl('Fe3+',11.17640,4.61470,7.38630,0.30050,3.39480,
     *     11.67290,0.07240,38.55660,0.97070,9.54)
      call settbl('Co',12.28410,4.27910,7.34090,0.27840,4.00340,
     *     13.53590,2.34880,71.16920,1.01180,2.50)
      call settbl('Co2+',11.22960,4.12310,7.38830,0.27260,4.73930,
     *     10.24430,0.71080,25.64660,0.93240,2.50)
      call settbl('Co3+',10.33800,3.90969,7.88173,0.23867,4.76795,
     *     8.35583,0.72559,18.34910,0.28667,2.50)
      call settbl('Ni',12.83760,3.87850,7.29200,0.25650,4.44380,
     *     12.17630,2.38000,66.34210,1.03410,10.3)
      call settbl('Ni2+',11.41660,3.67660,7.40050,0.24490,5.34420,
     *     8.87300,0.97730,22.16260,0.86140,10.3)
      call settbl('Ni3+',10.78060,3.54770,7.75868,0.22314,5.22746,
     *     7.64468,0.84711,16.96730,0.38604,10.3)
      call settbl('Cu',13.33800,3.58280,7.16760,0.24700,5.61580,
     *     11.39660,1.67350,64.81260,1.19100,7.718)
      call settbl('Cu+',11.94750,3.36690,7.35730,0.22740,6.24550,
     *     8.66250,1.55780,25.84870,0.89000,7.718)
      call settbl('Cu2+',11.81680,3.37484,7.11181,0.24408,5.78135,
     *     7.98760,1.14523,19.89700,1.14431,7.718)
      call settbl('Zn',14.07430,3.26550,7.03180,0.23330,5.16520,
     *     10.31630,2.41000,58.70970,1.30410,5.680)
      call settbl('Zn2+',11.97190,2.99460,7.38620,0.20310,6.46680,
     *     7.08260,1.39400,18.09950,0.78070,5.680)
      call settbl('Ga',15.23540,3.06690,6.70060,0.24120,4.35910,
     *     10.78050,2.96230,61.41350,1.71890,7.288)
      call settbl('Ga3+',12.69200,2.81262,6.69883,0.22789,6.06692,
     *     6.36441,1.00660,14.41220,1.53545,7.288)
      call settbl('Ge',16.08160,2.85090,6.37470,0.25160,3.70680,
     *     11.44680,3.68300,54.76250,2.13130,8.1929)
      call settbl('Ge4+',12.91720,2.53718,6.70003,0.20585,6.06791,
     *     5.47913,0.85904,11.60300,1.45572,8.1929)
      call settbl('As',16.67230,2.63450,6.07010,0.26470,3.43130,
     *     12.94790,4.27790,47.79720,2.53100,6.58)
      call settbl('Se',17.00060,2.40980,5.81960,0.27260,3.97310,
     *     15.23720,4.35430,43.81630,2.84090,7.970)
      call settbl('Br',17.17890,2.17230,5.23580,16.57960,5.63770,
     *     0.26090,3.98510,41.43280,2.95570,6.79)
      call settbl('Br-',17.17180,2.20590,6.33380,19.33450,5.57540,
     *     0.28710,3.72720,58.15350,3.17760,6.79)
      call settbl('Kr',17.35550,1.93840,6.72860,16.56230,5.54930,
     *     0.22610,3.53750,39.39720,2.82500,7.80)
      call settbl('Rb',17.17840,1.78880,9.64350,17.31510,5.13990,
     *     0.27480,1.52920,164.93401,3.48730,7.08)
      call settbl('Rb+',17.58160,1.71390,7.65980,14.79570,5.89810,
     *     0.16030,2.78170,31.20870,2.07820,7.08)
      call settbl('Sr',17.56630,1.55640,9.81840,14.09880,5.42200,
     *     0.16640,2.66940,132.37601,2.50640,7.02)
      call settbl('Sr2+',18.08740,1.49070,8.13730,12.69630,2.56540,
     *     24.56510,-34.19300,-0.01380,41.40250,7.02)
      call settbl('Y',17.77600,1.40290,10.29460,12.80060,5.72629,
     *     0.12560,3.26588,104.35400,1.91213,7.75)
      call settbl('Y3+',17.92680,1.35417,9.15310,11.21450,1.76795,
     *     22.65990,-33.10800,-0.01319,40.26020,7.75)
      call settbl('Zr',17.87650,1.27618,10.94800,11.91600,5.41732,
     *     0.11762,3.65721,87.66270,2.06929,7.16)
      call settbl('Zr4+',18.16680,1.21480,10.05620,10.14830,1.01118,
     *     21.60540,-2.64790,-0.10276,9.41454,7.16)
      call settbl('Nb',17.61420,1.18865,12.01440,11.76600,4.04183,
     *     0.20479,3.53346,69.79570,3.75591,7.054)
      call settbl('Nb3+',19.88120,0.01918,18.06530,1.13305,11.01770,
     *     10.16210,1.94715,28.33890,-12.91200,7.054)
      call settbl('Nb5+',17.91630,1.12446,13.34170,0.02878,10.79900,
     *     9.28206,0.33790,25.72280,-6.39340,7.054)
      call settbl('Mo',3.70250,0.27720,17.23560,1.09580,12.88760,
     *     11.00400,3.74290,61.65840,4.38750,6.95)
      call settbl('Mo3+',21.16640,0.01473,18.20170,1.03031,11.74230,
     *     9.53659,2.30951,26.63070,-14.42100,6.95)
      call settbl('Mo5+',21.01490,0.01434,18.09920,1.02238,11.46320,
     *     8.78809,0.74063,23.34520,-14.31600,6.95)
      call settbl('Mo6+',17.88710,1.03649,11.17500,8.48061,6.57891,
     *     0.05888,0.00000,0.00000,0.34494,6.95)
      call settbl('Tc',19.13010,0.86413,11.09480,8.14487,4.64901,
     *     21.57070,2.71263,86.84720,5.40428,6.8)
      call settbl('Ru',19.26740,0.80852,12.91820,8.43467,4.86337,
     *     24.79970,1.56756,94.29280,5.37874,.721)
      call settbl('Ru3+',18.56380,0.84733,13.28850,8.37164,9.32602,
     *     0.01766,3.00964,22.88700,-3.18920,.721)
      call settbl('Ru4+',18.50030,0.84458,13.17870,8.12534,4.71304,
     *     0.03650,2.18535,20.85040,1.42357,.721)
      call settbl('Rh',19.29570,0.75154,14.35010,8.21758,4.73425,
     *     25.87490,1.28918,98.60620,5.32800,5.88)
      call settbl('Rh3+',18.87850,0.76425,14.12590,7.84438,3.32515,
     *     21.24870,-6.19890,-0.01036,11.86780,5.88)
      call settbl('Rh4+',18.85450,0.76082,13.98060,7.62436,2.53464,
     *     19.33170,-5.65260,-0.01020,11.28350,5.88)
      call settbl('Pd',19.33190,0.69866,15.50170,7.98929,5.29537,
     *     25.20520,0.60584,76.89860,5.26593,5.91)
      call settbl('Pd2+',19.17010,0.69622,15.20960,7.55573,4.32234,
     *     22.50570,0.00000,0.00000,5.29160,5.91)
      call settbl('Pd4+',19.24930,0.68384,14.79000,7.14833,2.89289,
     *     17.91440,-7.94920,0.00513,13.01740,5.91)
      call settbl('Ag',19.28080,0.64460,16.68850,7.47260,4.80450,
     *     24.66050,1.04630,99.81560,5.17900,5.922)
      call settbl('Ag+',19.18120,0.64618,15.97190,7.19123,5.27475,
     *     21.73260,0.35753,66.11470,5.21572,5.922)
      call settbl('Ag2+',19.16430,0.64564,16.24560,7.18544,4.37090,
     *     21.40720,0.00000,0.00000,5.21404,5.922)
      call settbl('Cd',19.22140,0.59460,17.64440,6.90890,4.46100,
     *     24.70080,1.60290,87.48250,5.06940,5.1)
      call settbl('Cd2+',19.15140,0.59792,17.25350,6.80639,4.47128,
     *     20.25210,0.00000,0.00000,5.11937,5.1)
      call settbl('In',19.16240,0.54760,18.55960,6.37760,4.29480,
     *     25.84990,2.03960,92.80290,4.93910,4.065)
      call settbl('In3+',19.10450,0.55152,18.11080,6.32470,3.78897,
     *     17.35950,0.00000,0.00000,4.99635,4.065)
      call settbl('Sn',19.18890,5.83030,19.10050,0.50310,4.45850,
     *     26.89090,2.46630,83.95710,4.78210,6.228)
      call settbl('Sn2+',19.10940,0.50360,19.05480,5.83780,4.56480,
     *     23.37520,0.48700,62.20610,4.78610,6.228)
      call settbl('Sn4+',18.93330,5.76400,19.71310,0.46550,3.41820,
     *     14.00490,0.01930,-0.75830,3.91820,6.228)
      call settbl('Sb',19.64180,5.30340,19.04550,0.46070,5.03710,
     *     27.90740,2.68270,75.28250,4.59090,5.641)
      call settbl('Sb3+',18.97550,0.46720,18.93300,5.22126,5.10789,
     *     19.59020,0.28875,55.51130,4.69626,5.641)
      call settbl('Sb5+',19.86850,5.44853,19.03020,0.46797,2.41253,
     *     14.12590,0.00000,0.00000,4.69263,5.641)
      call settbl('Te',19.96440,4.81742,19.01380,0.42088,6.14487,
     *     28.52840,2.52390,70.84030,4.35200,5.43)
      call settbl('I',20.14720,4.34700,18.99490,0.38140,7.51380,
     *     27.76600,2.27350,66.87760,4.07120,5.28)
      call settbl('I-',20.23320,4.35790,18.99700,0.38150,7.80690,
     *     29.52590,2.88680,84.93040,4.07140,5.28)
      call settbl('Xe',20.29330,3.92820,19.02980,0.34400,8.97670,
     *     26.46590,1.99000,64.26580,3.71180,4.85)
      call settbl('Cs',20.38920,3.56900,19.10620,0.31070,10.66200,
     *     24.38790,1.49530,213.90401,3.33520,5.42)
      call settbl('Cs+',20.35240,3.55200,19.12780,0.30860,10.28210,
     *     23.71280,0.96150,59.45650,3.27910,5.42)
      call settbl('Ba',20.33610,3.21600,19.29700,0.27560,10.88800,
     *     20.20730,2.69590,167.20200,2.77310,5.25)
      call settbl('Ba2+',20.18070,3.21367,19.11360,0.28331,10.90540,
     *     20.05580,0.77634,51.74600,3.02902,5.25)
      call settbl('La',20.57800,2.94817,19.59900,0.24448,11.37270,
     *     18.77260,3.28719,133.12399,2.14678,8.24)
      call settbl('La3+',20.24890,2.92070,19.37630,0.25070,11.63230,
     *     17.82110,0.33605,54.94530,2.40860,8.24)
      call settbl('Ce',21.16710,2.81219,19.76950,0.22684,11.85130,
     *     17.60830,3.33049,127.11300,1.86264,4.84)
      call settbl('Ce3+',20.80360,2.77691,19.55900,0.23154,11.93690,
     *     16.54080,0.61238,43.16920,2.09013,4.84)
      call settbl('Ce4+',20.32350,2.65941,19.81860,0.21885,12.12330,
     *     15.79920,0.14458,62.23550,1.59180,4.84)
      call settbl('Pr',22.04400,2.77393,19.66970,0.22209,12.38560,
     *     16.76690,2.82428,143.64400,2.05830,4.45)
      call settbl('Pr3+',21.37270,2.64520,19.74910,0.21430,12.13290,
     *     15.32300,0.97518,36.40650,1.77132,4.45)
      call settbl('Pr4+',20.94130,2.54467,20.05390,0.20248,12.46680,
     *     14.81370,0.29669,45.46430,1.24285,4.45)
      call settbl('Nd',22.68450,2.66248,19.68470,0.21063,12.77400,
     *     15.88500,2.85137,137.90300,1.98486,7.69)
      call settbl('Nd3+',21.96100,2.52722,19.93390,0.19924,12.12000,
     *     14.17830,1.51031,30.87170,1.47588,7.69)
      call settbl('Pm',23.34050,2.56270,19.60950,0.20209,13.12350,
     *     15.10090,2.87516,132.72099,2.02876,12.6)
      call settbl('Pm3+',22.55270,2.41740,20.11080,0.18577,12.06710,
     *     13.12750,2.07492,27.44910,1.19499,12.6)
      call settbl('Sm',24.00420,2.47274,19.42580,0.19645,13.43960,
     *     14.39960,2.89604,128.00700,2.20963,4.2)
      call settbl('Sm3+',23.15040,2.31641,20.25990,0.17408,11.92020,
     *     12.15710,2.71488,24.82420,0.95459,4.2)
      call settbl('Eu',24.62740,2.38790,19.08860,0.19420,13.76030,
     *     13.75460,2.92270,123.17400,2.57450,6.68)
      call settbl('Eu2+',24.00630,2.27783,19.95040,0.17353,11.80340,
     *     11.60960,3.87243,26.51560,1.36389,6.68)
      call settbl('Eu3+',23.74970,2.22258,20.37450,0.16394,11.85090,
     *     11.31100,3.26503,22.99660,0.75934,6.68)
      call settbl('Gd',25.07090,2.25341,19.07980,0.18195,13.85180,
     *     12.93310,3.54545,101.39800,2.41960,9.5)
      call settbl('Gd3+',24.34660,2.13553,20.42080,0.15552,11.87080,
     *     10.57820,3.71490,21.70290,0.64509,9.5)
      call settbl('Tb',24.37360,2.46637,13.86490,0.47517,13.25100,
     *     14.04240,3.24435,117.44600,10.24200,7.38)
      call settbl('Tb3+',24.95590,2.05601,20.32710,0.14953,12.24710,
     *     10.04990,3.77300,21.27730,0.69197,7.38)
      call settbl('Dy',26.50700,2.18020,17.63830,0.20217,14.55960,
     *     12.18990,2.96577,111.87400,4.29728,1.69)
      call settbl('Dy3+',25.53950,1.98040,20.28610,0.14338,11.98120,
     *     9.34972,4.50073,19.58100,0.68969,1.69)
      call settbl('Ho',26.90490,2.07051,17.29400,0.19794,14.55830,
     *     11.44070,3.63837,92.65660,4.56796,8.5)
      call settbl('Ho3+',26.12960,1.91072,20.09940,0.13936,11.97880,
     *     8.80018,4.93676,18.59080,0.85280,8.5)
      call settbl('Er',27.65630,2.07356,16.42850,0.22354,14.97790,
     *     11.36040,2.98233,105.70300,5.92046,8.03)
      call settbl('Er3+',26.72200,1.84659,19.77480,0.13729,12.15060,
     *     8.36225,5.17379,17.89740,1.17613,8.03)
      call settbl('Tm',28.18190,2.02859,15.88510,0.23885,15.15420,
     *     10.99750,2.98706,102.96100,6.75621,7.05)
      call settbl('Tm3+',27.30830,1.78711,19.33200,0.13697,12.33390,
     *     7.96778,5.38348,17.29220,1.63929,7.05)
      call settbl('Yb',28.66410,1.98890,15.43450,0.25712,15.30870,
     *     10.66470,2.98963,100.41700,7.56672,12.4)
      call settbl('Yb2+',28.12090,1.78503,17.68170,0.15997,13.33350,
     *     8.18304,5.14657,20.39000,3.70983,12.4)
      call settbl('Yb3+',27.89170,1.73272,18.76140,0.13879,12.60720,
     *     7.64412,5.47647,16.81530,2.26001,12.4)
      call settbl('Lu',28.94760,1.90182,15.22080,9.98519,15.10000,
     *     0.26103,3.71601,84.32980,7.97628,7.3)
      call settbl('Lu3+',28.46280,1.68216,18.12100,0.14229,12.84290,
     *     7.33727,5.59415,16.35350,2.97573,7.3)
      call settbl('Hf',29.14400,1.83262,15.17260,9.59990,14.75860,
     *     0.27512,4.30013,72.02900,8.58154,7.77)
      call settbl('Hf4+',28.81310,1.59136,18.46010,0.12890,12.72850,
     *     6.76232,5.59927,14.03660,2.39699,7.77)
      call settbl('Ta',29.20240,1.77333,15.22930,9.37046,14.51350,
     *     0.29598,4.76492,63.36440,9.24354,6.91)
      call settbl('Ta5+',29.15870,1.50711,18.84070,0.11674,12.82680,
     *     6.31524,5.38695,12.42440,1.78555,6.91)
      call settbl('W',29.08180,1.72029,15.43000,9.22590,14.43270,
     *     0.32170,5.11982,57.05600,9.88750,4.77)
      call settbl('W6+',29.49360,1.42755,19.37630,0.10462,13.05440,
     *     5.93667,5.06412,11.19720,1.01074,4.77)
      call settbl('Re',28.76210,1.67191,15.71890,9.09227,14.55640,
     *     0.35050,5.44174,52.08610,10.47200,9.2)
      call settbl('Os',28.18940,1.62903,16.15500,8.97948,14.93050,
     *     0.38266,5.67589,48.16470,11.00050,11.0)
      call settbl('Os4+',30.41900,1.37113,15.26370,6.84706,14.74580,
     *     0.16519,5.06795,18.00300,6.49804,11.0)
      call settbl('Ir',27.30490,1.59279,16.72960,8.86553,15.61150,
     *     0.41792,5.83377,45.00110,11.47220,10.6)
      call settbl('Ir3+',30.41560,1.34323,15.86200,7.10909,13.61450,
     *     0.20463,5.82008,20.32540,8.27903,10.6)
      call settbl('Ir4+',30.70580,1.30923,15.55120,6.71983,14.23260,
     *     0.16725,5.53672,17.49110,6.96824,10.6)
      call settbl('Pt',27.00590,1.51293,17.76390,8.81174,15.71310,
     *     0.42459,5.78370,38.61030,11.68830,9.63)
      call settbl('Pt2+',29.84290,1.32927,16.72240,7.38979,13.21530,
     *     0.26330,6.35234,22.94260,9.85329,9.63)
      call settbl('Pt4+',30.96120,1.24813,15.98290,6.60834,13.73480,
     *     0.16864,5.92034,16.93920,7.39534,9.63)
      call settbl('Au',16.88190,0.46110,18.59130,8.62160,25.55820,
     *     1.48260,5.86000,36.39560,12.06580,7.63)
      call settbl('Au+',28.01090,1.35321,17.82040,7.73950,14.33590,
     *     0.35675,6.58077,26.40430,11.22990,7.63)
      call settbl('Au3+',30.68860,1.21990,16.90290,6.82872,12.78010,
     *     0.21287,6.52354,18.65900,9.09680,7.63)
      call settbl('Hg',20.68090,0.54500,19.04170,8.44840,21.65750,
     *     1.57290,5.96760,38.32460,12.60890,12.66)
      call settbl('Hg+',25.08530,1.39507,18.49730,7.65105,16.88830,
     *     0.44338,6.48216,28.22620,12.02050,12.66)
      call settbl('Hg2+',29.56410,1.21152,18.06000,7.05639,12.83740,
     *     0.28474,6.89912,20.74820,10.62680,12.66)
      call settbl('Tl',27.54460,0.65515,19.15840,8.70751,15.53800,
     *     1.96347,5.52593,45.81490,13.17460,8.785)
      call settbl('Tl+',21.39850,1.47110,20.47230,0.51739,18.74780,
     *     7.43463,6.82847,28.84820,12.52580,8.785)
      call settbl('Tl3+',30.86950,1.10080,18.38410,6.53852,11.93280,
     *     0.21907,7.00574,17.21140,9.80270,8.785)
      call settbl('Pb',31.06170,0.69020,13.06370,2.35760,18.44200,
     *     8.61800,5.96960,47.25790,13.41180,9.4003)
      call settbl('Pb2+',21.78860,1.33660,19.56820,0.48838,19.14060,
     *     6.77270,7.01107,23.81320,12.47340,9.4003)
      call settbl('Pb4+',32.12440,1.00566,18.80030,6.10926,12.01750,
     *     0.14704,6.96886,14.71400,8.08428,9.4003)
      call settbl('Bi',33.36890,0.70400,12.95100,2.92380,16.58770,
     *     8.79370,6.46920,48.00930,13.57820,8.5256)
      call settbl('Bi3+',21.80530,1.23560,19.50260,6.24149,19.10530,
     *     0.47000,7.10295,20.31850,12.47110,8.5256)
      call settbl('Bi5+',33.53640,0.91654,25.09460,0.03904,19.24970,
     *     5.71414,6.91555,12.82850,-6.79940,8.5256)
      call settbl('Po',34.67260,0.70100,15.47330,3.55078,13.11380,
     *     9.55642,7.02588,47.00450,13.67700,0.0)
      call settbl('At',35.31630,0.68587,19.02110,3.97458,9.49887,
     *     11.38240,7.42518,45.47150,13.71080,0.0)
      call settbl('Rn',35.56310,0.66310,21.28160,4.06910,8.00370,
     *     14.04220,7.44330,44.24730,13.69050,0.0)
      call settbl('Fr',35.92990,0.64645,23.05470,4.17619,12.14390,
     *     23.10520,2.11253,150.64500,13.72470,0.0)
      call settbl('Ra',35.76300,0.61634,22.90640,3.87135,12.47390,
     *     19.98870,3.21097,142.32500,13.62110,10.0)
      call settbl('Ra2+',35.21500,0.60491,21.67000,3.57670,7.91342,
     *     12.60100,7.65078,29.84360,13.54310,10.0)
      call settbl('Ac',35.65970,0.58909,23.10320,3.65155,12.59770,
     *     18.59900,4.08655,117.02000,13.52660,0.0)
      call settbl('Ac3+',35.17360,0.57969,22.11120,3.41437,8.19216,
     *     12.91870,7.05545,25.94430,13.46370,0.0)
      call settbl('Th',35.56450,0.56336,23.42190,3.46204,12.74730,
     *     17.83090,4.80703,99.17220,13.43140,9.84)
      call settbl('Th4+',35.10070,0.55505,22.44180,3.24498,9.78554,
     *     13.46610,5.29444,23.95330,13.37600,9.84)
      call settbl('Pa',35.88470,0.54775,23.29480,3.41519,14.18910,
     *     16.92350,4.17287,105.25100,13.42870,9.1)
      call settbl('U',36.02280,0.52930,23.41280,3.32530,14.94910,
     *     16.09270,4.18800,100.61300,13.39660,8.417)
      call settbl('U3+',35.57470,0.52048,22.52590,3.12293,12.21650,
     *     12.71480,5.37073,26.33940,13.30920,8.417)
      call settbl('U4+',35.37150,0.51660,22.53260,3.05053,12.02910,
     *     12.57230,4.79840,23.45820,13.26710,8.417)
      call settbl('U6+',34.85090,0.50708,22.75840,2.89030,14.00990,
     *     13.17670,1.21457,25.20170,13.16650,8.417)
      call settbl('Np',36.18740,0.51193,23.59640,3.25396,15.64020,
     *     15.36220,4.18550,97.49080,13.35730,10.55)
      call settbl('Np3+',35.70740,0.50232,22.61300,3.03807,12.98980,
     *     12.14490,5.43227,25.49280,13.25440,10.55)
      call settbl('Np4+',35.51030,0.49863,22.57870,2.96627,12.77660,
     *     11.94840,4.92159,22.75020,13.21160,10.55)
      call settbl('Np6+',35.01360,0.48981,22.72860,2.81099,14.38840,
     *     12.33000,1.75669,22.65810,13.11300,10.55)
      call settbl('Pu',36.52540,0.49938,23.80830,3.26371,16.77070,
     *     14.94550,3.47947,105.98000,13.38120,14.1)
      call settbl('Pu3+',35.84000,0.48494,22.71690,2.96118,13.58070,
     *     11.53310,5.66016,24.39920,13.19910,14.1)
      call settbl('Pu4+',35.64930,0.48142,22.64600,2.89020,13.35950,
     *     11.31600,5.18831,21.83010,13.15550,14.1)
      call settbl('Pu6+',35.17360,0.47320,22.71810,2.73848,14.76350,
     *     11.55300,2.28678,20.93030,13.05820,14.1)
      call settbl('Am',36.67060,0.48363,24.09920,3.20647,17.34150,
     *     14.31360,3.49331,102.27300,13.35920,8.3)
      call settbl('Cm',36.64880,0.46515,24.40960,3.08997,17.39900,
     *     13.43460,4.21665,88.48340,13.28870,9.5)
      call settbl('Bk',36.78810,0.45102,24.77360,3.04619,17.89190,
     *     12.89460,4.23284,86.00300,13.27540,0.0)
      call settbl('Cf',36.91850,0.43753,25.19950,3.00775,18.33170,
     *     12.40440,4.24391,83.78810,13.26740,0.0)
      return
      end

      function asf(yb,jk)  ! atomic scattering factor for each atom
      parameter (natom=5)
      common /asft/ ne,ascf(35,20),df1(20),df2(20),iand,iel(3,natom)
      i=yb/0.05+1.0
      if(i.ge.29) i=(yb-1.4)/0.1+29.0
      if(i.ge.35) go to 20
      j=i+1
      if(i.ge.29) go to 21
      ai=float(i-1)*0.05
      asf=ascf(i,jk)+(ascf(j,jk)-ascf(i,jk))*(yb-ai)/0.05+df1(jk)
      return
 21   ai=float(i-29)*0.1+1.4
      asf=ascf(i,jk)+(ascf(j,jk)-ascf(i,jk))*(yb-ai)/0.1+df1(jk)
      return
 20   continue
      asf=0.0
      return
      end

      function dwfact(yb,yi)    ! Debye Waller factor
      common /asftb/ be
      dwfact=exp(-be*yb)
c     write(6,*) 'yb,dwfact=',yb,dwfact
      return
      end

      function asf0(yb,yi)      ! average atomic scattering factor for all atoms
      common /asft0/ascf0(35)   ! average scattering factors
      common /asftb/ be
      i=yb/0.05+1.0
      if(i.ge.29) i=(yb-1.4)/0.1+29.0
      if(i.ge.35) go to 20
      j=i+1
      if(i.ge.29) go to 21
      ai=float(i-1)*0.05
      asf0=ascf0(i)+(ascf0(j)-ascf0(i))*(yb-ai)/0.05
      asf0=asf0*exp(-be*yb)
      return
 21   ai=float(i-29)*0.1+1.4
      asf0=ascf0(i)+(ascf0(j)-ascf0(i))*(yb-ai)/0.1
      asf0=asf0*exp(-be*yb)
      return
 20   continue
      asf0=0.0
      return
      end

      subroutine getne(ned)
      parameter (natom=5)
      common /asft/ ne,ascf(35,20),df1(20),df2(20),iand,iel(3,natom)
      ned=ne
      return
      end

      subroutine rdasf()
      common /cwflag/ iwflag(6)
      parameter (natom=5)
      common /asft/ ne,ascf(35,20),df1(20),df2(20),iand,iel(3,natom)
      common /asft0/ascf0(35)  ! average scattering factors
      real a(4,20),b(4,20),c(20)
      common /cpnf1/ atp(10),atp0(10)
      common /asftb/ be
      character*80 dummy
      character*5 atm
      character xorn
      integer getimode
      write(6,*) 'enter rdasf'
      call getinp(inpu)
      call getout(iout)

      imode=getimode()
      write(6,*) 'imode in rdasf=',imode
      if(imode.eq.0) then
         call freadi('ne=',1,ne,1)
         call freadrv('atp=',ne,atp,0)
         call freadr('be=',1,be,0)
         call genasf()          ! generate atomic scattering factor tables
      else
         call freadr('be=',1,be,1)
         return
      end if

      total=0
      do i=1,ne
         total=total+atp(i)
      end do
      write(6,*) 'normalized atp'
      do i=1,ne
         atp(i)=atp(i)/total   ! normalization
         write(6,*) 'atp=',atp(i)
      end do

      do ik=1,ne
         atm(1:5)='     '
         read(5,*) atm
         write(6,*) 'atm=',atm
         call gettbl(atm,a(1,ik),b(1,ik),c(ik))
      end do
c     set scattering factor table for X-ray
      do ik=1,ne
         do i=1,35
            sint=((i-1)*0.05)**2
            ascf(i,ik)=c(ik)
            do j=1,4
               ascf(i,ik)=ascf(i,ik)+a(j,ik)*exp(-b(j,ik)*sint)
            end do
         end do
      end do
      call zerovr(ascf0,35)
      do ik=1,ne
         do i=1,35
            ascf0(i)=ascf0(i)+ascf(i,ik)*atp(ik)  ! average scattering factor
         end do
      end do
      return
      end

      subroutine rdlc()
      call getsys(isys)
      if(isys.eq.0) then
         call rdlc0()      ! for crystals
      else
         call rdlc1()      ! for quasicrystals
      end if
      return
      end

      subroutine rdlc1()
      call getsys(isys)
      call freadr('a=',1,al,1)
      if(isys.ge.3) then
          call freadr('c=',1,cl,0)
      end if
      if(isys.le.2) then
          write(6,*) 'isys=',isys,' a=',al
      else 
          write(6,*) 'isys=',isys,' a=',al,' c=',cl
      end if
      call setac(al,cl)
      return
      end

      subroutine rdlc0()
      common /cwflag/ iwflag(6)
      common /ltccnt/ a,b,c,cosa,cosb,cosc,sina,sinb,sinc,
     *     as,bs,cs,cosas,cosbs,coscs,sinas,sinbs,sincs
      character*80 mater
      common /ctitle/ mater
      integer zmtrx(6*3)
      common /czmtrx/ zmtrx, nsub

      real ui(6,3),u1(3,3),u2(3,3)
      write(6,*) 'enter rdlc0'
      call getinp(inpu)
      call getout(iout)
c     read(inpu,99) mater
c     call freadc('title=',1,mater,1)
c     read(inpu,100) a,b,c,cosa,cosb,cosc
      call freadr('a=',1,a,1)
      call freadr('b=',1,b,0)
      call freadr('c=',1,c,0)
      call freadr('cosa=',1,cosa,0)
      call freadr('cosb=',1,cosb,0)
      call freadr('cosc=',1,cosc,0)
      if(iwflag(2).eq.1) then
         write(iout,'(a)') mater
      end if
      if(iwflag(3).eq.1) then
         write(iout,300) a,b,c,cosa,cosb,cosc
      end if
      sina=sqrt(1.0-cosa**2)
      sinb=sqrt(1.0-cosb**2)
      sinc=sqrt(1.0-cosc**2)
      call calltcs(a,b,c,cosa,cosb,cosc,as,bs,cs,cosas,cosbs,coscs)
      icase=1
      call calu1(icase,as,bs,cs,cosas,cosbs,coscs,u1)
      do i=1,3
         do j=1,3
            u2(i,j)=0
         end do
      end do
      call setu1(u1)
      call setu2(u2)
c     call setui(ui)
      call getgij(zmtrx,nsub)
      return
 100  format(6f12.6)
 300  format('a=',f9.4,1x,'b=',f9.4,1x,'c=',f9.4,1x,
     *     'cosa=',f9.5,1x,'cosb=',f9.5,1x,'cosc=',f9.5)
      end

      subroutine getgij(zmtrx,nsub)
      integer zmtrx(6,*)
c     get metric tensor for each substructure and
c     get wave vector of modulation wave for each substructure
      common /ltccnt/ a,b,c,cosa,cosb,cosc,sina,sinb,sinc,
     *     as,bs,cs,cosas,cosbs,coscs,sinas,sinbs,sincs
      common /mtrtns/ gabx(3,3),gaby(3,3),gab(3,3,3),gabn(3,3,3)
      common /mtrtno/ ax(3,3,3),ay(3,3,3)
      common /kwave/ y(3,3,3),vc(3)
      common /cdim/ idm,nde
      real bb(3),piv(3)
      integer ipiv(3),indx(3,2)
      real bx(3,6)
      data bx/1,0,0,
     *     0,1,0,
     *     0,0,1,
     *     0,0,0,
     *     0,0,0,
     *     0,0,0/

      sina=sqrt(1-cosa**2)
      sinb=sqrt(1-cosb**2)
      sinc=sqrt(1-cosc**2)

      gabx(1,1)=a*sqrt(1.0-cosa**2-cosb**2-cosc**2+2.0*cosa*cosb*cosc)/
     *     sina
      gabx(1,2)=0.0
      gabx(1,3)=0.0
      gabx(2,1)=a*(cosc-cosa*cosb)/sina
      gabx(2,2)=b*sina
      gabx(2,3)=0.0
      gabx(3,1)=a*cosb
      gabx(3,2)=b*cosa
      gabx(3,3)=c
      vc0=gabx(1,1)*gabx(2,2)*gabx(3,3)
      call mcpy3(gabx,gaby)
      call matinv(gaby,3,3,bb,0,det,ipiv,indx,piv)
c      call wtmtrxrf('gabx',gabx,3,'(3f10.4)')
c      call wtmtrxrf('gaby',gaby,3,'(3f10.4)')

      call yyy(bx(1,1))
      call yyy(bx(1,2))
      call yyy(bx(1,3))
      
      do n=1,nde
         do j=1,3
            bx(j,n+3)=y(j,n,1)
         end do
         call yyy(bx(1,n+3))
      end do
c     for k-th subsustem
c     ax(i,j,k) : i-th components of j-th unit vector in the direct space
c     in orthogonal coordinate system
c     ay(i,j,k) : i-th components of j-th unit vector in the reciprocal space
c     in orthogonal coordinate system
      do n=1,nsub
         do id=1,3
            i=zmtrx(id,n)
            ax(id,1,n)=bx(1,i)
            ax(id,2,n)=bx(2,i)
            ax(id,3,n)=bx(3,i)
            ay(id,1,n)=bx(1,i)
            ay(id,2,n)=bx(2,i)
            ay(id,3,n)=bx(3,i)
         end do
         call matinv(ax(1,1,n),3,3,bb,0,det,ipiv,indx,piv)
c     vc(n)=1/det(ax)
c     vc : unit cell volume for n-th substructure
         vc(n)=1/det
         do id=1,3
            do jd=1,3
               i=zmtrx(id,n)
               j=zmtrx(jd,n)
               gabn(id,jd,n)=vinp(bx(1,i),bx(1,j),3)
               gab(id,jd,n)=gabn(id,jd,n)
            end do
         end do
         call matinv(gab(1,1,n),3,3,bb,0,det,ipiv,indx,piv)
         
c     write(6,*) 'ax=',((ax(i,j,n),i=1,3),j=1,3)
c     write(6,*) 'ay=',((ay(i,j,n),i=1,3),j=1,3)
c     ax: a,b,c for n-th substructure (orthogonal system)
         an=vabs(ax(1,1,n),3)
         bn=vabs(ax(1,2,n),3)
         cn=vabs(ax(1,3,n),3)
         cosan=vinp(ax(1,2,n),ax(1,3,n),3)/(bn*cn)
         cosbn=vinp(ax(1,3,n),ax(1,1,n),3)/(cn*an)
         coscn=vinp(ax(1,1,n),ax(1,2,n),3)/(an*bn)
         if(nsub.gt.1) then
            write(6,*) 'substructure',n
            write(6,400) an,bn,cn,cosan,cosbn,coscn
         end if
         do kd=1,nde
            k=zmtrx(kd+3,n)
            y(1,kd,n)=vinp(ax(1,1,n),bx(1,k),3)
            y(2,kd,n)=vinp(ax(1,2,n),bx(1,k),3)
            y(3,kd,n)=vinp(ax(1,3,n),bx(1,k),3)
            if(nsub.gt.1) then
               write(6,'(a,3f10.5)') ' k=',(y(j,kd,n),j=1,3)
            end if
         end do
      end do
      as=sqrt(gabn(1,1,1))
      bs=sqrt(gabn(2,2,1))
      cs=sqrt(gabn(3,3,1))
      cosas=gabn(2,3,1)/(bs*cs)
      cosbs=gabn(1,3,1)/(as*cs)
      coscs=gabn(1,2,1)/(as*bs)
      sinas=sqrt(1.0-cosas**2)
      sinbs=sqrt(1.0-cosbs**2)
      sincs=sqrt(1.0-coscs**2)
      return
 400  format(' a=',f9.4,1x,'b=',f9.4,1x,'c=',f9.4,1x,
     *     'cosa=',f9.5,1x,'cosb=',f9.5,1x,'cosc=',f9.5)
      end

      subroutine calltcs(a,b,c,cosa,cosb,cosc,as,bs,cs,cosas,
     *     cosbs,coscs)
      real gabs(3,3),bd(3),piv(3)
      integer ip(3),indx(6)
      gabs(1,1)=a*a
      gabs(2,2)=b*b
      gabs(3,3)=c*c
      gabs(1,2)=a*b*cosc
      gabs(1,3)=a*c*cosb
      gabs(2,3)=b*c*cosa
      gabs(2,1)=gabs(1,2)
      gabs(3,1)=gabs(1,3)
      gabs(3,2)=gabs(2,3)
      call matinv(gabs,3,3,bd,0,det,ip,indx,piv) !  matrix inversion of gabs
      as=sqrt(gabs(1,1))
      bs=sqrt(gabs(2,2))
      cs=sqrt(gabs(3,3))
      cosas=gabs(2,3)/bs/cs
      cosbs=gabs(3,1)/cs/as
      coscs=gabs(1,2)/as/bs
      return
      end

      subroutine calu1(icase,as,bs,cs,cosas,cosbs,coscs,ud)
      real ud(3,3)              ! first 3x3 part of u matrix
      sinas=sqrt(1.-cosas**2)
      sinbs=sqrt(1.-cosbs**2)
      sincs=sqrt(1.-coscs**2)
      d=sqrt(1.0-cosas**2-cosbs**2-coscs**2+2.0*cosas*cosbs*coscs)
      if(icase.eq.1) then
         ud(1,1)=as
         ud(1,2)=0.0
         ud(1,3)=0.0
         ud(2,1)=bs*coscs
         ud(2,2)=bs*sincs
         ud(2,3)=0.0
         ud(3,1)=cs*cosbs
         ud(3,2)=cs*(cosas-cosbs*coscs)/sincs
         ud(3,3)=cs*d/sincs
      else if(icase.eq.2) then
         ud(1,1)=bs
         ud(1,2)=0.0
         ud(1,3)=0.0
         ud(2,1)=cs*cosas
         ud(2,2)=cs*sinas
         ud(2,3)=0.0
         ud(3,1)=as*coscs
         ud(3,2)=as*(cosbs-coscs*cosas)/sinas
         ud(3,3)=as*d/sinas
      else if(icase.eq.3) then
         ud(1,1)=cs
         ud(1,2)=0.0
         ud(1,3)=0.0
         ud(2,1)=as*cosbs
         ud(2,2)=as*sinbs
         ud(2,3)=0.0
         ud(3,1)=bs*cosas
         ud(3,2)=bs*(coscs-cosas*cosbs)/sinbs
         ud(3,3)=bs*d/sinbs
      else
         call quit('icase should be 1,2 or 3')
      end if
      return
      end

      subroutine getltcs(ad,bd,cd,cosad,cosbd,coscd)
      common /ltccnt/ a,b,c,cosa,cosb,cosc,sina,sinb,sinc,
     *     as,bs,cs,cosas,cosbs,coscs,sinas,sinbs,sincs
      ad=as
      bd=bs
      cd=cs
      cosad=cosas
      cosbd=cosbs
      coscd=coscs
      return
      end

      subroutine setu1(u1)
      real u1(3,3)
      common /cui/ ui(6,3)
      do i=1,3
         do j=1,3
            ui(i,j)=u1(i,j)
         end do
      end do
      return
      end

      subroutine setu2(u2)
      real u2(3,3)
      common /cui/ ui(6,3)
      do i=1,3
         do j=1,3
            ui(i+3,j)=u2(i,j)
         end do
      end do
      return
      end

      subroutine getu1(u1)
      real u1(3,3)
      common /cui/ ui(6,3)
      do i=1,3
         do j=1,3
            u1(i,j)=ui(i,j)
         end do
      end do
      return
      end

      subroutine getu2(u2)
      real u2(3,3)
      common /cui/ ui(6,3)
      do i=1,3
         do j=1,3
            u2(i,j)=ui(i+3,j)
         end do
      end do
      return
      end

      subroutine tounitcell(h)
      integer h(*)
      return
      end
      
      subroutine yyy(r)
      common /mtrtns/ gabx(3,3),gaby(3,3),gab(3,3,3),gabn(3,3,3)
      real rr(3),r(3)
      do i=1,3
         rr(i)=0.0
         do j=1,3
            rr(i)=rr(i)+r(j)*gaby(j,i)
         end do
      end do
      r(1)=rr(1)
      r(2)=rr(2)
      r(3)=rr(3)
      return
      end
      
