c     
c             generate an occupation domain in icosahedral phases
c               (superspace group p-5-3m is assumed)
c                   note the following assumption
c            structure has different decorations in ve
c            depending on positions of verteces in vi.
c            therefore the atom position of each atom
c            has to be the vector in ve.
c
c
c                         input data
c
c     (0) inpm iapprx
c         inpm=0 separatel file (standard input, flnm1 and flnm2)
c             =1 standard input only
c         iapprx 0 reshape of OD
c                1 not reshape  (use this)
c     (1) title    (72a1)
c     (2) a,eps1,eps2,eps3,isys,eps0,ists,mode,itst
c         a: lattice papameter
c         eps1: axis component for pentagonal or trigonal distortion
c         eps2: planar component for pentagonal or trigonal distortion
c         eps3: rotation from icosahedral to cubic
c         isys: 1,2 for icosahedral
c         eps0: eps for domain (default 1.0 E-6)
c         ists: 0 normal mode
c               1 test mode (xi0 is neglected)
c         mode: 0 independent ODs (occupation domains) are written
c               1 all ODs are written 
c               2 all ODs are are written in different sheest
c              -1 all independent ODs plus equivalent symmetry operators
c                 which are specified in (11)
c                
c         itst:  0: no test output 1-7 test output
c                7: output showing subdivision of a plane
c     (3) na, npod
c         na: number of ODs    (<=namax=50)
c         npod : number of predefined occupation domains  (<=namax=50)
c
c     ******** if inpm=0 skip (4) ******
c     (4) flnm1
c         flnm1 : filename which gives the following items
c     
c     ******** if inpm=1 skip (01)-(07) ******
c##########################################################################
c     (01) nsymo,icent,brv
c          nsymo: number of generators in symmetry operations
c          icent: give 1 for centrosymmetric
c          brv  : 'p','i','f' for primitive, body-centered,face-centered
c             's' for face-centered with input data based on the primitive cell a0=a/2
c     (02) 'symmetry operator'        header  and
c     (03) x,y,z,t,u,v etc  (symmetry operator in I.T. format)
c          ***** repeat (03) nsymo times
c     (04) n, nej0, itype, str
c         n    : serial number for the guide to the eye
c         nej0 :number of edge vectors for predefined occupation domain (<=nemax=5)
c         itype  :  1 domain with icosahedral, degagonal, octagonal or dodecagonal symmetry
c                   2 domain with other symmetry
c                  -1 domain with icosahedral, degagonal or octagonal symmetry for test
c                  -2 domain with other symmetry for test
c                    negative itype is faster than positive one but
c                    two calculations should give the same result (please test)
c         str  : string for comments
c     (05) ej  : edge vectors of their internal components or principal axes
c         ej(1)-ej(6)

c     (05) ej  : edge vectors of their internal components or principal axes
c         ej(1)-ej(6)
c               repeat (5) ne times
c     (06) nth, (mej(1),mej(2),,),... nth times
c          nth: number of tetrahedra
c          mej: a triplet of vector numbers specifying a tetrahedron
c     (07) iaslct(1-(nsymo+icent*nsymo))
c           (60i1) 1:selected, 0:skipped **na times
c       ******** repeat (04)-(07) npod times
c
c ###########################################################################
c
c     ******** if inpm=0 skip (5) ******
c     
c     (5) flnm2
c         flnm2 : filename which gives the following items
c
c     ******** if inpm=1 skip (01)-(08) ******
c ############################################################################
c     (01) ia,symb,ien,ish,ne,be,bi,ramin,ramax,p,s1,s2 (*)
c         ia: OD number
c             OD with the same ia is regarded as the fragment of an
c                OD specified by ia.  This enables to construct a complex
c                OD with several fragments or several holes
c       ******note that each OD with the same ia should have the same 
c             position in ve but may have a different position in vi
c         symb: site symbol
c         ien: color number
c         ish: 0: tetrahedron,
c              1: triacontahedron
c              2: sphere
c           if ish=2 then r=1 means 1.4325 (ca)
c         ne : number of edge vectors      (ne <= 20)
c              consequtive three vectors are regarded as the edge vectors 
c              of a tetrahedron defining the occupation domain
c         be: isotropic temperature factor in external space
c         bi: isotropic temperature factor in internal space
c         ramin :  scale for unoccupied tetrahedron or rhombohedron
c         ramax :  scale for occupied tetrahedron or rhombohedron
c            if ish=2 ramax=1 means shere with radius of 1.4325 (ca)
c            (radius of the shere equivalent to unit triacontahedron
c                    creating the 3d penrose pattern)
c         p    : occupation probability
c                negative p means that a hole in a occupation domain
c         s1    : percentage of second component
c         s2    : percentage of third component
c         (** one site is assumed to be occupied by two atoms**)
c          ***** ish=-1 to use truncated triacontahedron *****
c              ra or rb=1 corresponds to the triacontahedron with
c                   unit edge length
c     (02)  x(1)-x(6): coordinate of atom in 5D
c     (03)  xe1(1)-xe1(6),u1 shift vector in ve and magnitude  (6D coordinates)
c     (04)  xe2(1)-xe2(6),u2 shift vector in ve and magnitude  (6D coordinates)
c     (05)  xe3(1)-xe3(6),u3 shift vector in ve and magnitude  (6D coordinates)
c     (06)  xi(1)-xi(6),v shift vector in vi and magnitude  (6D coordinates)
c**************************************************************************
c            note x(1)-x(6) are set to be zeto in the program !!
c**************************************************************************
c        ******* if(ish > 0) skip (07) ****
c     (07) ej
c          ej:  edge vectors of two triangles in the 6D coordinate system
c               repeat (07) ne times
c                ***************notice****************
c               the internal components of consecutive ej are regarded as
c               the consecutive edge vectors of the polyhedron defining
c               the occupation domain
c                ***************notice***************
c
c     (08) nth, (mej(1),mej(2),,),... nth times
c          nth: number of tetrahedra
c          mej: a triplet of vector numbers specifying a tetrahedron
c       ******** repeat (01)-(08) na times
c ############################################################################
c
c     (6) ldev,ng,amag
c          ldev : 1 B&W 2 color
c          ng  number of figures
c          amag magnification
c     (7)  jcol
c                 jcol : brightness (0 <=jcol <=10)
c                 if jcol=-1 jcol is proportional to 1+cos(t)
c                    where t is the angle between the plane normal and the
c                    z axis
c                 if jcol=-2 jcol is proportional to 1+cos(u)
c                    where u is the angle between the plane normal and the
c                    (112) axis
c                 if jcol=-3 jcol is proportional to 1+cos(v)
c                    where v is the angle between the plane normal and the
c                    (111) axis
c     (8)   iornt : orientation 1: portrait 2: landscape
c     (9)   imode, wrl_file
c             imode : 0 shading 1 wire-frame & VRML
c              *****if imode=0 wrl_file is not necessary***
c               wrl_file: file_name of wrl_file
c     (10) jaslct (80i1)
c            if j-th column is 1, 2, or 3, j-th occupation domain is drawn
c              1: with shading
c              2: nonshading
c              3: shading all faces of a tetrahedron
c
c              0: j-th occupation domain is not drawn
c
c     **********if mode < 0  (11) is necessary otherwise skip (11)**********
c     (11) nt
c          nt: number of symetry operators for each selected OD  (<=12)
c                   **** nt <=12 ****
c     (11') nr, tt(1-6)
c          nr: symmetry operator number  (its rotational part is used)
c          tt: translation vector
c                nr,tt for each atom
c                   **** repeat (11') nt times ****
c     ****************repeat (11) na times*********************
c     (12) rmag,icm,icase
c          rmag:  magnification for ramin, ramax of ODs
c          icm : 0 give orientation matrix in (16) determined by VRML
c                1 give rotation angle around x, y or z axis in (16)
c                2 use unir matrix for orm in (16) This is necessary for
c                  determining orientation matrix by VRML
c          icase: 1 projection along 5-fold axis
c                 2 projection along 3-fold axis
c                 3 projection along 2-fold axis             
c     (13) shiftx,shifty,inum
c          shiftx: shift before plot in cm
c          shifty: shift before plot in cm
c          inum  : 0 no number 1: occupation domain number is ploted
c      **** if imode = 1 skip (14)
c     (14) icolor(i),i=1,5        (only for imode=0)
c          if imode = 0 skip (15)
c      (15) rgbtbl                 (only for imode=1)    
c           rgbtbl: three integers for R G B color (0<= R, G, B <=255)
c            **** repeat (15) 5 times for 5 atom colors
c       ****** if icm=2 skip (16) ******
c     (16) orm  (transposed)
c               (orm(1,1),orm(1,2),...,orm(3,3)
c              for orientation matrix (icm=0)
c           or a series of cmnd  cmnd='rotn', ended with cmd='end' (icm=1)
c         ***********repeat (10-16) ng times**************
c          
      include 'impr8.inc'
      common /cwflag/ iwflag(6)
      parameter (ngm=6)
      parameter (nl2=70)
      parameter (imax=30)
      include 'qcstrc.inc'
      common /ccolor/ icolor(5)
      common /ccoltbl/ coltbl(3,5)
      real amag,aa,rmag
      common /camag/ amag,aa,rmag
      real ram,rbm,dx,dy
      common /clmt1/ ram,rbm,dx,dy
      real shiftx,shifty
      common /cdevi/ ldev,linw,klin,jcol,inum
      common /cdevr/ shiftx,shifty
      real x0(6,namax), x1(6,namax)
      real xi(6)
      real ej

      common /cisym/ ieqs,ists,mode
      common /cskip/ iskip,ltype
      real qg
      common /cfatm/ nasym(2),nnsym(60,2),qg(6,60,2)
      integer pcol
      common /cpcol/ pcol

      real al,am,cl,eps1,eps2,eps3,eps4
      common /cgetq2/ al,am,cl,eps1,eps2,eps3,eps4

      real eps0
      common /ceps0/ eps0
      real prj(3,3)  ! viewing transformation matrix
      real c0m(3,3)

      integer isk(0:120,namax),irs(0:120,namax),iski(0:120,namax),hc(6)
      real ni(0:6,4,nemax,namax),mpl(namax)
      integer isk0(0:120,namax),irs0(0:120,namax)

      integer isk1(0:120,namax),irs1(0:120,namax)
      real mpl1(namax)

      character brv
      real shifto(6),abszc
c      common /toatm/ ni,mpl,isk,irs,iski,hc,na,abszc,shifto,brv
      common /toatmi/ ni,mpl,isk,irs,iski,hc,na,brv
      common /toatmr/ abszc,shifto

      real tt
      common /cislct/ iaslct(120,namax),jaslct(namax),nt(namax),
     *     nr(120,namax),tt(6,120,namax)
      common /cislct0/ iaslct0(120,namax)

      integer ind(2*ngm)
      real xy(3),raplt(4)
      real dt(nmax),ds(nmax),xs(3,nmax)
      real re1(nmax),ri1(nmax),dt1(nmax),ds1(nmax),xs1(3,nmax)
      integer hs(6,nmax),ms(4,nmax)
      integer hs1(6,nmax),ms1(4,nmax)
      integer ns(nmax)
      character(4) cmnd
      real xd(6)
      integer h(6)

      real qb(6),zc(6)
      character(72) title,flnm1,flnm2,flnm3
      real shftx,shfty
      integer iskd(0:120,namax),irsd(0:120,namax)
      character dummy
      real xg(6,2)
      real td(6)
      real vinp
      real vabs,absh,absn1
      real eps,ph
      integer irgb(3)
      real rgbtbl(3,5)
      common /crgbtbl/ rgbtbl
      integer getish
      data eps/0.001/
      data td/6*0.0/
      data ind/2,1,4,3,6,5,8,7,10,9,12,11/
      data xg
     */ 0.5,     -0.223607,-0.223607,-0.223607,-0.223607,-0.223607,
     *  0.723607,-0.723607, 0.0     ,-0.447214,-0.447214,0.0    /

c     initialize
c     call trnsp3(om1)
      call setinp(5)
      call setout(6)

      call freadi('inpm=',1,inpm,1)
c     call freadi('iapprx=',1,iapprx,0)
      iapprx=1
      call setapprx(iapprx)
      write(6,'(2(a,i3))') 'inpm=',inpm,' iapprx=',iapprx
c     read(5,'(a)') title
      call freadc('title=',1,title,1)
      write(6,'(a)') title
      call rdlc()  ! read a,c eps0,..,eps6
      aa=al
c     isys=2
c     check orthogonality
c      do l=1,2
c         call setsys(l)
c         call setd()
c      end do

c     call setsys(isys)
      call setd()       ! lattice vector
      call setdei()     ! phason and phonon distortion
      if(iapprx.eq.0) then
         call setsmtrm(1)       ! for reshape
         call intreshape()
      end if
      call setprjop0()  ! calculate projection operator under no phason
      call setprjop()   ! calculate projection operator under phason

c     read(5,'(i3)') na
      call freadi('na=',1,na,1)
      call freadi('npod=',1,npod,0)
      write(6,*) 'na=',na,' npod=',npod
      iwflag(3)=1

      ram=0
      rbm=+eps

      if(inpm.eq.0) then
         call freadc('flnm1=',1,flnm1,1)
         lfn1=istrln(flnm1)
         call freadc('flnm2=',1,flnm2,1)
         lfn2=istrln(flnm2)

         write(6,*) 'flnm1=',flnm1(1:lfn1)
         write(6,*) 'flnm2=',flnm2(1:lfn2)
         
         call openfile(7,flnm1,'OLD')
         call setinp(7)
      end if
      call setout(6)
      call rdsymo(na,brv)
      call setbrv(brv)
      call setmpltbl()    ! set multiplication table for symm. op.
      call rdpod(npod)
      call setnpod(npod)
      close(unit=7)

      if(inpm.eq.0) then
         call openfile(8,flnm2,'OLD')
         call setinp(8)
      end if

      if(brv.eq.'p') then
c     am=1.0
         am=al
      else if(brv.eq.'f'.or.brv.eq.'s') then
c     am=2.0
         am=al
      else
         call quit('invalid brv')
      end if

      call setout(6)
      call rdap(na) 
      close(unit=8)

      if(brv.eq.'s') then
         call xtoxs(na)         
      end if

      do i=1,na
         call getx0(i,x0(1,i))
         write(6,'(a,6f10.5)') 'x0=',(x0(j,i),j=1,6)
      end do

      call getsym
      call sitesy(na,mpl,x0,isk0,irs0)  ! sytesymmetry of large domain center

c     for drawing
      do i=1,na
         call getxi(i,xi)
         do j=1,6
            x0(j,i)=x0(j,i)+xi(j)*getv0(i)
         end do
         write(6,'(a,6f10.6)')'x=',(x0(j,i),j=1,6)
      end do

      call sitesy(na,mpl,x0,isk,irs)   ! symmetry of small domain center
      call setinp(5)
      call setout(6)

      idm=6
      nde=3

c     select symmetry operator leaving x0 invaliant
      do i=1,na
         n=0
         do j=1,isk(0,i)
            do k=1,irs0(0,i)
               if(isk(j,i).eq.irs0(k,i)) then
                  n=n+1
                  isk(n,i)=isk(j,i)
                  go to 600
               end if
            end do
 600        continue
         end do
         isk(0,i)=n
      end do

c     call invisk(na,isk,iski)

      call getnsymo(nsymo,icent)
      write(6,*) '******nsymo,icent=',nsymo,icent
      do j=1,na
         if(getish(j).eq.0) then
         else if(getish(j).gt.0) then
            call vicpy(iaslct0(1,getish(j)),iaslct(1,j),nsymo*(1+icent))
         end if
      end do

c     replace site symmetry operator with fictitious one
      write(6,*) 'pseudo-site symmetry operator'
      do n=1,na
         if(getish(n).gt.0) then
            isrd=0
            do i=1,nsymo*(1+icent)
               if(iaslct(i,n).eq.1) then
                  isrd=isrd+1
                  irs(isrd,n)=i
               end if
            end do
            irs(0,n)=isrd
         end if
         write(6,'(a,i3,20i4/(8x,20i4))')
     *        'atom=',n,(irs(j,n),j=1,irs(0,n))
      end do

      do i=1,2
         nasym(i)=isk(0,i)
         do jd=1,isk(0,i)
            j=isk(jd,i)
            call getrx1(xg(1,i),xd,td,j,0)
c            call getqe(xd,qg(1,jd,i)) ! distorted
            call getqed(xd,qg(1,jd,i)) ! distorted qg
         end do
      end do

c     for volume
      do i=1,na
         call getxi(i,xi)
         do j=1,6
            x1(j,i)=x0(j,i)+xi(j)*getv0(i)
         end do
c      write(6,*) 'v0,xi=',getv0(i),(xi(j),j=1,6)
      end do
      call sitesy(na,mpl1,x1,isk1,irs1)   ! symmetry of small domain
      call vivoli(na,mpl1,isk1,irs)

      pcol=1
      icol=1
c     read(5,'(6i3)') ldev,ngr,amag
      job=0
      call freadi('ldev=',1,ldev,1)
      call freadi('ng=',1,ngr,0)
      call freadr('amag=',1,amag,0)

      call freadi('jcol=',1,jcol,1)
      call freadi('iornt=',1,iornt,1)

      call freadi('imode=',1,imode,1)

c     lfn3=istrln(flnm3)
      if(imode.eq.1) then
         call freadc('wrl_file=',1,flnm3,0)
         lfn3=istrln(flnm3)
         call openfile(3,flnm3,'UNKNOWN')
      end if
      call setimd(imode)

      if(ngr.eq.0) ngr=1         
      call orient(iornt)
      x00=0.
      y00=0.

      call plots(x00,y00,ldev)
      do nng=1,ngr
         read(5,'(80i1)') (jaslct(j),j=1,na)
         write(6,'(80i1)') (jaslct(j),j=1,na)
         call resetja(jaslct,na)  ! set 2 for jaslct if occupancy is zero
         if(mode .lt. 0) then
            do i=1,na
               if(jaslct(i).gt.0) then
                  call freadi('nt=',1,nt(i),1)
                  do j=1,nt(i)
                     call freadi('nr=',1,nr(j,i),1)
                     call freadrv('tt=',6,tt(1,j,i),0)
                     write(6,'(3(a,i3),a,6f10.5)') '#',i,
     *                    ' nt=',nt(i),' nr=',nr(j,i),
     *                    ' tt=',(tt(k,j,i),k=1,6)
                  end do
               else
                  nt(i)=0
               end if
            end do
         end if
c     read(5,'(7f8.5)') rmag,icm,icase
         call freadr('rmag=',1,rmag,1)
         call freadi('icm=',1,icm,0)
         call freadi('icase=',1,icase,0)
         write(6,'(a,f10.5,2(a,i3))')
     *        'rmag=',rmag,' icm=',icm,' icase=',icase
         call setprjm0(icase)   ! set projection matrix under phason strain
c     read(5,'(7f8.3)') shiftx,shifty,inum
         call freadr('shiftx=',1,shftx,1)
         call freadr('shifty=',1,shfty,0)
         call freadi('inum=',1,inum,0)
         write(6,*) 'shiftx=',shiftx,' shifty=',shifty,' inum=',inum
         shiftx=shftx
         shifty=shfty

         if(imode.eq.0) then
            call freadiv('icolor=',5,icolor,1)
            write(6,*) 'icolor=',(icolor(j),j=1,5)
         else if(imode.eq.1) then
            do i=1,5
               call freadiv('rgbtbl=',3,irgb,1)
               write(6,'(a,3i5)') 'rgbtbl=',irgb
               rgbtbl(1,i)=irgb(1)/255.0
               rgbtbl(2,i)=irgb(2)/255.0
               rgbtbl(3,i)=irgb(3)/255.0
c              write(6,'(a,3f10.5)') 'rgbtbl=',(rgbtbl(j,i),j=1,3)
            end do
         end if
         call setormt(icm)
         call getprjm(prj)
         call wtmtrxrf('orientation matrix',prj,3,'(3f10.5)')         

         if(amag.eq.0) amag=1.
         if(rmag.eq.0) rmag=1.

         call setamgn(amag)

         if(eps0.eq.0) eps0=1.e-6*amag
c        if(no1.eq.0) no1=1
         no1=1

         write(6,'(2a)') 'ldev,jcol'
         write(6,'(9i3)') ldev,jcol
         write(6,'(a,f8.5)') 'amag=',amag
         write(6,'(a,2f8.3,i3)') 'shiftx,shifty,inum=',shiftx,
     *        shifty,inum
         call getimd(imode)
         if(imode.eq.0) then
            write(6,'(a,5i3)') 'icolor=',(icolor(i),i=1,5)
         end if

         call zerov(shifto,6)
         iixx=0

         call setprjop()        ! calculate projection operator
         write(6,*) 'before getatm'
         call getatm(1,no1)     ! calculate occupation domains
         write(6,*) 'no1=',no1

         nn=0
         do i=1,no1
            ii=ns(i)
            call vcpyi(hs(1,ii),h,6)
            do j=1,4
               if(ms(j,ii).gt.0) then
                  icol=j
                  go to 1512
               end if
            end do
 1512       continue
            xy(1)=xs(1,ii)*amag*al
            xy(2)=xs(2,ii)*amag*al
            xy(3)=xs(3,ii)*amag*al
c     write(6,*) 'i,ii,ms,xy',i,ii,ms(1,ii),xy
            sx=0.3
            sy=0.3
            ht=0.2
            rno=i
         end do
      end do
      call plote('end of plot')
      if(imode.eq.1) then
         close(unit=3)
      end if
      if(ldev.eq.1.or.ldev.eq.2) call getzap()
      stop
      end

      subroutine setormt(icm)
      real c0m(3,3),prj(3,3)
      character(4) cmnd
c     write(6,*) 'icm=',icm
      if(icm.eq.0) then         ! for orientation matrix from VRML
         call freadrv('orm=',9,c0m,1)
         call trnsp3(orm)       ! added 
         call setprjm1(c0m)     ! c0m as the projection matrix
c     write(6,*) 'after setprjm1'
      else if(icm.eq.1) then    ! plot 10 commands
         call setprjm00()       ! set unit matrix for prj
 52      continue
         call freadc('cmnd=',1,cmnd,1)
c     write(6,*) 'cmnd=',cmnd
         if(cmnd.eq.'rotn') then
            read(5,*) ix,ph
            call setprjm2(ix,ph)
            go to 52
         else if(cmnd.eq.'end') then
            ! nothing to do
         else
            call quit('invalid cmnd')
         end if
      else if(icm.eq.2) then
         call getprjm0(prj)
         call setprjm1(prj)
      else
         write(6,*) 'invalid icm, icm=',icm
      end if
      return
      end

      subroutine getsym()
      include 'impr8.inc'
      integer r
      real t
      common /sym/ r(6,6,60),t(6,60),nsymo,icent

      real r1,t1,axis,angl
      common /sym1/ r1(6,6,60),t1(6,60),axis(3,60),angl(60),irot(60)
      real dr,dq,de,di
      common /cgetq1/ dr(6,6),dq(6,6),de(6,6),di(6,6)
      real al,am,cl,eps1,eps2,eps3,eps4
      common /cgetq2/ al,am,cl,eps1,eps2,eps3,eps4

      real rd(6,6),re(6,6),dd(6,6),d0(6,6)
      real ri(3,3),rj(3,3),piv(3),b(3),det
      integer ipv(3),ind(2*3),imps(60)

      call mtcpyr(dr,d0,6)
      call mtcpyr(dq,dd,6)
      call ordr(r,imps,nsymo,6)
      do ns=1,nsymo
         call mtcpyir(r(1,1,ns),rd,6)
         call mtmpl(rd,dd,re)
         call mtmpl(d0,re,r1(1,1,ns))
c         call wtmtrxrf(' ',r1(1,1,ns),6,'(6f7.3)')
         do i=1,3
            do j=1,3
               ri(i,j)=r1(i+3,j+3,ns)
               rj(i,j)=r1(i+3,j+3,ns)
            end do
         end do

         call matinv(ri,3,3,b,0,det,ipv,ind,piv)
         if(det.lt.0) then
            irot(ns)=-1
         else
            irot(ns)=1
         end if
         call uniqax(rj,irot(ns),imps(ns),axis(1,ns),angl(ns))
c     write(6,*) 'irot=',irot(ns),'imps=',imps(ns),'axis=',
c     *         (axis(i,ns),i=1,3),'angl=',angl(ns)
      end do
      return
      end

      subroutine uniqax(rj,irot,imps,axis,angl)
      include 'impr8.inc'
      real rj(3,3),axis(3),angl

      real t0(3),t1(3),t2(3),t3(3),tt(3),td(3)
      real vinp,abst,sign
      real vnp

      data t0/1.,1.414,3.1415/

      if(imps.eq.1) then
c      for identity operator axis is arbitrary
          axis(1)=1
          axis(2)=0
          axis(3)=0
          angl=0
          return
      end if
      call vcpyr(t0,td,3)
      axis(1)=td(1)/imps
      axis(2)=td(2)/imps
      axis(3)=td(3)/imps
      do ns=1,imps-1
         call mtmplmnor(rj,td,tt,3,3,1)
         td(1)=tt(1)*irot
         td(2)=tt(2)*irot
         td(3)=tt(3)*irot
         axis(1)=axis(1)+td(1)/imps
         axis(2)=axis(2)+td(2)/imps
         axis(3)=axis(3)+td(3)/imps
      end do
      absa=sqrt(axis(1)**2+axis(2)**2+axis(3)**2)
      axis(1)=axis(1)/absa
      axis(2)=axis(2)/absa
      axis(3)=axis(3)/absa
c     t1=t0-ax.t0*ax  is normal to ax
      do i=1,3
         t1(i)=t0(i)-(axis(1)*t0(1)+axis(2)*t0(2)+
     *        axis(3)*t0(3))*axis(i)
      end do
      abst=sqrt(t1(1)**2+t1(2)**2+t1(3)**2)
      t1(1)=t1(1)/abst
      t1(2)=t1(2)/abst
      t1(3)=t1(3)/abst
      call mtmplmnor(rj,t0,tt,3,3,1)      
      td(1)=tt(1)*irot
      td(2)=tt(2)*irot
      td(3)=tt(3)*irot
c     t2=td-ax.td*ax  is normal to ax
      do i=1,3
         t2(i)=td(i)-(axis(1)*td(1)+axis(2)*td(2)+
     *        axis(3)*td(3))*axis(i)
      end do
      abst=sqrt(t2(1)**2+t2(2)**2+t2(3)**2)
      t2(1)=t2(1)/abst
      t2(2)=t2(2)/abst
      t2(3)=t2(3)/abst
      vnp=vinp(t1,t2,3)
      if(vnp.gt.0) then
         one=1.d0
         vnp=min(one,vnp)
      else
         oned=-1.d0
         vnp=max(oned,vnp)
      end if
      t3(1)=t1(2)*t2(3)-t1(3)*t2(2)
      t3(2)=t1(3)*t2(1)-t1(1)*t2(3)
      t3(3)=t1(1)*t2(2)-t1(2)*t2(1)
      sign=t3(1)*axis(1)+t3(2)*axis(2)+t3(3)*axis(3)
      if(sign.ge.0) then
          angl= acos(vnp)*180/3.141592
      else
          angl=-acos(vnp)*180/3.141592
      end if
      return
      end

      subroutine setshadtyp(n)
      include 'qcstrc.inc'
      real tt
      common /cislct/ iaslct(120,namax),jaslct(namax),nt(namax),
     *     nr(120,namax),tt(6,120,namax)
      if(jaslct(n).eq.1) then
         call setshd(1)         ! shading
      else if(jaslct(n).eq.2) then
         call setshd(0)         ! nonshading
      else if(jaslct(n).eq.3) then
         call setshd(1)         ! shading all faces of a tetrahedron
      else
         call quit('invalid jaslct')
      end if
      return
      end

      integer function getmtend(n)
      common /cisym/ ieqs,ists,mode
      include 'qcstrc.inc'
      real ni(0:6,4,nemax,namax), mpl(namax)
      integer isk(0:120,namax),irs(0:120,namax),iski(0:120,namax),hc(6)
      real shifto(6),abszc
      character brv
c      common /toatm/ ni,mpl,isk,irs,iski,hc,na,abszc,shifto,brv
      common /toatmi/ ni,mpl,isk,irs,iski,hc,na,brv
      common /toatmr/ abszc,shifto
      real tt
      common /cislct/ iaslct(120,namax),jaslct(namax),nt(namax),
     *     nr(120,namax),tt(6,120,namax)
      common /cislct0/ iaslct0(120,namax)
      if(mode.eq.0) then
         mtend=1                ! write independent occupation domain only
      else if(mode.gt.0) then
         mtend=isk(0,n)         ! number of site symmetry operations
      else
         mtend=nt(n)
      end if
      getmtend=mtend
      return
      end

      integer function geticol(n,jaslct)  ! get icolor dependeng on jaslct
      common /ccolor/ icolor(5)
      integer getien
      if(jaslct.eq.2) then
         geticol=0
      else
         geticol=icolor(getien(n))
      end if
      return
      end

      subroutine getatm(ipsig,no)  ! calculate occupation domains
      include 'impr8.inc'
      include 'qcstrc.inc'
      parameter (ngm=6)
      parameter (ng=6)
      parameter (nl2=70)
      parameter (imax=30)
      real shifto(6),abszc

      real ni(0:6,4,nemax,namax), mpl(namax)
      integer isk(0:120,namax),irs(0:120,namax),iski(0:120,namax),hc(6)
      character brv
c      common /toatm/ ni,mpl,isk,irs,iski,hc,na,abszc,shifto,brv
      common /toatmi/ ni,mpl,isk,irs,iski,hc,na,brv
      common /toatmr/ abszc,shifto

      real tt
      common /cislct/ iaslct(120,namax),jaslct(namax),nt(namax),
     *     nr(120,namax),tt(6,120,namax)
      common /cislct0/ iaslct0(120,namax)

      real amag,aa,rmag
      common /camag/ amag,aa,rmag

      real xiv(6)
      real ram,rbm,dx,dy
      common /clmt1/ ram,rbm,dx,dy
      integer h(6)
      real x0(6),v0
      real xi(6)

      common /cisym/ ieqs,ists,mode
      common /cskip/ iskip,ltype

c     real qg
c     common /cfatm/ nasym(2),nnsym(60,2),qg(6,60,2)
      common /cnpln/ npln

      integer ind(2*ngm)
      real xy(3)
      real xk(6)
      real rx1(3),xy1(3)
      real q(6),q0(6),qd(6)
      real hr(6),ry(6)
      real getv0
      integer select
      integer natom(namax)
      real td(6)
      real teq(6),teq1(6),teq2(6)
      real prj
c     common /euler/ prj(3,3)
      real eps0
      common /ceps0/ eps0

      character(4) text
      real rxw(2),angl,hgt
      real rgbtbl(3,5)
      common /crgbtbl/ rgbtbl

      integer getien,getish,getisyd,getinum,getmtend,geticol,genpolyh0
      data eps/-0.0001/
      data td/6*0.0/
      data ind/2,1,4,3,6,5,8,7,10,9,12,11/

      call getnsymo(nsymo,icent)
      call getimd(imode)
      nf=0
      do i=1,na
         if(jaslct(i).ge.1) then
            nf=nf+1
         end if
      end do

      if(abs(mode).eq.2) then
         call setnfil(nf)
      end if

      npln=0
      no=0
      call zerovi(natom,na)
      call zerov(xk,ng)

c     call getqe(xk,q)    ! undistorted xk to undistorted q
c     call getqed(xk,q)    ! undistorted xk to distorted q
c     call trcs(q,xy)

      call getimd(imode)
      ih=1
      ik=1

      write(6,*) 'na in getatm=',na

      do n=1,na
c     call rescale
         if(imode.eq.1) then
            ien=getien(n)
c     write(6,*) 'ien=',ien
            call setrgb(rgbtbl(1,ien))
         end if
         if(jaslct(n).ge.1) then 
            call setshadtyp(n)  ! set shading type
            call getshd(ishd)
            mtend=getmtend(n)   ! number of equivalent sites
            if(abs(mode).eq.2) then
               npln=0
            end if
            icol=geticol(n,jaslct(n))
c           write(6,*) 'mtend=',mtend
            do nsd=1,mtend       ! number of equivalent sites
               call getxi(n,xi)  ! center of small OD
               call sclvctr(xi,getv0(n),6)   ! xi times v
               call vcpyr(xi,xiv,6)
               if(mode.ge.0) then ! use no or all symmetry operator
                  nsyd=isk(nsd,n)
               else              ! use specified symmetry operator
                  nsyd=nr(nsd,n)
               end if
               call rednsic(nsyd,nsy,icnt)
               call getrx1(xiv,ry,td,nsy,icnt)
               call getqedrs(ry,teq1) ! reshaped teq1 (q space vector)
               call getqed(tt(1,nsd,n),teq2) ! tt is not reshaped
               call vadd(teq1,teq2,teq,6)    ! q space vector
c     call wtvctrrf('teq=',teq,6,'(6f12.5)')

c     call vcpyr(ry,requiv,6) ! equivalent site
c     call setteq(ry)   ! for ifatm0
               call setteq(teq)   ! for ifatm0
               call setneq(nsy,icnt)  ! for ifatm0

c     genpolyh0 : generate edge vectors of an occupation domain
               isydn=getisyd(n)
               ishn=getish(n)
c              write(6,*) 'ishn=',ishn
               if(ishn.ge.0) then
                  if(n.eq.na) call setend(1)
c     generate a small occupation domain
                  kskip=genpolyh0(n,ni(0,1,1,n),irs(0,n),icol,jaslct(n))
                                ! get small OD
               else
                  call quit('invalid ish')
               end if
            end do
c           if(abs(mode).eq.2) then
            if(abs(mode).eq.2.or.imode.eq.1) then
               call drwdom()    ! draw occupation domains
            end if
         end if
      end do

c     write(6,*) 'mode, imode in getatm=',mode,imode
      if(abs(mode).ne.2.and.imode.eq.0) then
          call drwdom()          ! draw occupation domains
      end if

      if(getinum().eq.1) then   ! write occupation domain number
         do n=1,na
            if(jaslct(n).ge.1) then
               call getxi(n,xi)
               do ij=1,6
                  xiv(ij)=xi(ij)*getv0(n)
               end do
               mtend=getmtend(n)
c              write(6,*) 'mode,nsymo,mtend=',mode,nsymo,mtend
               do nsd=1,mtend   ! number of equivalent sites
                  if(mode.ge.0) then
                     icnt=0
                     nsy=isk(nsd,n) ! symmetry operation
                     call getrx1(xiv,ry,td,nsy,icnt)
                  else if(mode.lt.0) then
                     nsyd=nr(nsd,n)  ! specified symmetry operations
                     call rednsic(nsyd,nsy,icnt)
                     call getrx1(xiv,ry,td,nsy,icnt)  ! just rotation for reshape
                  end if

                  call getqedrs(ry,teq1) ! reshaped teq1
                  call getqed(tt(1,nsd,n),teq2)  ! tt is not reshaped
c     call wtvctrrf('tt=',tt(1,nsd,n),6,'(6f10.5)')
c     call wtvctrrf('teq2=',teq2,6,'(6f10.5)')
                  call vadd(teq1,teq2,teq,6)  

                  call trcs(teq(4),xy1) ! get x,y,z depending on isys
c     call mv(prj,xy1,rx1) ! viewing transformation
                  call viewtrns(xy1,rx1) ! viewing transformation
                  call rescl(rx1,amag,3)
                  call getnumtxt(n,text,nchar)
                  angl=0
                  hgt=0.3
                  rxw(1)=rx1(1) ! double to single
                  rxw(2)=rx1(2)
                  call csymbol(rxw(1),rxw(2),hgt,text,angl,nchar)
               end do
            end if
         end do
      end if
      return
      end

      subroutine getnumtxt(n,text,nchar)  ! get string for integer n
      character(*) text
      if(n.le.9) then
         nchar=1
         write(text,'(i1)') n
      else if(n.le.99) then
         nchar=2
         write(text,'(i2)') n
      else if(n.le.999) then
         nchar=3
         write(text,'(i3)') n
      end if
      return
      end

      subroutine setshd(i)
      common /cstshd/ ishd
      ishd=i
      return
      end

      subroutine getshd(i)
      common /cstshd/ ishd
      i=ishd
      return
      end


      integer function getinum()
      real shiftx,shifty
      common /cdevi/ ldev,linw,klin,jcol,inum
      common /cdevr/ shiftx,shifty
      getinum=inum
      return
      end


      real function dstnc(nrm,x)
      real nrm(*),x(*)
      dstnc=vinp(nrm,x,3)
      return
      end

      subroutine getrxyz(e1,e2,e3,rx,ry,rz)
      real e1(*),e2(*),e3(*),rx(*),ry(*),rz(*)
      rx(1)=e1(1)
      rx(2)=e2(1)
      rx(3)=e3(1)
      ry(1)=e1(2)
      ry(2)=e2(2)
      ry(3)=e3(2)
      rz(1)=e1(3)
      rz(2)=e2(3)
      rz(3)=e3(3)
      return
      end

      subroutine isskp(x,isk,i)
      include 'impr8.inc'
      real x(*),rx(6),rxs(6,120)
      integer isk(20,*)
      call getnsymo(nsymo,icent)
      n=1
      isk(i,1)=0
      call vrcpy(x(1),rxs(1,n),6)
      do k=0,icent
         do j=2,nsymo
            isk(i,j)=0
            call getrx(x,rx,j,k)
            inew=newatm(n,rx,rxs)
            if(inew.eq.0) isk(i,j)=1
         end do
      end do
c     write(6,'(80i1)') (isk(i,j),j=1,nsymo)
      return
      end
   
      integer function newatm(n,rx,rxs)
      include 'impr8.inc'
      real rx(6),rxs(6,*)
c***    zero translation vector is assumed
      real eps
      data eps/0.0001/
      do i=1,n
         do j=1,6
            if(abs(rx(j)-rxs(j,i)).ge.eps) go to 100
         end do
         newatm=0
         return
 100     continue
      end do
      n=n+1
      do j=1,6
         rxs(j,n)=rx(j)
      end do
      newatm=1
      return
      end

      integer function geticl(ms,ne)
      real ms(*)
      do i=1,ne
        if(ms(i).gt.0) then
            geticl=i
            return
        end if
      end do
      geticl=0
      return
      end

      subroutine trcs(x,y)
      real x(3),y(3)
      real n1,n2,n3
      common /clmt2/ n1(6),n2(6),n3(6)
c*****transformation of coordinate systems********************
c     x: coordinates in primary system
c     y: coordinates in plotter system
c        y(1),y(2) are x and y coordinates in plotter system
      y(1)= (n1(4)*x(1)+n1(5)*x(2)+n1(6)*x(3))
      y(2)= (n2(4)*x(1)+n2(5)*x(2)+n2(6)*x(3))
      y(3)= (n3(4)*x(1)+n3(5)*x(2)+n3(6)*x(3))
      return
      end

      subroutine reduc(ry)
      real ry(*)
      do i=1,6
         one=1.d0
         ry(i)=mod(ry(i),one)
         rd=ry(i)
         if(rd.lt.-0.499) then
            ry(i)=ry(i)+1
         else if(rd.gt.0.501) then
            ry(i)=ry(i)-1
         end if
      end do
      return
      end

      subroutine setend(i)
      common /cend/ isave
      isave=i
      return
      end

      subroutine getend(i)
      common /cend/ isave
      i=isave
      return
      end

      integer function iand(i,j)
      iand=mod(i,j*2)
      return
      end

      integer function ior(i,j)
      ior=i+j
      return
      end

      subroutine quit(str)
      character(*) str
      write(6,*) str
      close(unit=3)
      close(unit=4)
      stop
      end

      integer function getisys()
      call getsys(isysd)
      getisys=isysd
      return
      end

      subroutine getsys(isysd)
      real d0,d1,d2,d3
      common /cgethk/ d0(6,6),d1(6,6),d2(6,6),d3(6,6),isys
      isysd=isys
      return
      end            

      subroutine setsys(isysd)
      real d0,d1,d2,d3
      common /cgethk/ d0(6,6),d1(6,6),d2(6,6),d3(6,6),isys
      isys=isysd
      return
      end            

      subroutine sitetr(na,x0,irs0,adtrn)
      include 'impr8.inc'
c     additional translations for site symmetry operations
      integer irs0(0:120,*)
      real x0(6,*),adtrn(6,120,*),ry(6)
      integer rcomp1

      call getnsymo(nsymo,icent)
      do i=1,na
         do j=1,irs0(0,i)
            if(irs0(j,i).le.nsymo) then
               jd=irs0(j,i)
               icnt=0
            else
               jd=irs0(j,i)-nsymo
               icnt=1
            end if
            call getrx(x0(1,i),ry,jd,icnt)
            idummy=rcomp1(x0(1,i),ry,adtrn(1,j,i))
         end do
      end do
      return
      end

      integer function rcomp1(x,rx,adtrn)
      include 'impr8.inc'
      real x(*),rx(*),adtrn(*)
      common /cskip/ iskip,ltype
      real trc(6,32),trcd(6)
      real eps
      data eps/0.00001/
c     ltype=1,2,3,4 mean p,i,f,s in icosahedral lattice
      call gettrc(ltype,trc,ntrc)
      rcomp1=0
      do j=1,ntrc
         call vcpyr(trc(1,j),trcd,6)
         do i=1,6
            xx=abs(x(i)-rx(i)-trc(i,j))+eps/2
            one=1.d0
            xx=mod(xx,one)
            if(xx.gt.1-eps) then
               xx=abs(xx-1)
               trcd(i)=trcd(i)+1
            else if(xx.lt.-1+eps) then
               xx=abs(xx+1)
               trcd(i)=trcd(i)-1
            end if
            if(xx.gt.eps) go to 110
         end do
         rcomp1=1
         call vcpyr(trcd,adtrn,6)
         write(6,*) 'adtrn=',(adtrn(k),k=1,6)
         return
 110     continue
      end do
      call quit('exited in rcomp1')
      return
      end

      subroutine setamgn(bmgn)
      real bmgn
      real amgn
      common /camgn/ amgn
      amgn=bmgn
      return
      end

      subroutine setitst(itstd)
      common /ctst/ itst, nmax, iout
      itst=itstd
      return
      end

      subroutine rdhkl(hk,mpl,q,fcal,nh)
      integer hk(6,*),mpl(*)
      real q(*),fcal(*)
      nh=0
 100  continue
      nh=nh+1
      read(1,'(6i2,i5,f12.6)',end=200)
     *     (hk(i,nh),i=1,6),mpl(nh),q(nh)
      go to 100
 200  continue
      nh=nh-1
      return
      end

      subroutine setimd(jmode)
      common /cimode/ imode
      imode=jmode
      return
      end

      subroutine getimd(jmode)
      common /cimode/ imode
      jmode=imode
      return
      end


      subroutine setneq(nsy,icnt)
      common /cifatm/ nequiv,icnteq
      nequiv=nsy
      icnteq=icnt
      return
      end

      subroutine getneq(nsy,icnt)
      common /cifatm/ nequiv,icnteq
      nsy=nequiv
      icnt=icnteq
      return
      end
      
      subroutine setbrv(brv)
      character brv0,brv
      common /cbrv/ brv0
      brv0=brv
      return
      end

      subroutine getbrv(brv)
      character brv0,brv
      common /cbrv/ brv0
      brv=brv0
      return
      end

      subroutine geta(a1,a2)
      real al,am,cl,eps1,eps2,eps3,eps4
      common /cgetq2/ al,am,cl,eps1,eps2,eps3,eps4
      a1=al
      a2=am
      return
      end

      subroutine getc(c)
      real al,am,cl,eps1,eps2,eps3,eps4
      common /cgetq2/ al,am,cl,eps1,eps2,eps3,eps4
      c=cl
      return
      end

      subroutine getdet(det0)
      real det
      common /cgetd/ det
      det0=det
      return
      end

      subroutine openfile(iun,flnm,st)
      integer iun
      character(*) flnm,st

      lflnm=istrln(flnm)
      lst=istrln(st)
      open (unit=iun,file=flnm(1:lflnm),status=st(1:lst),err=910)
      go to 810
 910  write(6,*) 'can''t open ',flnm(1:lflnm)
      call exit(1)
 810  continue
      return
      end
